﻿/* Developer:       Prasoon
 * Code Reviewer:   Stanley
 * 
*/
#region "References"
// Web References
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Configuration;
using System.Drawing;
using System.Web.UI.HtmlControls;
using System.Data;
using System.Data.SqlClient;
using System.Text;
using System.IO;
using System.Net;                       // do we need it?
using System.Security.Cryptography;     // do we need it?
using System.Drawing.Printing;          // do we need it? if not, should drop those for lighter

// MSI References
using MSISalesOrder;
using SecurityPassport;
using AjaxControlToolkit;
using SalesOrderCreationQuote;
using Infragistics.Web.UI.GridControls;
using Infragistics.Web.UI.ListControls;
#endregion

namespace SalesOrderCreationQuote
{
    public partial class SOEntry : System.Web.UI.Page
    {

        #region Parameter Property
        /// <summary>
        /// Identifies the company in which this SO will be created.
        /// </summary>
        private string strCompanyID = "MSI";
        /// <summary>
        /// The UserID of the currently logged on User. Comes from Team MSI Cookies.
        /// </summary>
        private string UserID = string.Empty;
        string login = string.Empty;
        #endregion

        #region Page PreInit
        /// <summary>
        /// Page Pre INIT
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        override protected void OnInit(EventArgs e)
        {
            //if (ViewState["SalesOrder"] != null)
            //{
            //    SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            //    webgvSOLineItem.DataSource = SOE.LineItems;
            //    webgvSOLineItem.DataBind();
            //}
        }
        #endregion

        #region "Page_Load"
        /// <summary>
        /// Page_Load: Login UserID Handling with nouser/security access.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            //adding virtual scrolling in look up grid
            wdgLookup.Behaviors.VirtualScrolling.RowsRequested += new RowsRequestedHandler(VirtualScrolling_RowsRequested);

            #region "UserID/LoginName Handling"
            try
            {
                // Displaying the current User ID
                this.lblLogin.Text = getUserID();
                // if lblLogin is blank or nothing return from GetUserID, please route to noUser.html.
                // if user did not login, it should route to "http://team.msistone.com/nouser.html"
                lblDebug.Text = string.Empty;
            }
            catch (Exception ex)
            {
                // since userid is blank, it will route to 'nouser' page to notify user to login instead of 'noaccess' page.
                Response.Write(ex.Message);
                Response.Redirect("http://team.msistone.com/nouser.html", true);
            }
            #endregion

            //checking authorization
            SecurityPassport.SecurityPassport SP = new SecurityPassport.SecurityPassport("MAS500ERP", UserID);
            string UserAccessType = string.Empty;

            try
            {
                if (SP.Screens["SOCreationOnWeb"].SecureObjects["EntirePage"].Rights.Count == 0)
                {
                    Response.Redirect("http://team.msistone.com/noaccess.html", true);
                }

            }
            catch (Exception exe)
            {
                Response.Redirect("http://team.msistone.com/noaccess.html", true);
            }

            //checking SO radio button is checked or not
            if (rbSO.Checked)
            {
                imgQuoteToSO.Visible = false;
                imgQuoteToSO.Attributes.Add("Visibility", "hidden");
            }
            else
            {
                imgQuoteToSO.Visible = true;
            }


            #region "CompanyID Switch"
            // in production, we will use MSI Company Switch. Need to consult with Sharan.
            // strCompanyID = Request.Cookies[MSISwitch.CompanyIDCookieName] == null ? "MSI" : Request.Cookies[MSISwitch.CompanyIDCookieName].Value;
            // Testing / HardCoded
            strCompanyID = "MSI";
            hdfCompanyID.Value = "MSI";
            #endregion

            #region "PostBack Works"
            if (!Page.IsPostBack)
            {
                PageDefault();
                ViewState["SalesOrder"] = null;
                #region "Auto Complete Context Keys"
                // context key for auto complete for customer
                aceCustomer.ContextKey = strCompanyID + "-" + getUserID();
                //context key for auto complete for Sales Order

                aceSalesOrder.ContextKey = strCompanyID + "-" + this.lblLogin.Text + "-" + hdfSalesQuote.Value;
                autoCommPlan.ContextKey = strCompanyID;
                #endregion

                //Assigning current date
                txtrequest.Text = System.DateTime.Now.ToShortDateString();
                txtSchedShip.Text = System.DateTime.Now.ToShortDateString();
                txtPromised.Text = System.DateTime.Now.ToShortDateString();

                // retreiving SO Source from DB
                DataTable dt = SalesOrder.GetSOSource(strCompanyID);
                DataRow dr = dt.NewRow();
                dr[0] = "";
                dr[1] = "0";
                dt.Rows.InsertAt(dr, 0);
                ddlSource.DataSource = dt;
                ddlSource.DataTextField = "SalesSourceID";
                ddlSource.DataValueField = "SalesSourceKey";
                ddlSource.DataBind();



            }


            #endregion

            #region "Closing Browser(X) : Code"
            // Stanley: ?? -> we should have some documentation about what it does.
            if (hdClose.Value.ToString().Equals("PageClosed"))
            {
                hdClose.Value = string.Empty;
                //unlocking the SO
                UnlockSO_Edit();


            }
            else if (hdClose.Value.ToString().Equals("Finish"))
            {
                //Finish the changes and unlocking the SO
                FinishSO();
            }

            //else if (hdClose.Value.ToString().Equals("PageSaved"))
            //{
            //    hdClose.Value = string.Empty;
            //    SaveSO();
            //}
            #endregion


            //Sales order text box focus
            txtSalesOrder.Focus();
            //making all the textboxes read only
            //txtrequest.Attributes.Add("readonly", "readonly");
            //txtSchedShip.Attributes.Add("readonly", "readonly");
            //txtPromised.Attributes.Add("readonly", "readonly");

            //after cell exited event this condition executs to make changes in grid
            if (hdClose.Value.ToString().Equals("changerecord"))
            {
                hdClose.Value = "0";
               ChangeItem();
            }


        }
        

        // end "Page_Load"
        #endregion

        #region "SaveSO Method"
        /// <summary>
        /// Saving SO, Everything from SOHeader, its Comment, each SOLine and its Comments, Final Wrapping
        /// Called by btnSave_Click, btnFinish_Click + other areas ( such as force closing, session expired popup screen )
        /// </summary>
        private void SaveSO()
        {
            /*
             * Saving SO Protocol, by Stanley
             * 1. if SO is changed, calling SO Change Module ( refer to Kush's SO UI Rules, Sheet 'ChangeOrder Rules' )
             * 2. Saving SOHeader
             * 3. Saving SOHeader Comment, if any. 
             * 4. For each SOLine, saving SOLine in which, its SOLine Comment, if any.
             * 5. Final SO Saving/Wrapping. ( in which, SOHeader's various Amt and AmtHC + Each SOline's STax and Summary back to SOHeader and SOLog's Amt Handling. )
             */
            try
            {
                SalesOrder soFinal = (SalesOrder)ViewState["SalesOrder"];
                soFinal.CustPONo = txtCustomerPO.Text.Trim();
                if (chkHold.Checked)
                {
                    soFinal.HoldSO(true, txtHold.Text.Trim());
                }
                else
                {
                    soFinal.HoldSO(false, txtHold.Text.Trim());         // Stan_Note: if False, Hold Reason should wipe out/ empty string.
                }
                soFinal.SalesSourceKey = Convert.ToInt32(ddlSource.SelectedValue);
                soFinal.ChangeLinesDate(Convert.ToDateTime(txtrequest.Text), Convert.ToDateTime(txtPromised.Text), Convert.ToDateTime(txtSchedShip.Text));
                // soFinal.ChangeShipDate(Convert.ToDateTime(txtSchedShip.Text));
                // soFinal.ChangePromiseDate(Convert.ToDateTime(txtPromised.Text));


                //Calling Save Sales Order
                int result = soFinal.SaveSalesOrder(UserID, Session["SessionKey"].ToString(), strCompanyID);
                if (result == 3)
                {
                    lblDebug.Text = "Error in final saving.";
                    lblDebug.Visible = true;
                    BindSoLIneItem(soFinal.LineItems);
                
                }else
                if (result == 2)
                {
                    lblDebug.Text = "Session Expires.Please reopen the SalesOrder.";
                    lblDebug.Visible = true;
                }
                else
                    if (result == 0)
                    {
                        // Without creating aline if user submits then this condtion executes
                        lblDebug.Text = "Item or Item Description Required.";
                        lblDebug.Visible = true;
                        BindSoLIneItem(soFinal.LineItems);
                    }
                    else
                    {
                        //Get Saved SOInfo from DB
                        if (ViewState["finish"] == null)
                        {
                            SalesOrder soFromDB = new SalesOrder(soFinal.SOKey, getUserID(), txtSalesOrder.Text.ToString().Trim(), hdfSalesQuote.Value.Trim(), strCompanyID, Session["SessionKey"].ToString());
                            ViewState["SalesOrder"] = soFromDB;
                            
                            LoadSOInfo(soFromDB);  // if this one is retrieving saved SO Data back to SO Objects, it should not be in here (Stanley)

                            if (result == 1)
                            {
                                ViewState["result"] = result;
                                lblDebug.Text = "Saved Sucessfully";
                                lblDebug.Visible = true;
                                imgBtnCustmer.Enabled = false;
                                hfCheckEdit.Value = "1";
                               // imgbtnPSPrint.Enabled = true;
                               // imgbtnSOPrint.Enabled = true;
                                imgSOLookUp.Enabled = false;
                            }

                            txtCustomer.Enabled = false;
                            upTop.Update();
                            upsoheader.Update();
                            upSOLine.Update();
                        }
                        else
                        {
                            //releasing the SO after clicking on finish button
                            soFinal.ReleaseLock(Session["SessionKey"].ToString());
                            Response.Redirect(Request.RawUrl);
                        }
                    }

            }
            catch (Exception exe)
            {
                // Stanley: ?? what will do, if there is an error at any steps of SO Saving.
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;
            }
        }

        /// <summary>
        /// Finals the saving.
        /// </summary>
        private void FinalSaving()
        {
            //SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            //int result = SOE.FinalSaving();
            //if (result == 1)
            //{
            //    lblDebug.Text = "Saved Successfully";
            //    lblDebug.Visible = true;
            //}
        }   // end "SaveSO()"
        #endregion

        #region "UnlockSO_Edit Method"
        /// <summary>
        /// UnLock SO Edit Method.
        /// Note: Stanley Do not Understand, what it does.
        /// </summary>
        private void UnlockSO_Edit()
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            // releasing the so 
            SOE.ReleaseLock(Session["SessionKey"].ToString());

        }   // end "UnlockSO_Edit"        
        #endregion

        #region  "Binding Items"
        /// <summary>
        /// Item Autofill in SOLINE GRIDVIEW
        /// Stan_Note: Can we have what each function does?
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void wdgItemID_ItemsRequested(object sender, Infragistics.Web.UI.ListControls.DropDownItemsRequestedEventArgs e)
        {
            AutoFill oAutoFill = new AutoFill();
            //oAutoFill.GetItemNumberAutoFill(e.Value.ToString());
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

            //WDG_DropDownProviderItemID.EditorControl.Items.Clear();
            WDG_DropDownProviderItemID.EditorControl.EnableAjaxViewState = true;
            WDG_DropDownProviderItemID.EditorControl.EnableViewState = true;
            WDG_DropDownProviderItemID.EditorControl.DataSource = null;
            WDG_DropDownProviderItemID.EditorControl.DataBind();
            upSOLine.Update();
            if (e.Value != null || (e.Value.ToString() != string.Empty))
            {
                WDG_DropDownProviderItemID.EditorControl.Items.Clear();
                //Passing the item text and getting the corresponding data
                DataTable dt = oAutoFill.GetItemNumberAutoFill(e.Value.ToString(), strCompanyID, SOE.DfltWhseKey);
                try
                {
                    if (dt.Rows.Count > 0)
                    {
                        //binding data
                        WDG_DropDownProviderItemID.EditorControl.DataSource = dt;
                        WDG_DropDownProviderItemID.EditorControl.TextField = "ItemID";
                        WDG_DropDownProviderItemID.EditorControl.ValueField = "ItemID";
                        WDG_DropDownProviderItemID.EditorControl.DataBind();
                    }
                    else
                    {
                        WDG_DropDownProviderItemID.EditorControl.Items.Clear();
                        WDG_DropDownProviderItemID.EditorControl.DataSource = null;
                        WDG_DropDownProviderItemID.EditorControl.DataBind();
                    }
                }
                catch (Exception exe)
                {
                    lblDebug.Text = exe.Message;
                    lblDebug.Visible = true;
                }
            }
        }   // end "wdgItemID_ItemsRequested"

        /// <summary>
        /// UOM Autofill
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Infragistics.Web.UI.ListControls.DropDownItemsRequestedEventArgs"/> instance containing the event data.</param>
        protected void wdgUOM_ItemsRequested(object sender, Infragistics.Web.UI.ListControls.DropDownItemsRequestedEventArgs e)
        {
            AutoFill oAutoFill = new AutoFill();

            ddp_UnitMeasID.EditorControl.EnableAjaxViewState = true;
            ddp_UnitMeasID.EditorControl.EnableViewState = true;
            ddp_UnitMeasID.EditorControl.DataSource = null;
            ddp_UnitMeasID.EditorControl.DataBind();
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            //getting itemkey
            int itemKey = SOE.LineItems[Convert.ToInt32(hdfRowindexUOM.Value)].ItemKey;
            upSOLine.Update();

            if (e.Value != null || (e.Value.ToString() != string.Empty))
            {
                ddp_UnitMeasID.EditorControl.Items.Clear();
                //you can bind to any data here and the changes will be reflected on the client side
                MSISalesOrderAPI MSISOE = new MSISalesOrderAPI();
                DataTable dtUOM = MSISOE.GetLookupList("", "", MSISalesOrderAPI.LookupType.UnitOfMeasurement, "", itemKey);

                ddp_UnitMeasID.EditorControl.DataSource = dtUOM;

                ddp_UnitMeasID.EditorControl.TextField = "UnitMeasID";
                ddp_UnitMeasID.EditorControl.ValueField = "UnitMeasKey";
                ddp_UnitMeasID.EditorControl.DataBind();
            }
        }   // end "wdgItemID_ItemsRequested"

        // Stan_Note: Item's UOM can't change ( 1. SOLine is closed, 2. even open SOLine -> If SOLine is not deletable ( means it has some activities ), then, it can't change at all.
        /// <summary>
        /// Handles the SelectionChanged event of the wdgUOM control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Infragistics.Web.UI.ListControls.DropDownSelectionChangedEventArgs"/> instance containing the event data.</param>
        protected void wdgUOM_SelectionChanged(object sender, Infragistics.Web.UI.ListControls.DropDownSelectionChangedEventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
           
            if (e.NewSelection.ToString() != null)
            {
                try
                {
                    string UOMkey = ddp_UnitMeasID.EditorControl.SelectedValue;
                    hidUOM.Value = UOMkey;
                    ViewState["UOMKey"] = UOMkey;
                  
                }
                catch (Exception exe)
                {
                    
                    BindSoLIneItem(SOE.LineItems);
                }
            }
            //SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            //if (e.NewSelection.ToString() != null)
            //{
            //    try
            //    {
            //        string UOMkey = ddp_UnitMeasID.EditorControl.SelectedValue;
            //        hidUOM.Value = UOMkey;
            //        ViewState["UOMKey"] = UOMkey;
            //        ddp_UnitMeasID.EditorControl.EnableAjaxViewState = true;
            //          ddp_UnitMeasID.EditorControl.EnableViewState = true;
            //    if (e.NewSelection.ToString() != null)
            //    {
            //        int itemKey = SOE.LineItems[Convert.ToInt32(hdfRowindexUOM.Value)].ItemKey;
            //        ddp_UnitMeasID.EditorControl.CurrentValue = ddp_UnitMeasID.EditorControl.SelectedItem.Text;
            //        MSISalesOrderAPI MSISOE = new MSISalesOrderAPI();
            //        DataTable dtUOM = MSISOE.GetLookupList("", "", MSISalesOrderAPI.LookupType.UnitOfMeasurement, "", itemKey);

            //        ddp_UnitMeasID.EditorControl.DataSource = dtUOM;

            //        ddp_UnitMeasID.EditorControl.TextField = "UnitMeasID";
            //        ddp_UnitMeasID.EditorControl.ValueField = "UnitMeasKey";
            //        ddp_UnitMeasID.EditorControl.DataBind();
            //    }
                    
            //    }
            //    catch (Exception exe)
            //    {
            //        //BindSoLIneItem(SOE.LineItems);
            //    }
            //}
        }
        // Stan_Note: what I do know is.. @ 'existing "open" SOLine', if SOline is not deletable, then, ItemID can't change 'at all'
        // Close SOLine can't change anything.
        /// <summary>
        /// Handles the SelectionChanged event of the wdgItemID control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Infragistics.Web.UI.ListControls.DropDownSelectionChangedEventArgs"/> instance containing the event data.</param>
        protected void wdgItemID_SelectionChanged(object sender, Infragistics.Web.UI.ListControls.DropDownSelectionChangedEventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            try
            {
                WDG_DropDownProviderItemID.EditorControl.EnableAjaxViewState = true;
                WDG_DropDownProviderItemID.EditorControl.EnableViewState = true;
                if (e.NewSelection.ToString() != null)
                {
                    //string itemkey = WDG_DropDownProviderItemID.EditorControl.SelectedValue;
                    // hfItemKey.Value = itemkey;

                }
            }
            catch (Exception Exe)
            {
                BindSoLIneItem(SOE.LineItems);
            }
        }
        #endregion

        #region "Binding Whse"
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void ddp_Whse_ItemsRequested(object sender, Infragistics.Web.UI.ListControls.DropDownItemsRequestedEventArgs e)
        {

        }   // end "ddp_Whse_ItemsRequested"
        #endregion

        #region "Get UserID/LoginID Method"
        /// <summary>
        /// Get Login UserID
        /// </summary>
        /// <returns></returns>
        protected string getUserID()
        {
#if DEBUG

            UserID = "SALEEM";

#else
            UserID = Request.Cookies["teammsiuid"].Value;
#endif

            return UserID;
        }   // end "getUserID"
        #endregion

        // Have question about logic.
        // Usage: Existing SO Retrieving, if not existing SO#, take as Assigned SO.
        // Correction: Only SO Retrieving happened, here.

        #region "SalesOrder Text Box Text Changed Event"
        /// <summary>
        /// Handles the OnTextChanged event of the txtSalesOrder control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void txtSalesOrder_OnTextChanged(Object sender, System.EventArgs e)
        {
            // Stanley: What if, they just type '735540'. Will it concatenate with 'leading pending zero' to varchar(10) in here?
            // how to distinguish between SO retrieve and Assigned SO# Creation?
            try
            {

                CheckInitailsConditionsForSO();
                upsoheader.Update();
                upSOLine.Update();
                upTop.Update();
                upBillTo.Update();
                udpShipTo.Update();


            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;    // printing the exception
                lblDebug.Visible = true;
                upTop.Update();
                //ScriptManager.RegisterStartupScript(this.upNewSalesOrder, this.upNewSalesOrder.GetType(), "", "alert('" + exe.Message + "');", true);
                /*
                 * What kind of Exception message will show in here.
                 */
            }
        }

        //private string CheckValidUser()
        //{

        //   return SalesOrder.CheckValidUser(getUserID());
        //}   // end "txtSalesOrder_OnTextChanged"  
        #endregion

        // Usage: ?
        #region "CheckInitailsConditionsForSO: Checking Initials Condition For SO"
        /// <summary>
        /// Checks the initails conditions for SO.
        /// </summary>
        private void CheckInitailsConditionsForSO()
        {
            try
            {
                SalesOrder SOE = null;
                /*
                   1. If sokey is null in hidden field then follwing condition arise
                 * a)Either user typed very fast
                 * b) Or he is creating user defined sales order
                 */
                if (hidSOKey.Value == string.Empty) //if hidden field is null
                {
                    // Here i am retreiving SO based on tran no
                    SOE = new SalesOrder(txtSalesOrder.Text.Trim(), strCompanyID, hdfSalesQuote.Value, getUserID(), Session["SessionKey"].ToString());
                    //if SO not found then that SO's SOKEY is null
                    if (SOE.SOKey == 0)// if sokey is null or 0 then user wants to create User defined SO
                    {
                        //This is the code for creating user defined SO
                        #region Creating User define SO
                        SOE.CreateNewSOWithTranNo(txtSalesOrder.Text.Trim(), strCompanyID, hdfSalesQuote.Value, getUserID(), Session["SessionKey"].ToString());
                        ViewState["SalesOrder"] = SOE;
                        // CreateUserDefineSO();
                        txtSalesOrder.Text = SOE.TranNo;
                        txtOrderDate.Text = System.DateTime.Now.ToShortDateString();
                        ValueEnableDisable();
                        txtSalesOrder.Enabled = false;
                        txtCustomer.Focus();
                        btnCreate.Enabled = false;
                        imgBtnCustmer.Enabled = true;
                        txtChngOrderNo.Text = "0";
                        #endregion
                    }
                    else
                    {
                        //If sokey is not null then it has SO Record
                        #region Retreiving Sales Order
                        ViewState["SalesOrder"] = SOE;
                        //Temporary checking
                        txtSalesOrder.Text = SOE.TranNo;
                        LoadSOInfo(SOE);
                        DisableAllControls();

                        if (SOE.ReadOnlyAccess)
                        {
                            ContextMenu.Visible = false;
                            imgbtnEdit.Visible = false;
                            this.webgvSOLineItem.Columns["UnitPrice"].Hidden = true;
                            this.webgvSOLineItem.Columns["ExtAmt"].Hidden = true;
                            txtOrderTotal.Visible = false;
                        }
                        else
                        {

                            ContextMenu.Visible = false;
                            imgbtnEdit.Visible = true;
                            imgbtnEdit.Enabled = true;
                            this.webgvSOLineItem.Columns["UnitPrice"].Hidden = false;
                            this.webgvSOLineItem.Columns["ExtAmt"].Hidden = false;
                            txtOrderTotal.Visible = true;

                        }
                        upsoheader.Update();
                        upSOLine.Update();
                        upTop.Update();
                        upBillTo.Update();
                        udpShipTo.Update();
                        #endregion
                    }
                }
                else
                {
                    //If SO KEY is not null then 2 condition arise
                    /*
                       1. That order might be incomplete order.
                     * 2. Or it has full SO record
                     */
                    SOE = new SalesOrder(Convert.ToInt32(hidSOKey.Value), getUserID(), txtSalesOrder.Text.ToString().Trim(), hdfSalesQuote.Value, strCompanyID, Session["SessionKey"].ToString());
                    if (SOE.SOKey != 0 && SOE.CustKey == 0)// That SO has SOKEY but it has not customer. Means that SO is incomplete.
                    {
                        //Response.Write("<script language=Javascript>confirm((\"There is no Backout Plan attached.\");</script>");
                        ViewState["SalesOrder"] = SOE;
                        mpeIncompleteOrder.Show();
                    }
                    else
                    {
                        #region Retreiving Sales Order
                        ViewState["SalesOrder"] = SOE;
                        //Temporary checking
                        LoadSOInfo(SOE);
                        DisableAllControls();

                        if (SOE.ReadOnlyAccess)
                        {
                            ContextMenu.Visible = false;
                            imgbtnEdit.Visible = false;
                            this.webgvSOLineItem.Columns["UnitPrice"].Hidden = true;
                            this.webgvSOLineItem.Columns["ExtAmt"].Hidden = true;
                            txtOrderTotal.Visible = false;
                        }
                        else
                        {

                            ContextMenu.Visible = false;
                            imgbtnEdit.Visible = true;
                            imgbtnEdit.Enabled = true;
                            this.webgvSOLineItem.Columns["UnitPrice"].Hidden = false;
                            this.webgvSOLineItem.Columns["ExtAmt"].Hidden = false;
                            txtOrderTotal.Visible = true;
                        }
                        #endregion
                    }
                }

                gvSOHeaderComment.Enabled = false;
                btnCommSave.Enabled = false;
                btnHeaderCom.Enabled = true;
                upGrid.Update();
                upsoheader.Update();
                upTop.Update();


            }
            catch (Exception Exe)
            {

                lblDebug.Text = Exe.Message;
                lblDebug.Visible = true;
            }
        }
        // end "CheckCreditLimit"
        #endregion


        /// <summary>
        /// Handles the Click event of the btnIncompleteOrderOk control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnIncompleteOrderOk_Click(object sender, EventArgs e)
        {
            //enable diavle the values
            ValueEnableDisable();
            gvSOHeaderComment.Enabled = true;
            txtCustomer.Focus();
            btnCommSave.Enabled = true;
            this.txtSalesOrder.Enabled = false;
            //txtCustomer.Enabled = true;
            //txtCustomer.Focus();
            //btnHeaderCom.Enabled = false;
            upTop.Update();
            upsoheader.Update();
        }
        /// <summary>
        /// Handles the Click event of the btnIncompleteOrderCancel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnIncompleteOrderCancel_Click(object sender, EventArgs e)
        {
            //refreshing the page
            Response.Redirect(Request.RawUrl);
        }

        #region "btnSave_Click Event"
        /// <summary>
        /// btnSave_Click Event: will call Save_SO Method, and retrieve that Saved Data back to SO Object and UI.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnSave_Click(object sender, EventArgs e)
        {
            // This is wrong ( Stanley ). Changed Order Module should be in Save_SO Method.
            // Here is, called Save_SO, and Retrieve SO back to SO Objects and UI. That's all.
            SalesOrder oSO = (SalesOrder)ViewState["SalesOrder"];

            if (oSO != null)
            {
                //Checking SO radion button is checked
                if (rbSO.Checked)
                {
                    if (oSO.IsSOChang)//checking whther SO CHANGE or not
                    {

                        txtChangeNo.Text = (oSO.ChngOrdNo + 1).ToString();
                        txtChanreason.Text = "";
                        txtChanreason.Focus();
                        upTop.Update();
                        mpeChnageorder.Show();
                    }
                    else
                    {
                        //saving the SO
                        SaveSO();
                    }
                }
                else
                {
                    //saving the SO
                    SaveSO();
                }
            }
            else
            {   // Stan_Note: if oSO is null, are you saving?
                // SaveSO();
            }
        }   // end "btnSave_Click"
        #endregion

        #region Editing SO
        /// <summary>
        /// Editing SO depend upon access
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void imgbtnEdit_Click(object sender, EventArgs e)
        {
            //Making lookup and edit button visible false
            imgSOLookUp.Enabled = false;
            imgbtnEdit.Visible = false;

            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            //checking whether SO is already locked or not by calling this function
            CheckSOLock();

            if (hdfSalesQuote.Value == "840")
            {
                //Quote
                if (!SOE.ReadOnlyAccess)// if user has no read only access
                {
                    lblEditMode.Text = "This quote is currently in edit mode.";
                    hfCheckEdit.Value = "1";
                    btnHeaderCom.Enabled = true;
                    LoadSOInfo(SOE);
                    EnableLines();
                    ViewState["SalesOrder"] = SOE;
                }
                else
                {// for read only access this condition arise
                    lblDebug.Text = "You do not have access to edit SO.";
                    lblDebug.Visible = true;
                }

            }
            else
            {
                //SalesOrder
                if (!SOE.ReadOnlyAccess)
                {
                    if (SOE.EditableFlag)
                    {

                        lblEditMode.Text = "This sales order is currently in edit mode.";
                        hfCheckEdit.Value = "1";
                        btnHeaderCom.Enabled = true;
                        //loading complte SO
                        LoadSOInfo(SOE);
                        
                        //enacle all so line
                        EnableLines();
                        ViewState["SalesOrder"] = SOE;
                    }
                    else
                    {

                        //diabling all controls
                        DisableAllControls();
                    }
                }
                else
                {
                    lblDebug.Text = "You do not have access to edit SO.";
                    lblDebug.Visible = true;
                }
            }
            upsoheader.Update();
            upSOLine.Update();
        }
        #endregion

        #region "EnableLines: Enable all the columns as write only"
        /// <summary>
        /// EnableLines: ReadOnly On/Off for some column.
        /// </summary>
        private void EnableLines()
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

            grdLineComments.Enabled = true;
            btnCommSaveLine.Enabled = true;
            //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings[0].ReadOnly = false;
            //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings[1].ReadOnly = false;
            //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings[2].ReadOnly = false;
            //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings[3].ReadOnly = false;
            //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings[4].ReadOnly = true;
            //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings[5].ReadOnly = false;
            //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings[6].ReadOnly = true;
            //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings[7].ReadOnly = false;
            //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings[8].ReadOnly = false;
            //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings[10].ReadOnly = false;


            webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["ItemID"].ReadOnly = false;
            webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["Description"].ReadOnly = false;
            webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["QtyOrd"].ReadOnly = false;
            webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["UnitMeasID"].ReadOnly = false;
            webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["ExtAmt"].ReadOnly = true;
            webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["UnitPrice"].ReadOnly = false;
            webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["QtyShip"].ReadOnly = true;
            webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["TradeDiscAmt"].ReadOnly = false;
            webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["TradeDiscPct"].ReadOnly = false;

            webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["WhseID"].ReadOnly = true;
            webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["Status"].ReadOnly = false;


            //for (int i = 0; i <= SOE.LineItems.Count - 1; i++)
            //{
            //    LinkButton imgbtnLookupSOLINE = (LinkButton)webgvSOLineItem.Rows[SOE.LineItems.Count - 1].Items[1].FindControl("imgbtnLookupSOLINE");
            //    if (imgbtnLookupSOLINE != null)
            //    {
            //        imgbtnLookupSOLINE.Visible = true;
            //    }
            //}
            if (SOE.ReadOnlyAccess)
            {
                ContextMenu.Visible = false;
            }
            else
            {
                ContextMenu.Visible = true;
            }
            // upSOLine.Update();
        }   // end "EnableLines"
        #endregion

        #region "Check who locked SO_Edit"
        /// <summary>
        /// CheckSOLock
        /// </summary>
        private void CheckSOLock()
        {
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                string val;
                SOE.LockSOForEditing(getUserID(), Session["SessionKey"].ToString(), out val);// (SOE);
                lblDebug.Text = val;
                lblDebug.Visible = true;
                ViewState["SalesOrder"] = SOE;
                upTop.Update();
                upsoheader.Update();
                upSOLine.Update();

            }
            catch (Exception ex)
            {
                lblDebug.Text = ex.Message;
                lblDebug.Visible = true;
            }
        }   // end "CheckSOLock"
        #endregion

        #region ""
        /// <summary>
        /// Handles the SelectedIndexChanged event of the ddStatus control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs" /> instance containing the event data.</param>
        protected void ddStatus_SelectedIndexChanged(object sender, EventArgs e)
        //ClientScript.RegisterClientScriptBlock(GetType(), "Messssage", "<script language=JavaScript>alert('Quantity should be greater than zero.');</script>");
        {
            try
            {

                //For new order creation if user pressed cancel than what i have to do?
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                bool EditFlag = false;
                if (SOE != null)
                {
                    if (hdfSalesQuote.Value == "840")
                    {
                        //--
                        //Added by Vinay Gawli on 10/30/2013
                        try
                        {

                            SOE.CheckQuoteConvertedToSO(SOE.SOKey);

                            SOE.ChangeHeaderToLineStatus(Convert.ToInt32(ddStatus.SelectedValue));
                            BindSoLIneItem(SOE.LineItems);
                            upsoheader.Update();
                            upSOLine.Update();
                        }
                        catch (Exception ex)
                        {
                            btnSave.Enabled = false;
                            btnFinish.Enabled = false;
                            lblDebug.Text = "You may not change the status of a closed quote that has generated a sales order.";
                            lblDebug.Visible = true;
                            upTop.Update();
                        }
                        //--
                    }
                    else
                    {
                        SOE.ChangeHeaderToLineStatus(Convert.ToInt32(ddStatus.SelectedValue));
                        BindSoLIneItem(SOE.LineItems);
                        upsoheader.Update();
                        upSOLine.Update();
                        // end "if (SOE != null)"
                    }
                }
            }
            catch (Exception ex)
            {
                lblDebug.Text = ex.Message;
                lblDebug.Visible = true;
                upTop.Update();
            }

        }
        #endregion

        #region "upSOLine_Load"
        /// <summary>
        /// At SOLine Loading, if Status is = 3?
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void upSOLine_Load(object sender, EventArgs e)
        {
            if (ddStatus.SelectedValue == "3")
            {
                DisableAllControls();
            }
        }   // end "upSOLine_Load"
        #endregion

        #region "Disable All Controls"
        /// <summary>
        /// DisableAllControls: 
        /// </summary>
        private void DisableAllControls()
        {   // Stanley Changed from DisableAll to DisableAllControls
            try
            {
                #region Disabling Bill To Address controls
                try
                {
                    //Customer Cust = new Customer();



                    //Assigning cust name
                    txtBillToCustName.Enabled = false;
                    txtBillTo.Enabled = false;

                    txtBillToName.Enabled = false;
                    // Assigning Address line 
                    txtBillToAddr1.Enabled = false;
                    txtBillToAddr2.Enabled = false;
                    txtBillToAddr3.Enabled = false;
                    txtBillToAddr4.Enabled = false;
                    txtBillToAddr5.Enabled = false;
                    // Assigning Address Key

                    txtBillToAddrCity.Enabled = false;
                    txtBillToPostalCode.Enabled = false;
                    txtBillToCountry.Enabled = false;
                    // binding state From Function to drop down

                    txtBillToState.Enabled = false;

                }
                catch (Exception exe)
                {
                    lblDebug.Text = exe.Message;

                }
                #endregion

                txtPromised.Enabled = false;
                txtSchedShip.Enabled = false;
                txtrequest.Enabled = false;
                txtCommPlan.Enabled = false;
                chkHold.Enabled = false;
                txtHold.Text = string.Empty;
                txtSalesOrder.Enabled = false;
                this.btnHeaderCom.Enabled = false;
                this.btnSalesOrder.Enabled = false;
                this.txtSalesOrder.Enabled = false;
                this.btnCreate.Enabled = false;
                txtCustomer.Enabled = false;
                btnFinish.Enabled = false;
                ddStatus.Enabled = false;
                this.btnSave.Enabled = false;
                this.btnCancel.Enabled = true;


                this.txtCustomer.Enabled = false;
                txtShipTo.Enabled = false;
                this.txtOrderDate.Enabled = false;
                this.lblMessage.Visible = false;
                this.btnBillTo.Enabled = false;
                this.btnShipTo.Enabled = false;


                this.imgLookupShpTo.Enabled = false;
                this.btnLookupCommPlan.Enabled = false;
                imgQuoteToSO.Enabled = false;
                imgbtnCustClass.Enabled = false;
                imgbtnWareHouse.Enabled = false;
                imgbtnShipVia.Enabled = false;
                imgbtnCustClass.Enabled = false;
                imgLookupShpTo.Enabled = false;
                this.btnLookupCommPlan.Enabled = false;
                imgBtnCustmer.Enabled = false;
                imgbtnWareHouse.Enabled = false;
                //imgSOLookUp.Enabled = false; //Added By Vinay on 11/08/2013

                txtHeadShipVia.Enabled = false;
                txtHeadWarehouse.Enabled = false;
                txtCustomerPO.Enabled = false;

                txtCustomerPO.Enabled = false;
                txtShipTo.Enabled = false;
                ddStatus.Enabled = false;
                ddlSource.Enabled = false;
                txtHeadShipVia.Enabled = false;
                txtHeadWarehouse.Enabled = false;

                imgLookupShpTo.Enabled = false;
                this.btnLookupCommPlan.Enabled = false;
                btnHeaderCom.Enabled = false;
                webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["ItemID"].ReadOnly = true;
                webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["Description"].ReadOnly = true;
                webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["QtyOrd"].ReadOnly = true;
                webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["UnitMeasID"].ReadOnly = true;
                webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["ExtAmt"].ReadOnly = true;
                webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["UnitPrice"].ReadOnly = true;
                webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["QtyShip"].ReadOnly = true;
                webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["TradeDiscAmt"].ReadOnly = true;
                webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["TradeDiscPct"].ReadOnly = true;

                webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["WhseID"].ReadOnly = true;
                webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["Status"].ReadOnly = true;
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                for (int i = 0; i <= SOE.LineItems.Count - 1; i++)
                {
                    LinkButton imgbtnLookupSOLINE = (LinkButton)webgvSOLineItem.Rows[i].Items[1].FindControl("imgbtnLookupSOLINE");
                    if (imgbtnLookupSOLINE != null)
                    {
                        imgbtnLookupSOLINE.Visible = false;
                    }
                }
                upSOLine.Update();


                grdLineComments.Enabled = false;
                btnCommSaveLine.Enabled = false;
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;
            }
        }   // end "DisableAllControls"
        #endregion

        // Stan_Note: I think it is not done yet... UserDefined SO#
        #region "Creating User Define SO#"
        /// <summary>
        /// CreateUserDefineSO
        /// </summary>
        private void CreateUserDefineSO()
        {
            try
            {

            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
            }
        }   // end "CreateUserDefineSO"
        #endregion

        // All Address Works are here...
        #region "BillTo/ShipTo Address Works ( Click Event, Get Data, Saving/Cancel works, country and state ddl binding"

        // Usage: ShipTo Address Changes/Modification
        // Correction: Not Default AddrKey: It should be SOHeader's DfltShipToAddrKey
        // Naming: found only mdlBillToAddress, if one modle pop up is handling both BillTo and ShipTo, we should call it mdlAddressPopUp
        // Stan_Note: SO's ShipTo Address is changable and if change, there will be new addrKey / DfltShipToAddrKey at SOHeader and 
        // Apply to all open SOLD.ShipToAddrKey 
        #region "ShipTo Address Button Click Event"
        /// <summary>
        /// Handles the OnClick event of the btnShipTo control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnShipTo_OnClick(Object sender, EventArgs e)
        {
            try
            {

            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end of "btnShipTo_OnClick"
        #endregion

        /// <summary>
        /// Handles the OnClick event of the btnLookUpSearch control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnLookUpSearch_OnClick(Object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

            if (ViewState["text"].ToString().Equals("SOLookUP"))
            {
                #region "SOLookUP Look UP"
                MSISalesOrderAPI MSIAPIE = new MSISalesOrderAPI();
                DataSet dsValues = CreateDataset();

                if (!txtValue1_1.Text.ToString().Equals(string.Empty))
                {
                    dsValues.Tables[1].Rows.Add(drpfields1.SelectedItem.Value, drpcondition1.SelectedItem.Value, txtValue1_1.Text.Trim(), txtValue2_1.Text.Trim());
                }
                if (!txtValue1_2.Text.ToString().Equals(string.Empty))
                {
                    dsValues.Tables[1].Rows.Add(drpfields2.SelectedItem.Value, drpcondition2.SelectedItem.Value, txtValue1_2.Text.Trim(), txtValue2_2.Text.Trim());
                }
                if (!txtValue1_3.Text.ToString().Equals(string.Empty))
                {
                    dsValues.Tables[1].Rows.Add(drpfields3.SelectedItem.Value, drpcondition3.SelectedItem.Value, txtValue1_3.Text.Trim(), txtValue2_3.Text.Trim());
                }
                if (!txtValue1_4.Text.ToString().Equals(string.Empty))
                {
                    dsValues.Tables[1].Rows.Add(drpfields4.SelectedItem.Value, drpcondition4.SelectedItem.Value, txtValue1_4.Text.Trim(), txtValue2_4.Text.Trim());
                }
                if (dsValues.Tables[1].Rows.Count > 0)
                {
                    DataSet dsSOLookUP = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Orders, dsValues.Tables[1], getUserID());
                    if (dsSOLookUP.Tables.Count > 1)
                    {
                        wdgLookSO.Rows.Clear();
                        wdgLookSO.DataSource = dsSOLookUP.Tables[1];
                        wdgLookSO.DataBind();
                        ViewState["dsSOLookUP"] = dsSOLookUP.Tables[1];
                    }
                }
                mpeLookupShipTo.Show();
                #endregion
            }
            if (ViewState["text"].ToString().Equals("CommPlan"))
            {
                #region "Comm Plan Look UP"
                MSISalesOrderAPI MSIAPIE = new MSISalesOrderAPI();
                DataSet dsValues = CreateDataset();

                if (!txtValue1_1.Text.ToString().Equals(string.Empty))
                {
                    dsValues.Tables[1].Rows.Add(drpfields1.SelectedItem.Value, drpcondition1.SelectedItem.Value, txtValue1_1.Text.Trim(), txtValue2_1.Text.Trim());
                }
                if (!txtValue1_2.Text.ToString().Equals(string.Empty))
                {
                    dsValues.Tables[1].Rows.Add(drpfields2.SelectedItem.Value, drpcondition2.SelectedItem.Value, txtValue1_2.Text.Trim(), txtValue2_2.Text.Trim());
                }
                if (!txtValue1_3.Text.ToString().Equals(string.Empty))
                {
                    dsValues.Tables[1].Rows.Add(drpfields3.SelectedItem.Value, drpcondition3.SelectedItem.Value, txtValue1_3.Text.Trim(), txtValue2_3.Text.Trim());
                }
                if (!txtValue1_4.Text.ToString().Equals(string.Empty))
                {
                    dsValues.Tables[1].Rows.Add(drpfields4.SelectedItem.Value, drpcondition4.SelectedItem.Value, txtValue1_4.Text.Trim(), txtValue2_4.Text.Trim());
                }
                if (dsValues.Tables[1].Rows.Count > 0)
                {
                    DataSet dsCommPlan = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.CommissionPlan, dsValues.Tables[1]);
                    if (dsCommPlan.Tables.Count > 1)
                    {
                        wdgLookup.Rows.Clear();
                        wdgLookup.DataSource = dsCommPlan.Tables[1];
                        wdgLookup.DataBind();
                        ViewState["dsCommPlan"] = dsCommPlan.Tables[1];
                    }
                }
                mpeLookupShipTo.Show();
                #endregion
            }
            else
                if (ViewState["text"].ToString().Equals("WareHouse"))
                {
                    #region "WareHouse Look UP"
                    MSISalesOrderAPI MSIAPIE = new MSISalesOrderAPI();
                    DataSet dsValues = CreateDataset();
                    //dtValues.Columns.Add("FieldName", typeof(string));
                    //dtValues.Columns.Add("Operator",typeof(string));
                    //dtValues.Columns.Add("Value1", typeof(string));
                    //dtValues.Columns.Add("Value2", typeof(string));
                    if (!txtValue1_1.Text.ToString().Equals(string.Empty))
                    {
                        dsValues.Tables[1].Rows.Add(drpfields1.SelectedItem.Value, drpcondition1.SelectedItem.Value, txtValue1_1.Text.Trim(), txtValue2_1.Text.Trim());
                    }
                    if (!txtValue1_2.Text.ToString().Equals(string.Empty))
                    {
                        dsValues.Tables[1].Rows.Add(drpfields2.SelectedItem.Value, drpcondition2.SelectedItem.Value, txtValue1_2.Text.Trim(), txtValue2_2.Text.Trim());
                    }
                    if (!txtValue1_3.Text.ToString().Equals(string.Empty))
                    {
                        dsValues.Tables[1].Rows.Add(drpfields3.SelectedItem.Value, drpcondition3.SelectedItem.Value, txtValue1_3.Text.Trim(), txtValue2_3.Text.Trim());
                    }
                    if (!txtValue1_4.Text.ToString().Equals(string.Empty))
                    {
                        dsValues.Tables[1].Rows.Add(drpfields4.SelectedItem.Value, drpcondition4.SelectedItem.Value, txtValue1_4.Text.Trim(), txtValue2_4.Text.Trim());
                    }
                    if (dsValues.Tables[1].Rows.Count > 0)
                    {
                        DataSet dsWareHouse = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.WareHouseLookUP, dsValues.Tables[1]);
                        if (dsWareHouse.Tables.Count > 1)
                        {
                            wdgLookup.Rows.Clear();
                            wdgLookup.DataSource = dsWareHouse.Tables[1];
                            wdgLookup.DataBind();
                            ViewState["dsWareHouse"] = dsWareHouse.Tables[1];
                        }
                    }
                    mpeLookupShipTo.Show();
                    #endregion
                }
                else
                    if (ViewState["text"].ToString().Equals("ShipVia"))
                    {
                        #region "Ship Via Look UP"
                        MSISalesOrderAPI MSIAPIE = new MSISalesOrderAPI();
                        DataSet dsValues = CreateDataset();
                        //dtValues.Columns.Add("FieldName", typeof(string));
                        //dtValues.Columns.Add("Operator",typeof(string));
                        //dtValues.Columns.Add("Value1", typeof(string));
                        //dtValues.Columns.Add("Value2", typeof(string));
                        if (!txtValue1_1.Text.ToString().Equals(string.Empty))
                        {
                            dsValues.Tables[1].Rows.Add(drpfields1.SelectedItem.Value, drpcondition1.SelectedItem.Value, txtValue1_1.Text.Trim(), txtValue2_1.Text.Trim());
                        }
                        if (!txtValue1_2.Text.ToString().Equals(string.Empty))
                        {
                            dsValues.Tables[1].Rows.Add(drpfields2.SelectedItem.Value, drpcondition2.SelectedItem.Value, txtValue1_2.Text.Trim(), txtValue2_2.Text.Trim());
                        }
                        if (!txtValue1_3.Text.ToString().Equals(string.Empty))
                        {
                            dsValues.Tables[1].Rows.Add(drpfields3.SelectedItem.Value, drpcondition3.SelectedItem.Value, txtValue1_3.Text.Trim(), txtValue2_3.Text.Trim());
                        }
                        if (!txtValue1_4.Text.ToString().Equals(string.Empty))
                        {
                            dsValues.Tables[1].Rows.Add(drpfields4.SelectedItem.Value, drpcondition4.SelectedItem.Value, txtValue1_4.Text.Trim(), txtValue2_4.Text.Trim());
                        }
                        if (dsValues.Tables[1].Rows.Count > 0)
                        {
                            DataSet dsShipVia = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.ShipVia, dsValues.Tables[1]);
                            if (dsShipVia.Tables.Count > 1)
                            {
                                wdgLookup.Rows.Clear();
                                wdgLookup.DataSource = dsShipVia.Tables[1];
                                wdgLookup.DataBind();
                                ViewState["dsShipVia"] = dsShipVia.Tables[1];
                            }
                        }
                        mpeLookupShipTo.Show();
                        #endregion
                    }
                    else
                        if (ViewState["text"].ToString().Equals("HeaderComment"))
                        {
                            #region "Header Comments Look UP"
                            MSISalesOrderAPI MSIAPIE = new MSISalesOrderAPI();
                            DataSet dsValues = CreateDataset();
                            //dtValues.Columns.Add("FieldName", typeof(string));
                            //dtValues.Columns.Add("Operator",typeof(string));
                            //dtValues.Columns.Add("Value1", typeof(string));
                            //dtValues.Columns.Add("Value2", typeof(string));
                            if (!txtValue1_1.Text.ToString().Equals(string.Empty))
                            {
                                dsValues.Tables[1].Rows.Add(drpfields1.SelectedItem.Value, drpcondition1.SelectedItem.Value, txtValue1_1.Text.Trim(), txtValue2_1.Text.Trim());
                            }
                            if (!txtValue1_2.Text.ToString().Equals(string.Empty))
                            {
                                dsValues.Tables[1].Rows.Add(drpfields2.SelectedItem.Value, drpcondition2.SelectedItem.Value, txtValue1_2.Text.Trim(), txtValue2_2.Text.Trim());
                            }
                            if (!txtValue1_3.Text.ToString().Equals(string.Empty))
                            {
                                dsValues.Tables[1].Rows.Add(drpfields3.SelectedItem.Value, drpcondition3.SelectedItem.Value, txtValue1_3.Text.Trim(), txtValue2_3.Text.Trim());
                            }
                            if (!txtValue1_4.Text.ToString().Equals(string.Empty))
                            {
                                dsValues.Tables[1].Rows.Add(drpfields4.SelectedItem.Value, drpcondition4.SelectedItem.Value, txtValue1_4.Text.Trim(), txtValue2_4.Text.Trim());
                            }
                            if (dsValues.Tables[1].Rows.Count > 0)
                            {
                                DataSet dsHeaderComments = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Comments, dsValues.Tables[1]);
                                if (dsHeaderComments.Tables.Count > 1)
                                {
                                    wdgLookup.Rows.Clear();
                                    wdgLookup.DataSource = dsHeaderComments.Tables[1];
                                    wdgLookup.DataBind();
                                    ViewState["dsHeader"] = dsHeaderComments.Tables[1];
                                }
                            }
                            mpeLookupShipTo.Show();
                            upGrid.Update();
                            upLookUP.Update();
                            #endregion
                        }
                        else
                            if (ViewState["text"].ToString().Equals("LineComment"))
                            {
                                #region Line Comments Look UP
                                MSISalesOrderAPI MSIAPIE = new MSISalesOrderAPI();
                                DataSet dsValues = CreateDataset();

                                if (!txtValue1_1.Text.ToString().Equals(string.Empty))
                                {
                                    dsValues.Tables[1].Rows.Add(drpfields1.SelectedItem.Value, drpcondition1.SelectedItem.Value, txtValue1_1.Text.Trim(), txtValue2_1.Text.Trim());
                                }
                                if (!txtValue1_2.Text.ToString().Equals(string.Empty))
                                {
                                    dsValues.Tables[1].Rows.Add(drpfields2.SelectedItem.Value, drpcondition2.SelectedItem.Value, txtValue1_2.Text.Trim(), txtValue2_2.Text.Trim());
                                }
                                if (!txtValue1_3.Text.ToString().Equals(string.Empty))
                                {
                                    dsValues.Tables[1].Rows.Add(drpfields3.SelectedItem.Value, drpcondition3.SelectedItem.Value, txtValue1_3.Text.Trim(), txtValue2_3.Text.Trim());
                                }
                                if (!txtValue1_4.Text.ToString().Equals(string.Empty))
                                {
                                    dsValues.Tables[1].Rows.Add(drpfields4.SelectedItem.Value, drpcondition4.SelectedItem.Value, txtValue1_4.Text.Trim(), txtValue2_4.Text.Trim());
                                }
                                if (dsValues.Tables[1].Rows.Count > 0)
                                {
                                    DataSet dsLineComments = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Comments, dsValues.Tables[1]);

                                    if (dsLineComments.Tables.Count > 1)
                                    {
                                        wdgLookup.Rows.Clear();
                                        wdgLookup.DataSource = dsLineComments.Tables[1];
                                        wdgLookup.DataBind();
                                        ViewState["dsLine"] = dsLineComments.Tables[1];
                                    }
                                }
                                mpeLookupShipTo.Show();
                                upLineComment.Update();
                                upLookUP.Update();
                                #endregion
                            }
                            else
                                if (ViewState["text"].ToString().Equals("LineItem"))
                                {
                                    #region Line Item Look UP
                                    MSISalesOrderAPI MSIAPIE = new MSISalesOrderAPI();
                                    DataSet dsValues = CreateDataset();

                                    if (!txtValue1_1.Text.ToString().Equals(string.Empty))
                                    {
                                        dsValues.Tables[1].Rows.Add(drpfields1.SelectedItem.Value, drpcondition1.SelectedItem.Value, txtValue1_1.Text.Trim(), txtValue2_1.Text.Trim());
                                    }
                                    if (!txtValue1_2.Text.ToString().Equals(string.Empty))
                                    {
                                        dsValues.Tables[1].Rows.Add(drpfields2.SelectedItem.Value, drpcondition2.SelectedItem.Value, txtValue1_2.Text.Trim(), txtValue2_2.Text.Trim());
                                    }
                                    if (!txtValue1_3.Text.ToString().Equals(string.Empty))
                                    {
                                        dsValues.Tables[1].Rows.Add(drpfields3.SelectedItem.Value, drpcondition3.SelectedItem.Value, txtValue1_3.Text.Trim(), txtValue2_3.Text.Trim());
                                    }
                                    if (!txtValue1_4.Text.ToString().Equals(string.Empty))
                                    {
                                        dsValues.Tables[1].Rows.Add(drpfields4.SelectedItem.Value, drpcondition4.SelectedItem.Value, txtValue1_4.Text.Trim(), txtValue2_4.Text.Trim());
                                    }
                                    if (dsValues.Tables[1].Rows.Count > 0)
                                    {
                                        DataSet dsLookUP = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Item, dsValues.Tables[1], SOE.DfltWhseKey);

                                        if (dsLookUP.Tables.Count > 1)
                                        {
                                            wdgLookup.Rows.Clear();
                                            wdgLookup.DataSource = dsLookUP.Tables[1];
                                            wdgLookup.DataBind();
                                            ViewState["dsItem"] = dsLookUP.Tables[1];
                                            upSOLine.Update(); upsoheader.Update();
                                        }
                                    }
                                    mpeLookupShipTo.Show();
                                    //upLineComment.Update();
                                    upLookUP.Update();
                                    #endregion
                                }
                                else
                                    if (ViewState["text"].ToString().Equals("Customer"))
                                    {
                                        #region Customer Look UP
                                        MSISalesOrderAPI MSIAPIE = new MSISalesOrderAPI();
                                        DataSet dsValues = CreateDataset();
                                        //dtValues.Columns.Add("FieldName", typeof(string));
                                        //dtValues.Columns.Add("Operator",typeof(string));
                                        //dtValues.Columns.Add("Value1", typeof(string));
                                        //dtValues.Columns.Add("Value2", typeof(string));
                                        if (!txtValue1_1.Text.ToString().Equals(string.Empty))
                                        {
                                            dsValues.Tables[1].Rows.Add(drpfields1.SelectedItem.Value, drpcondition1.SelectedItem.Value, txtValue1_1.Text.Trim(), txtValue2_1.Text.Trim());
                                        }
                                        if (!txtValue1_2.Text.ToString().Equals(string.Empty))
                                        {
                                            dsValues.Tables[1].Rows.Add(drpfields2.SelectedItem.Value, drpcondition2.SelectedItem.Value, txtValue1_2.Text.Trim(), txtValue2_2.Text.Trim());
                                        }
                                        if (!txtValue1_3.Text.ToString().Equals(string.Empty))
                                        {
                                            dsValues.Tables[1].Rows.Add(drpfields3.SelectedItem.Value, drpcondition3.SelectedItem.Value, txtValue1_3.Text.Trim(), txtValue2_3.Text.Trim());
                                        }
                                        if (!txtValue1_4.Text.ToString().Equals(string.Empty))
                                        {
                                            dsValues.Tables[1].Rows.Add(drpfields4.SelectedItem.Value, drpcondition4.SelectedItem.Value, txtValue1_4.Text.Trim(), txtValue2_4.Text.Trim());
                                        }
                                        if (dsValues.Tables[1].Rows.Count > 0)
                                        {
                                            DataSet dsCustomer = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Customer, dsValues.Tables[1], getUserID());

                                            if (dsCustomer.Tables.Count > 1)
                                            {
                                                wdgLookup.Rows.Clear();
                                                wdgLookup.DataSource = dsCustomer.Tables[1];
                                                wdgLookup.DataBind();
                                                ViewState["dsCustomer"] = dsCustomer.Tables[1];
                                            }
                                        }
                                        mpeLookupShipTo.Show();
                                        #endregion
                                    }
                                    else
                                        if (ViewState["text"].ToString().Equals("ShipTo"))
                                        {
                                            #region Ship To Look UP
                                            MSISalesOrderAPI MSIAPIE = new MSISalesOrderAPI();
                                            DataSet dsValues = CreateDataset();
                                            //dtValues.Columns.Add("FieldName", typeof(string));
                                            //dtValues.Columns.Add("Operator",typeof(string));
                                            //dtValues.Columns.Add("Value1", typeof(string));
                                            //dtValues.Columns.Add("Value2", typeof(string));
                                            if (!txtValue1_1.Text.ToString().Equals(string.Empty))
                                            {
                                                dsValues.Tables[1].Rows.Add(drpfields1.SelectedItem.Value, drpcondition1.SelectedItem.Value, txtValue1_1.Text.Trim(), txtValue2_1.Text.Trim());
                                            }
                                            if (!txtValue1_2.Text.ToString().Equals(string.Empty))
                                            {
                                                dsValues.Tables[1].Rows.Add(drpfields2.SelectedItem.Value, drpcondition2.SelectedItem.Value, txtValue1_2.Text.Trim(), txtValue2_2.Text.Trim());
                                            }
                                            if (!txtValue1_3.Text.ToString().Equals(string.Empty))
                                            {
                                                dsValues.Tables[1].Rows.Add(drpfields3.SelectedItem.Value, drpcondition3.SelectedItem.Value, txtValue1_3.Text.Trim(), txtValue2_3.Text.Trim());
                                            }
                                            if (!txtValue1_4.Text.ToString().Equals(string.Empty))
                                            {
                                                dsValues.Tables[1].Rows.Add(drpfields4.SelectedItem.Value, drpcondition4.SelectedItem.Value, txtValue1_4.Text.Trim(), txtValue2_4.Text.Trim());
                                            }
                                            if (dsValues.Tables[1].Rows.Count > 0)
                                            {
                                                DataSet dsShipTo = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.CustomerAddress, dsValues.Tables[1], getUserID(), SOE.CustID);

                                                if (dsShipTo.Tables.Count > 1)
                                                {
                                                    wdgLookup.Rows.Clear();
                                                    wdgLookup.DataSource = dsShipTo.Tables[1];
                                                    wdgLookup.DataBind();
                                                    ViewState["dsShipTo"] = dsShipTo.Tables[1];
                                                }
                                            }
                                            mpeLookupShipTo.Show();
                                            #endregion
                                        }

                                        else
                                            if (ViewState["text"].ToString().Equals("SalesOrder"))
                                            {
                                                #region Sales Order Look UP
                                                MSISalesOrderAPI MSIAPIE = new MSISalesOrderAPI();
                                                DataSet dsValues = CreateDataset();
                                                //dtValues.Columns.Add("FieldName", typeof(string));
                                                //dtValues.Columns.Add("Operator",typeof(string));
                                                //dtValues.Columns.Add("Value1", typeof(string));
                                                //dtValues.Columns.Add("Value2", typeof(string));
                                                if (!txtValue1_1.Text.ToString().Equals(string.Empty))
                                                {
                                                    dsValues.Tables[1].Rows.Add(drpfields1.SelectedItem.Value, drpcondition1.SelectedItem.Value, txtValue1_1.Text.Trim(), txtValue2_1.Text.Trim());
                                                }
                                                if (!txtValue1_2.Text.ToString().Equals(string.Empty))
                                                {
                                                    dsValues.Tables[1].Rows.Add(drpfields2.SelectedItem.Value, drpcondition2.SelectedItem.Value, txtValue1_2.Text.Trim(), txtValue2_2.Text.Trim());
                                                }
                                                if (!txtValue1_3.Text.ToString().Equals(string.Empty))
                                                {
                                                    dsValues.Tables[1].Rows.Add(drpfields3.SelectedItem.Value, drpcondition3.SelectedItem.Value, txtValue1_3.Text.Trim(), txtValue2_3.Text.Trim());
                                                }
                                                if (!txtValue1_4.Text.ToString().Equals(string.Empty))
                                                {
                                                    dsValues.Tables[1].Rows.Add(drpfields4.SelectedItem.Value, drpcondition4.SelectedItem.Value, txtValue1_4.Text.Trim(), txtValue2_4.Text.Trim());
                                                }
                                                if (dsValues.Tables[1].Rows.Count > 0)
                                                {
                                                    DataSet dsOrders = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Orders, dsValues.Tables[1], getUserID());

                                                    if (dsOrders.Tables.Count > 1)
                                                    {
                                                        wdgLookup.Rows.Clear();
                                                        wdgLookup.DataSource = dsOrders.Tables[1];
                                                        wdgLookup.DataBind();
                                                        ViewState["dsOrders"] = dsOrders.Tables[1];
                                                    }
                                                }
                                                mpeLookupShipTo.Show();
                                                #endregion
                                            }
                                            else
                                            {
                                                mpeLookupShipTo.Show();
                                            }
            upLookUP.Update();
            upsoheader.Update();
        }
        protected void imgBtnEnotify_Click(Object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (SOE != null)
            {
                String str = MSISalesOrder.ConnectionClass.ENotify;
                string strURL = str + SOE.CustID + "&CompanyID=" + strCompanyID;
                // Response.Redirect("http://intranet.msistone.com/pickingsheet/PickingSheetPrinting.aspx?SO=" + SOE.TranNo + "&CompanyID=" + strCompanyID);
                ScriptManager.RegisterStartupScript(Page, Page.GetType(), "popup", "window.open('" + strURL + "','_blank')", true);
            }
        }
        protected void imgbtnWMS_Click(Object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (SOE != null)
            {
                String str = MSISalesOrder.ConnectionClass.WMS;
                string strURL = str + SOE.TranNo + "&CompanyID=" + strCompanyID; 
                // Response.Redirect("http://intranet.msistone.com/pickingsheet/PickingSheetPrinting.aspx?SO=" + SOE.TranNo + "&CompanyID=" + strCompanyID);
                ScriptManager.RegisterStartupScript(Page, Page.GetType(), "popup", "window.open('" + strURL + "','_blank')", true);
            }
        }
        protected void imgbtnBillOfLading_Click(Object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (SOE != null)
            {
                String str = MSISalesOrder.ConnectionClass.BillOfLading;
                string strURL = str + SOE.TranNo;
                // Response.Redirect("http://intranet.msistone.com/pickingsheet/PickingSheetPrinting.aspx?SO=" + SOE.TranNo + "&CompanyID=" + strCompanyID);
                ScriptManager.RegisterStartupScript(Page, Page.GetType(), "popup", "window.open('" + strURL + "','_blank')", true);
            }
        }
        protected void imgbtnSOPrint_Click(Object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (SOE != null)
            {
                String str = MSISalesOrder.ConnectionClass.SOPrintLink;
                string strURL = str + SOE.TranNo + "&CompanyID=" + strCompanyID;
                // Response.Redirect("http://intranet.msistone.com/pickingsheet/PickingSheetPrinting.aspx?SO=" + SOE.TranNo + "&CompanyID=" + strCompanyID);
                ScriptManager.RegisterStartupScript(Page, Page.GetType(), "popup", "window.open('" + strURL + "','_blank')", true);
            }
        }
        protected void imgbtnPSPrint_Click(Object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (SOE != null)
            {
                String str = MSISalesOrder.ConnectionClass.PickingSheetLink;
                string strURL = str + SOE.TranNo + "&CompanyID=" + strCompanyID;
               // Response.Redirect("http://intranet.msistone.com/pickingsheet/PickingSheetPrinting.aspx?SO=" + SOE.TranNo + "&CompanyID=" + strCompanyID);
                ScriptManager.RegisterStartupScript(Page, Page.GetType(), "popup", "window.open('" + strURL + "','_blank')", true);
            }

        }
        #region "BillTo Address Button Click Event"
        // Stan_Note: Well, it make sense for blocking BillTo
        /// <summary>
        /// Handles the OnClick event of the btnBillTo control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnBillTo_OnClick(Object sender, EventArgs e)
        {
            try
            {
                //hdBilltoShipTo.Value = "bill";// assigning bill to Hidden field
                //SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];     // Assigning Value From Session To 
                //GetBillToShipToAddress(SOE.BillToAddrKey); // passing address key
                //mdlBillToAddress.Enabled = true;
                //mdlBillToAddress.Show();
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }
        #endregion

        // Correction: BillToAddrKey should be AddrKey or BillToShipToAddrKey
        #region "GetBillTo ShipTo Address Data into Modal PopUp Window"
        /// <summary>
        /// Gets the ship to address.
        /// </summary>
        /// <param name="CustKey">The cust key.</param>
        /// <param name="ShipTo">The ship to.</param>
        private void GetShipToAddress(string CustKey, string ShipTo)
        {
            try
            {
                //Customer Cust = new Customer();
                
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                string result = SOE.ChangeShipToCAddr(Convert.ToInt32(CustKey), ShipTo);
                if (result != string.Empty)
                {
                    lblDebug.Text = result;
                    lblDebug.Visible = true;
                }

                #region Assigning Ship To Address To the controls

                //Assigning cust name
                txtCustName.Text = SOE._ShipToAddress.AddrName;
                txtShipTo.Text = SOE._ShipToAddress.CustAddrID;

                // Assigning Address line 
                txtShipToAddrLine1.Text = SOE._ShipToAddress.AddrLine1;
                txtShipToAddrLine2.Text = SOE._ShipToAddress.AddrLine2;
                txtShipToAddrLine3.Text = SOE._ShipToAddress.AddrLine3;
                txtShipToAddrLine4.Text = SOE._ShipToAddress.AddrLine4;
                txtShipToAddrLine5.Text = SOE._ShipToAddress.AddrLine5;
                chkResidentialShipTo.Checked = SOE._ShipToAddress.Residential;
                // Assigning Address Key
                hdShipToToAddrKey.Value = SOE._ShipToAddress.CustAddrKey.ToString();
                txtShipAddrCity.Text = SOE._ShipToAddress.City.ToString().Trim();

                txtShipToCountryID.Text = SOE._ShipToAddress.CountryID.ToString().Trim();
                txtShipPostCode.Text = SOE._ShipToAddress.PostalCode.ToString().Trim();
                // binding state From Function to drop down

                txtShipAddrState.Text = SOE._ShipToAddress.StateID.ToString().Trim();// (dtBillToShipTo.Rows[0]["StateID"].Equals(DBNull.Value) ? string.Empty : dtBillToShipTo.Rows[0]["StateID"]).ToString();// assigning state
                txtCommPlan.Text = SOE._ShipToAddress.CommPlanID;
                txtHeadShipVia.Text = SOE._ShipToAddress.ShipMethID;
                txtHeadWarehouse.Text = SOE._ShipToAddress.WhseID.ToString();
                txtSalesPerson.Text = SOE._ShipToAddress.SalesPersonID;

                aceShipToState.ContextKey = txtShipToCountryID.Text.Trim();
                aceShipToCity.ContextKey = txtShipToCountryID.Text.Trim() + "~" + txtShipAddrState.Text.Trim();
                aceShipToZipCode.ContextKey = txtShipToCountryID.Text.Trim() + "~" + txtShipAddrState.Text.Trim() + "~" + txtShipAddrCity.Text.Trim();
            
                BindSoLIneItem(SOE.LineItems);
                #endregion
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "GetBillToShipToAddress"
     
     
        #endregion

        // Usage: ShipTo/BillTo Address Entry/lookup Page's State ddl Binding, based on CountryID ( mandatory field)
        #region "Bind State Drop Down"
       
       
        /// <summary>
        /// Binds the bill state ID dropdown list.
        /// </summary>
        /// <param name="CountryID">The country ID.</param>
       
        // end "BuildStateIDDropdownList"
        #endregion

        // Usage: ShipTo/BillTo Address Entry/lookup Page's Country ddl Binding.
        // Correction: vaiable name 'list'

       

        #region "Address Modal PopUp Save and Cancel Works"
        // Usage: Modal PopUp (for regardless of ShipTo or BillTo ) Address "Cancel" Button Click        
        #region "ShipTo /Bill To Address Modal PopUp Cancel Button Click Event"
        /// <summary>
        /// Handles the Click event of the btnShipToAddrCancel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnShipToAddrCancel_Click(Object sender, EventArgs e)
        {
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                //SHowing customer info
                ShowCustomerInfo(SOE);
                // GetShipToAddress(hdShipToToAddrKey.Value);
                //GetSOInfo();
                mdlShipToAddress.Hide();
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }// hiding 'BillTo Address' Modal pop up
        }   // end of "btnBillAddrCancel_Click"
        /// <summary>
        /// Handles the Click event of the imgClose control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs" /> instance containing the event data.</param>
        protected void imgClose_Click(Object sender, EventArgs e)
        {
            mpeSalesTax.Hide();
        }
        /// <summary>
        /// Handles the Click event of the imgRClose control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void imgRClose_Click(Object sender, EventArgs e)
        {
            mpeRecenSales.Hide();
        }

        /// <summary>
        /// Handles the Click event of the btnBillToAddrCancel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnBillToAddrCancel_Click(Object sender, EventArgs e)
        {
            try
            {

                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                ShowCustomerInfo(SOE);
                mpeBillToAddress.Hide();
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }// hiding 'BillTo Address' Modal pop up
        }   // end of "btnBillAddrCancel_Click"
        #endregion

        // Usage: Modal PopUp (for regardless of ShipTo or BillTo ) Address "Save" Button Click
        #region "BillTo/ShipTo Address Modal PopUp Saving Button Click Event"
        /// <summary>
        /// Handles the Click event of the btnShipToAddrSave control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnShipToAddrSave_Click(Object sender, EventArgs e)
        {
            try
            {


                #region "This is for ShipTo Address Saving Works"



                int ReturnAddrKey = SaveShipToAddress();  // callingfunction to save ship to address
                if (ReturnAddrKey != 0)
                {
                    // ReturnAddrKey would be same or new AddrKey. Whatever it is, we have to put that in SO Object.

                    if (ViewState["SalesOrder"] != null)
                    {
                        SalesOrder oSO = (SalesOrder)ViewState["SalesOrder"];
                        // oSO._ShipToAddress.AddrKey = ReturnAddrKey;  // replacing previous address key with existing one
                        if (hfCheckEdit.Value.Equals("1"))
                        {
                            oSO.IsSOChanged = true;
                        }
                        ViewState["SalesOrder"] = oSO;              // Saving value to Session
                        lblDebug.Visible = true;
                        lblDebug.Text = "ShipTo Address Saved Sucessfully";
                        //ViewState["SalesOrder"] = null;
                        mdlShipToAddress.Hide();
                        BindSoLIneItem(oSO.LineItems);
                        //GetSOInfo();
                        // GetShipToAddress(oSO.CustKey.ToString(), oSO._ShipToAddress.CustAddrID.ToString());
                    }

                }
                else
                {
                    lblDebug.Text = "Error in saving";
                    lblDebug.Visible = true;
                }
                #endregion

                udpShipTo.Update();

            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "btnBillAddrSave_Click"

        /// <summary>
        /// Handles the Click event of the btnBillToAddrSave control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnBillToAddrSave_Click(Object sender, EventArgs e)
        {
            try
            {

                //#region "This is for BillTo Address Saving Works"

                //int ReturnAddrKey = SaveBilltoAddress();  //callingfunction to save BillTo address

                //if (ReturnAddrKey != 0)
                //{
                //    lblDebug.Text = "BillTo Address Saved Sucessfully";     // Displaying Message
                //    mpeBillToAddress.Hide();                                // Hiding Modal PopUP
                //    #region Assigning key to Salesorder/Cust Entity
                //    if (ViewState["SalesOrder"] != null)
                //    {
                //        SalesOrder oSO = (SalesOrder)ViewState["SalesOrder"];
                //        oSO.BillToCustAddrKey = Convert.ToInt32(ReturnAddrKey);  // replacing previous address key with existing one

                //        ViewState["SalesOrder"] = oSO;              // Saving value to Session
                //    }
                //    if (Session["CustEntity"] != null)
                //    {
                //        Customer CustE = (Customer)Session["CustEntity"];
                //        CustE.BillToAddrKey = ReturnAddrKey;  // replacing previous address key with existing one
                //        //CustE.ShipToAddrKey = ReturnAddrKey;
                //        Session["CustEntity"] = CustE;              // Saving value to Session

                //    }
                //    #endregion

                //    // ViewState["SalesOrder"] = null;
                //    GetBillToAddress(ReturnAddrKey.ToString());
                //    //GetSOInfo();
                //}
                //else
                //{
                //    // Saving did not succeed. What should we do?
                //}
                //#endregion

            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "btnBillAddrSave_Click"
        #endregion

        // Usage: called by Address Modal PopUp's Save Button Click event.
        #region Saving Address (Regardless of BillTo or ShipTo )
        /// <summary>
        /// Saves the billto address.
        /// </summary>
        private void SaveBilltoAddress()
        {   // For BillTo -> 1, For ShipTo -> 0             
            //try
            //{

            //    Address AddressE = new Address();
            //    // Address AddrBOL = new Address();
            //    AddressE.ShipToOrBillTo = 1;  // assigning value  (For BillTo -> 1, For ShipTo -> 0 )
            //    /*
            //     * For Convert.ToInt32(hdAddrKey.Value)
            //     * If ShipTo: pass SOHeader's DfltShipToAddrKey 
            //     * If BillTo: pass SOHeader's BillToAddrKey
            //     * */

            //    AddressE.AddrKey = Convert.ToInt32(hdBillToAddrKey.Value);



            //    AddressE.AddrName = txtBillToCustName.Text.Trim();
            //    AddressE.AddrLine1 = txtBillToAddr1.Text.Trim();
            //    AddressE.AddrLine2 = txtBillToAddr2.Text.Trim();
            //    AddressE.AddrLine3 = txtBillToAddr3.Text.Trim();
            //    AddressE.AddrLine4 = txtBillToAddr4.Text.Trim();
            //    AddressE.AddrLine5 = txtBillToAddr5.Text.Trim();
            //    AddressE.City = txtBillToAddrCity.Text.Trim();
            //    AddressE.PostalCode = txtBillToPostalCode.Text.Trim();
            //    AddressE.StateID = ddlBillToState.SelectedItem.Text;
            //    AddressE.CountryID = ddlBillToCountry.SelectedItem.Text;
            //    AddressE.Residential = chkResidentialBillTo.Checked;
            //    AddressE.ReturnAddrKey = 0;
            //    //Calling Save Address function and pass cutomer 
            //    AddressE.ReturnAddrKey = AddressE.SaveAddress(AddressE);
            //    /*
            //    * 
            //    * If ShipTo: returns DfltShipToAddrKey 
            //    * If BillTo: returns BillToAddrKey
            //    * */
            //    return AddressE.ReturnAddrKey;// return address key  
            //}
            //catch (Exception Exe)
            //{
            //    lblDebug.Text = Exe.Message;
            //    return 0;
            //}
        }   // end "SaveBilltoShipToAddress"

        /// <summary>
        /// Saves the ship to address.
        /// </summary>
        /// <returns></returns>
        private int SaveShipToAddress()
        {   // For BillTo -> 1, For ShipTo -> 0             
            try
            {

                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                // Address AddrBOL = new Address();
                // AddressE.ShipToOrBillTo = 0;  // assigning value  (For BillTo -> 1, For ShipTo -> 0 )
                /*
                 * For Convert.ToInt32(hdAddrKey.Value)
                 * If ShipTo: pass SOHeader's DfltShipToAddrKey 
                 * If BillTo: pass SOHeader's BillToAddrKey
                 * */

                // SOE._ShipToAddress.AddrKey = Convert.ToInt32(hdShipToToAddrKey.Value);

                SOE._ShipToAddress.AddrName = txtCustName.Text.Trim();
                SOE._ShipToAddress.AddrLine1 = txtShipToAddrLine1.Text.Trim();
                SOE._ShipToAddress.AddrLine2 = txtShipToAddrLine2.Text.Trim();
                SOE._ShipToAddress.AddrLine3 = txtShipToAddrLine3.Text.Trim();
                SOE._ShipToAddress.AddrLine4 = txtShipToAddrLine4.Text.Trim();
                SOE._ShipToAddress.AddrLine5 = txtShipToAddrLine5.Text.Trim();
                SOE._ShipToAddress.City = txtShipAddrCity.Text.Trim();
                SOE._ShipToAddress.PostalCode = txtShipPostCode.Text.Trim();
                SOE._ShipToAddress.StateID = txtShipAddrState.Text.Trim();
                SOE._ShipToAddress.CountryID = txtShipToCountryID.Text.Trim();
                SOE._ShipToAddress.Residential = chkResidentialShipTo.Checked;
                if (hfCheckEdit.Value.Equals("1"))
                {
                    SOE.IsSOChanged = true;
                    SOE.IsSOChang = true;
                }
                return 1;
                //Calling Save Address function and pass cutomer 
                // SOE.ChangeShipToAddr();
                /*
                * 
                * If ShipTo: returns DfltShipToAddrKey 
                * If BillTo: returns BillToAddrKey
                * */

                //return SOE._ShipToAddress.AddrKey;// return address key  
            }
            catch (Exception Exe)
            {
                lblDebug.Text = Exe.Message;
                return 0;
            }
        }   // end "SaveBilltoShipToAddress"
        #endregion
        #endregion

        #endregion

        // Usage: Alot of usage: Everytime, SO is prompt, whenever.
        // Note: How SOLine and its comment populate?
        #region "Get SO INFO: Prompting/Populating Entire SO Data"
        /// <summary>
        /// GetSOInfo: Gets Saved SO info into SOObject.
        /// </summary>
        /// <param name="SOEntityE">The SO entity E.</param>
        private void LoadSOInfo(SalesOrder SOEntityE)
        {
            try
            {


                //SalesOrder SOEntityE = (SalesOrder)ViewState["SalesOrder"];
                // SOHeader Data Population
                if (SOEntityE != null)
                {
                    if (SOEntityE.CustID != null)
                    {
                        #region Assigning all values from  To Particular Text Boxes
                        txtSalesOrder.Text = SOEntityE.TranNo;
                        txtOrderWeight.Text = SOEntityE.SOWeight;
                        hfReason.Value = SOEntityE.ChngReason;
                        hfChangeDate.Value = SOEntityE.ChngOrdDate.ToString(); //ToShortDateString();
                        hfChangedBy.Value = SOEntityE.ChngUserID;
                        txtChngOrderNo.Text = SOEntityE.ChngOrdNo.ToString();      // Correction: tbUpdateNum naming is wrong, should be 'ChngOrderNo'
                        txtSalesOrder.Text = SOEntityE.TranNo;
                        txtCustomer.Text = SOEntityE.CustID;
                        txtShipTo.Text = SOEntityE._ShipToAddress.AddrName;
                        txtSalesPerson.Text = SOEntityE.SalesPersonID;
                        txtCustomerPO.Text = SOEntityE.CustPONo;
                        if (((int)(SOEntityE.Status)) == 5)
                        {
                            ddStatus.SelectedValue = "1";
                        }
                        else
                        {
                            ddStatus.SelectedValue = ((int)(SOEntityE.Status)).ToString();
                        }
                        txtPaymentTerms.Text = SOEntityE.PmtTermsID;
                        lblStaxAmt.Text = SOEntityE.STaxAmt.ToString("c");

                        //  SOEntityE.IsSOChanged = true;
                        txtOrderWeight.Text = SOEntityE.SOWeight;
                        // SOEntityE.CalculateSOTotal();
                        txtOrderTotal.Text = SOEntityE.SalesAmt.ToString();//.ToString(SOEntityE.TranAmt);
                        txtBillTo.Text = SOEntityE._BillToAddress.AddrName;
                        txtBillToName.Text = SOEntityE.CustID;
                        txtCustClass.Text = SOEntityE.CustClassID.ToString();

                        txtHeadWarehouse.Text = SOEntityE.WhseID;
                        txtOrderDate.Text = SOEntityE.TranDate.ToShortDateString();
                        ddlSource.SelectedValue = SOEntityE.SalesSourceKey.ToString();
                        hdDefaultWH.Value = (SOEntityE._ShipToAddress.WhseKey).ToString();
                        txtHold.Text = SOEntityE.HoldReason;
                        chkHold.Checked = SOEntityE.Hold;
                        txtChangeNo.Text = (SOEntityE.ChngOrdNo + 1).ToString();
                        if (SOEntityE.DfltRequestDate != null)
                        {
                            txtrequest.Text = Convert.ToDateTime(SOEntityE.DfltRequestDate).ToShortDateString();
                        }
                        if (SOEntityE.DfltShipDate != null)
                        {
                            txtSchedShip.Text = Convert.ToDateTime(SOEntityE.DfltShipDate).ToShortDateString();
                        }
                        if (SOEntityE.DfltPromiseDate != null)
                        {
                            txtPromised.Text = Convert.ToDateTime(SOEntityE.DfltPromiseDate).ToShortDateString();
                        }
                        txtCommPlan.Text = SOEntityE.DfltCommPlanID;
                        hdCommPlan.Value = SOEntityE._ShipToAddress.CommPlanKey.ToString();

                        if (SOEntityE.TranType == "840")
                        {
                            txtChngOrderNo.Visible = false;
                        }
                        if (chkHold.Checked)
                        {
                            txtHold.Enabled = true;
                        }
                        else
                        {
                            txtHold.Enabled = false;
                        }
                        aceShipTo.ContextKey = SOEntityE.CustKey.ToString();
                        #endregion


                        lnkOrderTotal.Visible = true;
                        lnkOrderTotal.Enabled = true;

                        lnkOrderTotal.Attributes.Add("style", "Color:Blue");
                        pnlTooltipTxt.Attributes.Add("style", "display:block");
                        lblSalesAmount.Text = SOEntityE.SalesAmtHC.ToString("c"); ;
                        lblFreight.Text = SOEntityE.FreightAmt.ToString("c");
                        lblTradeDisc.Text = SOEntityE.TradeDiscAmt.ToString("c");
                        lblSalestax.Text = SOEntityE.STaxAmt.ToString("c");
                        lblOrderTotal.Text = SOEntityE.SalesAmt.ToString("c");


                        lblSalesAmount_new.Text = SOEntityE.SalesAmtHC.ToString("c"); ;
                        lblFreight_new.Text = SOEntityE.FreightAmt.ToString("c");
                        lblTradeDisc_new.Text = SOEntityE.TradeDiscAmt.ToString("c");
                        lblSalestax_new.Text = SOEntityE.STaxAmt.ToString("c");
                        lblOrderTotal_new.Text = SOEntityE.SalesAmt.ToString("c");


                        //  Session["OrigHeaderComments"] = null;
                        // Assigning Header Comments to view State
                        if (SOEntityE.Comments == null)
                        {
                            ViewState["HeaderComment"] = null;
                        }
                        else
                        {
                            ViewState["HeaderComment"] = SOEntityE.Comments;
                        }

                        if (SOEntityE.TranType == "Sales Order")
                        {
                            rbSO.Checked = true;
                        }
                        else
                            if (SOEntityE.TranType == "Quote")
                            {
                                rbQuote.Checked = true;
                                imgQuoteToSO.Enabled = true;
                                if (ddStatus.SelectedValue == "4")
                                {
                                    imgQuoteToSO.Visible = false;
                                }
                                else
                                {
                                    imgQuoteToSO.Visible = true;
                                }
                            }
                        ShowCustomerInfo(SOEntityE);
                        SetInitialsForPopUp();

                        BindSoLIneItem(SOEntityE.LineItems);

                        upGrid.Update();
                        ValueEnableDisable();
                        gvSOHeaderComment.Enabled = true; ;
                        btnCommSave.Enabled = true;
                        txtChanreason.Text = string.Empty;
                        ViewState["SalesOrder"] = SOEntityE;
                        upGrid.Update();
                        upsoheader.Update();

                        imgbtnPSPrint.Enabled = true;
                        imgbtnSOPrint.Enabled = true;
                        imgbtnBillOfLading.Enabled = true;
                        imgbtnWMS.Enabled = true;
                        imgBtnEnotify.Enabled = true;
                    }
                }
                else
                {
                    lblDebug.Text = "Error In loading";
                    lblDebug.Visible = true;
                }
            }
            catch (Exception exe)
            {
                // Tracing Exception
                lblDebug.Text = exe.Message;
            }
        }   // end "GetSOInfo"
        #endregion




        /// <summary>
        /// Handles the Click event of the btnSalesTax control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void btnSalesTax_Click(Object sender, System.EventArgs e)
        {

        }
        /// <summary>
        /// Handles the Click event of the lnkOrderTotal control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void lnkOrderTotal_Click(Object sender, System.EventArgs e)
        {

            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            DataTable dtSalesTax = SOE.GetSalesTaxDetails();
            if (dtSalesTax != null)
            {
                grdSalesTax.DataSource = dtSalesTax;
                grdSalesTax.DataBind();


            }

            upss.Update();
            //pnlTooltipTxt.Visible = true;
            mpeSalesTax.Show();

            upsoheader.Update();
        }
        /// <summary>
        /// Handles the Click event of the QuoteToSO control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void QuoteToSO_Click(Object sender, System.EventArgs e)
        {
            try
            {
                lblDebug.Text = string.Empty;
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                // Added by Vinay Gawli on 11/01/2013
                DataTable dtResult = SOE.CheckQuoteB4Conversion4Inventory(SOE.SOKey);
                if (dtResult.Rows.Count == 0)
                {
                    int result = SOE.QuoteToSO(getUserID(), Session["SessionKey"].ToString());
                    if (result == 0)
                    {
                        lblDebug.Text = "Conversion Failed.";
                        lblDebug.Visible = true;
                    }
                    else
                    {
                        lblDebug.Text = "Sucessfully Converted Quote To SO.";
                        lblDebug.Visible = true;
                        imgQuoteToSO.Visible = false;
                        imgQuoteToSO.Enabled = false;
                        upTop.Update();
                        ViewState["SalesOrder"] = null;
                        hdfSalesQuote.Value = "801";
                        hidSOKey.Value = string.Empty;
                        rbQuote.Checked = false;
                        // rbSO.Checked = true;
                        txtChngOrderNo.Visible = true;
                        // upsoheader.Update();
                        CheckInitailsConditionsForSO();
                    }
                }
                else
                {
                    string Error = string.Empty;
                    for (int i = 0; i < dtResult.Rows.Count; i++)
                    {
                        Error = Error + "Message:" + dtResult.Rows[i]["Message"].ToString() + " ErrorAt:" + dtResult.Rows[i]["ErrorArea"].ToString() + " Solution:" + dtResult.Rows[i]["Solution"].ToString() + "</br>";
                    }
                    lblDebug.Text = "Conversion Failed:</br>" + Error;
                    lblDebug.Visible = true;
                }
            }
            catch (Exception ex)
            {
                lblDebug.Text = "Conversion Failed:-" + ex;
                lblDebug.Visible = true;
                upTop.Update();
            }

        }
        #region "Customer is over Credit Limit Warning Message Handling (OK/Cancel)"
        // Usage: OK/Continuing of Customer CreditLimitOver Warning Message
        /// <summary>
        /// btnSOOverCreditOK_Click Event: Warning Message to end User about So's Customer's Credit Limit Overing.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        //#region "Customer CreditLimitOver Warning Message's OK Button Click"
        //protected void btnSOOverCreditOK_Click(Object sender, System.EventArgs e)
        //{
        //    if (Session["CustEntity"] != null)
        //    {
        //        getCustomerInfo();
        //    }

        //    else if (ViewState["SalesOrder"] != null)
        //    {
        //        //GetSOInfo();    // Show SO Info
        //        // ValueEnableDisable();
        //        SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
        //        if (SOE.Status == 1 || SOE.Status == 2)
        //        {
        //            DisableAllControls();
        //            webgvSOLineItem.Enabled = false;
        //            upSOLine.Update();upsoheader.Update();;
        //            btnSave.Enabled = false;
        //            btnFinish.Enabled = false;
        //            btnCancel.Enabled = true;
        //            ddStatus.Enabled = true;
        //        }
        //        else
        //        {
        //            ValueEnableDisable();
        //        }
        //    }
        //}   // end "btnSOOverCreditOK_Click"
        //#endregion

        // Usage: Canceling of Customer CreditLimitOver Warning Message
        //#region "Customer CreditLimitOver Warning Message's Cancel Button Click"
        //protected void btnSOOverCreditCancel_Click(Object sender, System.EventArgs e)
        //{

        //    if (Session["CustEntity"] == null)
        //    {
        //        Response.Redirect(Request.RawUrl);
        //    }
        //    else
        //    {
        //        txtCustomer.Text = string.Empty;
        //        txtCustomer.Enabled = true;
        //        SetInitialsForPopUp();

        //        BindSoLIneItem(null);
        //    }
        //}// end "btnSOOverCreditCancel_Click"
        //#endregion
        #endregion

        // All SOHeader Comment Works are here...
        #region "SO Header Comment Works ( Save/ Cancel Button Click)"
        // Usage: 
        #region "SO Header Comment Button Click Event"

        #endregion

        // Usage: Called by 'SOHeader Comment Button Click Event'
        #region "Set Initial For SO Header Comment PopUp"
        private void SetInitialsForPopUp()
        {
            try
            {
                gvSOHeaderComment.Visible = true;
                lblCommMessage.Visible = false;
                pnlComment.Visible = true;
                // assing first row to pop up
                List<Comment> lstHeaderComment = (List<Comment>)ViewState["HeaderComment"];
                //Checking list is null
                if (lstHeaderComment == null)
                {
                    //creating a brand new list
                    lstHeaderComment = new List<Comment>();
                }
                else
                    if (lstHeaderComment.Count == 0)
                    {
                        lstHeaderComment = new List<Comment>();
                    }
                #region Creating Blank Row And adding it to List
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                Comment CommentE = new Comment(SOE, 801);
                Comment temComment = null;
                if (lstHeaderComment.Count == 0)
                {
                    temComment = null;
                }
                else
                {
                    temComment = lstHeaderComment[lstHeaderComment.Count - 1];
                }
                if (temComment == null)
                {
                    lstHeaderComment.Add(CommentE);
                }
                else
                    if (lstHeaderComment[lstHeaderComment.Count - 1].ExtCmntID != string.Empty)
                    {
                        if (lstHeaderComment[lstHeaderComment.Count - 1].ExtCmntID != null)
                        {
                            lstHeaderComment.Add(CommentE);
                        }
                    }



                #endregion

                // Displaying Header Comments by passing list
                ViewState["bkupheader"] = lstHeaderComment;

                BindGridView(lstHeaderComment);
                CommentE = null;

            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "SetInitialsForPopUp"
        #endregion

        // Usage: ?
        #region "Bind GridView of SOHeader Comment"
        private void BindGridView(List<Comment> lstComment)
        {
            try
            {
                for (int i = 0; i <= lstComment.Count - 1; i++)
                {
                    lstComment[i].SeqNo = i + 1;
                }

                ViewState["HeaderComment"] = lstComment;// assigning list to view state
                gvSOHeaderComment.DataSource = lstComment;
                gvSOHeaderComment.DataBind();

                // for last row delete button should be false
                Button Del = (Button)gvSOHeaderComment.Rows[lstComment.Count - 1].FindControl("btnDeleteComm");
                Del.Visible = false;
                upGrid.Update();

            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "BindGridView"
        #endregion

        // Usage: ?
        #region "Bind Blank Row Event in SOHeader Comment's GridView"
        private Comment BindBlankRow()
        {
            try
            {
                // binding blank row
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                Comment CommentE = new Comment(SOE, 801);


                //CommentE.ParentDocType = 801;//Convert.ToInt32(rbSalesQuote.SelectedItem.Value);

                //CommentE.ExtCmntUsedKey = 0;

                //CommentE.ExtCmntKey = 0;
                //CommentE.CommentText = string.Empty;
                //CommentE.ExtCmntID = string.Empty;
                //CommentE.SeqNo = 0;
                // oComment.SeqNo = lstCommentEntity.Count + 1;
                return CommentE;
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                return null;
            }
        }   // end "BindBlankRow"
        #endregion

        // Usage: Save Button at SOHeader Comment Modal PopUp
        #region "Comment Modal PopUp Save Button Click"
        /// <summary>
        /// Handles the Click event of the btnSOHeaderCommSave control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnSOHeaderCommSave_Click(Object sender, EventArgs e)
        {

            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            List<Comment> lstComment = (List<Comment>)ViewState["HeaderComment"];// retreiving value from view state

            int flag = 0;
            for (int i = 0; i < lstComment.Count - 1; i++)          // loop from 0 to count-1
            {
                Comment TempCommentE = lstComment[i];         // assigning row to comment entity
                // if id Comment Id/Comment is null
                if (string.IsNullOrEmpty(TempCommentE.ExtCmntID) || string.IsNullOrEmpty(TempCommentE.CommentText))
                {

                    flag = 1;
                    break;
                }


            }
            TextBox txtSOCommID = (TextBox)gvSOHeaderComment.Rows[lstComment.Count - 1].FindControl("txtSOCommID");
            TextBox txtSOCommText = (TextBox)gvSOHeaderComment.Rows[lstComment.Count - 1].FindControl("txtSOCommText");
            if (txtSOCommID.Text.Equals(string.Empty) && txtSOCommText.Text.Equals(string.Empty))
            {

            }
            else
            {
                flag = 1;
            }


            //gvSOHeaderComment.DataSource = lstComment;
            //gvSOHeaderComment.DataBind();

            #region "if it is not a valid comment"
            if (flag == 1)
            {
                //Print Error
                lblCommentError.Text = "Please add valid comment";
                lblCommentError.ForeColor = System.Drawing.Color.Red;
                lblCommentError.Visible = true;
                mdlPopUpViewComment.Show();
            }
            else
            {
                ViewState["HeaderComment"] = lstComment;

                lstComment.RemoveAt(lstComment.Count - 1);

                lstComment.Add(new Comment());
                ViewState["bkupheader"] = lstComment;
                SOE.Comments = lstComment;
                ViewState["SalesOrder"] = SOE;
                BindGridView(lstComment);
                mdlPopUpViewComment.Hide();
                lblCommentError.Text = string.Empty;
                lblCommentError.Visible = false;
                lblDebug.Text = "";
                lblDebug.Visible = true;
                upTop.Update();

            }// end "if (flag == 1)"
            #endregion
        }   // end "btnCommSave_Click"
        #endregion

        // Usage: Cancel Button at SOHeader Comment Modal PopUp 
        #region "Comment Modal PopUp Cancel Button Click"
        /// <summary>
        /// Handles the Click event of the btnSOHeaderCommCancel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnSOHeaderCommCancel_Click(object sender, EventArgs e)
        {
            try
            {
                lblCommentError.Visible = false;
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                SOE.Comments = (List<Comment>)ViewState["bkupheader"];
                SOE.CancelSavingHeaderComment();
                BindGridView((List<Comment>)ViewState["bkupheader"]);
                upGrid.Update();
                upsoheader.Update();
                upTop.Update();
                mdlPopUpViewComment.Hide();
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;

            }
        }   // end of "btnCommCancel_Click"
        #endregion
        /// <summary>
        ///  Handles the Click event of the imgHdrCmtClose control.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void imgHdrCmtClose_Click(object sender, EventArgs e)
        {
            try
            {
                lblCommentError.Visible = false;
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                SOE.Comments = (List<Comment>)ViewState["bkupheader"];
                SOE.CancelSavingHeaderComment();
                BindGridView((List<Comment>)ViewState["bkupheader"]);
                upGrid.Update();
                upsoheader.Update();
                upTop.Update();
                mdlPopUpViewComment.Hide();
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;

            }
        }   // end of "imgHdrCmtClose_Click"

        // Usage: SOHeader CommentID Text Changed
        #region "txtSOHeaderCommID_OnTextChanged: SOHeader CommentID Text Changed"
        /// <summary>
        /// Handles the OnTextChanged event of the txtSOCommID control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        protected void txtSOHeaderCommID_OnTextChanged(Object sender, System.EventArgs e)
        {
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                List<Comment> lstCommentEntity = (List<Comment>)ViewState["HeaderComment"];     //retreiving header comment from view state
                lblCommMessage.Visible = false;
                //Comment CommE = new Comment();
                TextBox txtSOCommID = (TextBox)sender;                      //Finding out CommentID Text Box
                GridViewRow row = (GridViewRow)txtSOCommID.Parent.Parent;   //Finding out which row generate a event
                int index = row.RowIndex;// index ofrow
                ViewState["index"] = index;

             
                // txtSOCommText.Focus();
                string[] splitStr = hidComment.Value.ToString().Split('~');// split the hdfield. It contains comment iD and Comment
                if (splitStr[0].ToString().Equals("0"))
                {
                    string check = SOE.CheckValidComment(txtSOCommID.Text.Trim(), strCompanyID);
                    if (!check.Equals(string.Empty))
                    {
                        txtSOCommID.Text = txtSOCommID.Text.ToUpper().Trim();
                        BindCommentsToTextBox(check, index);
                    }
                    else
                    {
                        hidComment.Value = "0";// IF user didnot select the comment id then hidden field value is zero
                        lblCommentError.Visible = true;
                        lblCommentError.Text = "Invalid Comment";
                    }
                    upGrid.Update();
                    return;
                }
                else
                {
                    BindCommentsToTextBox(hidComment.Value.ToString(), index);   
                }   // end "outer else"
                hidComment.Value = "0";
                upGrid.Update();

            }   // end "try"
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
            }
        }

        private void BindCommentsToTextBox(string check, int index)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

            TextBox txtSOCommText = gvSOHeaderComment.Rows[index].FindControl("txtSOCommText") as TextBox;  // Finding comment textbox
            TextBox txtCommentID = gvSOHeaderComment.Rows[index].FindControl("txtSOCommID") as TextBox;     // comment ID text box
            Label lblExtCommentKey = gvSOHeaderComment.Rows[index].FindControl("lblExtCommentKey") as Label;
            Label lblExtCmntUsedKey = gvSOHeaderComment.Rows[index].FindControl("lblExtCmntUsedKey") as Label;
            Label lblSeqNo = gvSOHeaderComment.Rows[index].FindControl("lblSeqNo") as Label;

            List<Comment> lstCommentEntity = (List<Comment>)ViewState["HeaderComment"];     //retreiving header comment from view state
            lblCommentError.Visible = false;

            string[] splitStr = check.ToString().Split('~');

            StringBuilder sb = new StringBuilder();
            for (int i = 1; i <= splitStr.Length - 1; i++)
            {
                sb.Append(splitStr[i]);             // appending comment
            }
            txtSOCommText.Text = sb.ToString();     // adding comment to text box

            lblExtCommentKey.Text = splitStr[0];
            if (index < (lstCommentEntity.Count - 1))
            {
                // assigning comment to 
                Comment TempCommentE = lstCommentEntity[index];
                TempCommentE.InitializeComments(txtCommentID.Text, Convert.ToInt32(splitStr[0]), txtSOCommText.Text, TempCommentE.ExtCmntUsedKey);


            }
            else
            {
                //adding comment  to last row
                lstCommentEntity.RemoveAt(lstCommentEntity.Count - 1);

                Comment CommentE = new Comment(SOE, 801);

                if (lblExtCmntUsedKey.Text.ToString().Equals(string.Empty))
                {
                    CommentE.InitializeComments(txtCommentID.Text, Convert.ToInt32(splitStr[0]), txtSOCommText.Text, Convert.ToInt32(0));

                }
                else
                {
                    CommentE.InitializeComments(txtCommentID.Text, Convert.ToInt32(splitStr[0]), txtSOCommText.Text, Convert.ToInt32(lblExtCmntUsedKey.Text));
                    // CommentE.ExtCmntUsedKey = Convert.ToInt32(lblExtCmntUsedKey.Text);
                }



                lstCommentEntity.Add(CommentE);

                // adding blank row
                lstCommentEntity.Add(new Comment(SOE, 801));
                //binding grid view
                BindGridView(lstCommentEntity);
                hidComment.Value = "0";
                upGrid.Update();
                //gvSOHeaderComment.Rows[index].FindControl("txtSOCommText").Focus();

            }

        }

        // end "txtSOHeaderCommID_OnTextChanged"
        #endregion

        // Usage: SOHeader CommentTextBox Text Changed
        #region "txtSOHeaderCommText_OnTextChanged"
        /// <summary>
        /// Handles the OnTextChanged event of the txtSOCommText control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        protected void txtSOHeaderCommText_OnTextChanged(Object sender, System.EventArgs e)
        {
            try
            {

                List<Comment> lstCommentEntity = (List<Comment>)ViewState["HeaderComment"];// retreiving header comment

                lblCommMessage.Visible = false;
                Comment CommE = new Comment();
                TextBox txtSOCommID = (TextBox)sender;
                GridViewRow row = (GridViewRow)txtSOCommID.Parent.Parent;
                int index = row.RowIndex;// getting index

                TextBox txtSOCommText = gvSOHeaderComment.Rows[index].FindControl("txtSOCommText") as TextBox;// finding Comment textBox

                ViewState["index"] = index;
                Comment TempCommentE = lstCommentEntity[index];
                TempCommentE.CommentText = txtSOCommText.Text;

                BindGridView(lstCommentEntity);// binding grid view
                if (lstCommentEntity.Count > 1)
                {
                    gvSOHeaderComment.Rows[index + 1].FindControl("txtSOCommID").Focus();// setting focus on next row
                }

            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "txtSOHeaderCommText_OnTextChanged"
        #endregion

        // Usage: ? Used or not? Double triggering? For SOHeader or SOLine? Generic?
        #region "gvSOHeaderComment Row Command"
        /// <summary>
        /// Handles the RowCommand event of the gvSOHeaderComment control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="GridViewCommandEventArgs"/> instance containing the event data.</param>
        protected void gvSOHeaderComment_RowCommand(object sender, GridViewCommandEventArgs e)
        {
            try
            {
                if (e.CommandName == "Delete")// For delete operations
                {
                    GridViewRow gvr = (GridViewRow)((Control)e.CommandSource).NamingContainer;
                    int rowIndex = gvr.RowIndex;// row index
                    List<Comment> lstComment = (List<Comment>)ViewState["HeaderComment"];//getting header comment from view state
                    string strSeqNo = string.Empty;
                    Comment TempCommentE = lstComment[rowIndex];// retrving the particular row for delete

                    strSeqNo = TempCommentE.SeqNo.ToString();
                    if (TempCommentE.ExtCmntUsedKey == 0)
                    {
                        lstComment.RemoveAt(rowIndex);
                    }
                    else
                    {
                        SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                        SOE.DeleteCommentHeader(rowIndex);
                        lstComment.RemoveAt(rowIndex);
                        ViewState["HeaderComment"] = SOE.Comments;

                    }
                    //TempCommentE.SeqNo = Convert.ToInt32(strSeqNo);// txtSOCommText.Text;
                    BindGridView(lstComment);// bind grid view

                }
                else
                    if (e.CommandName == "commentlookup")// For delete operations
                    {
                        GridViewRow gvr = (GridViewRow)((Control)e.CommandSource).NamingContainer;
                        ViewState["HeaderCommentRowIndex"] = gvr.RowIndex;
                        BindLookData("HeaderComment");
                        mpeLookupShipTo.Show();
                        upLookUP.Update();
                        upGrid.Update();
                    }
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "gvSOHeaderComment_RowCommand"
        #endregion

        // Usage: ? Used or not? Double triggering? For SOHeader or SOLine? Generic?
        #region "gvSOHeaderComment Row Datat Bound"
        /// <summary>
        /// Handles the RowDataBound event of the gvSOHeaderComment control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="GridViewRowEventArgs"/> instance containing the event data.</param>
        protected void gvSOHeaderComment_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            try
            {
                if (e.Row.RowType == DataControlRowType.DataRow)
                {
                    if (ViewState["index"] != null)
                    {
                        if (gvSOHeaderComment.Rows.Count > Convert.ToInt32(ViewState["index"]))
                        {
                            // for mouse focus
                            gvSOHeaderComment.Rows[Convert.ToInt32(ViewState["index"])].FindControl("txtSOCommText").Focus();
                        }
                    }
                    Button lnkDel = (Button)e.Row.FindControl("btnDeleteComm");
                    lnkDel.Attributes.Add("onclick", "javascript:return " +
                    "confirm('Are you sure you want to delete this record. ')");
                    //  tb.Focus();
                }
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;

            }
        }   // end "gvSOHeaderComment_RowDataBound"
        #endregion

        // Usage: ? Used or not?
        #region  "gvSOHeaderComment Row Deleting"
        protected void gvSOHeaderComment_RowDeleting(object sender, GridViewDeleteEventArgs e)
        {
            // ?
        }   // end "gvSOHeaderComment_RowDeleting"
        #endregion
        #endregion

        // All SOLine Comment Works are here...
        #region "SOLine Comment Works (Opening/ TextChangedEvent/ BlankCommentRow Binding/ Saving/ Canceling"
        // Usage: SOLine's Comment Button Click
        #region "btnLineComment_Click: SOLine's Comment Button Click"
        /// <summary>
        /// Handles the Click event of the btnLineComment control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void btnLineComment_Click(Object sender, System.EventArgs e)
        {
            try
            {
                lblLineCommentError.Visible = false;
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                List<SOLine> lstSoLine = SOE.LineItems;//(SalesOrder) ViewState["SalesOrder"];
                LinkButton Comment = (LinkButton)sender;

                GridRecord row = (GridRecord)((GridRecordItem)((System.Object)((Infragistics.Web.UI.TemplateContainer)((System.Web.UI.Control)Comment.Parent)).Item)).Row;

                int lineno = Convert.ToInt32(Comment.CommandArgument);

                int index = 0;// row.Index;
                for (int i = 0; i <= lstSoLine.Count - 1; i++)
                {
                    if (lstSoLine[i].SOLineNo == lineno)
                    {
                        index = i;
                    }
                }
                if (SOE.LineItems[index].ItemKey != 0)
                {
                    mpeLineComments.Show();
                }
                else
                {
                    mpeLineComments.Hide();
                }

                //Label lblLineKey = webgvSOLineItem.Rows[index].FindControl("lblLineKey") as Label;
                ViewState["lblLineKey"] = Comment.CommandArgument.ToString();


                ViewState["rowindex"] = index;

                if (lstSoLine[index].SOLineComments != null)
                {
                    for (int i = 0; i < lstSoLine[index].SOLineComments.Count - 1; i++)
                    {
                        if (lstSoLine[index].SOLineComments[i].ExtCmntID.Equals(string.Empty))
                        {
                            lstSoLine[index].SOLineComments.RemoveAt(i);
                        }
                    }
                }
                ViewState["tempLine"] = lstSoLine[index].SOLineComments;
                ViewState["LineComments"] = lstSoLine[index].SOLineComments;
                pnlLineComment.Visible = true;
                SetInitialsForLinePopUp();      // see just below.

                lblCommMessage.Visible = false;

            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "btnLineComment_Click"
        #endregion

        // Usage: ?
        #region "Bind SOLine Comment GridView"
        private void BindLineCommentToGridView(List<Comment> lstComment)
        {
            try
            {
                grdLineComments.DataSource = lstComment;
                grdLineComments.DataBind();
                // for last row delete button should be false
                grdLineComments.Rows[lstComment.Count - 1].FindControl("btnDeleteLineComm").Visible = false;
                upLineComment.Update();
                //for (int i = 0; i < lstComment.Count; i++)
                //{
                //    //getting current line from loop
                //    Comment TempCommentE = lstComment[i];
                //    string strSeqNo = TempCommentE.SeqNo.ToString();
                //    if (TempCommentE.SeqNo.ToString().Length > 0)
                //    {
                //        //Bind only that row that does not contain minus
                //        if (strSeqNo[0].ToString().Equals("-"))
                //        {
                //            //gvSOHeaderComment.cu.CurrentCell = null;
                //            grdLineComments.Rows[i].Visible = false;// makerow visible false
                //        }
                //    }
                //}
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "BindLineCommentToGridView"
        #endregion

        // Usage: Called by btnLineComment_Click, Initial SOLineComment Modal PopUp
        #region "SetInitialsForLinePopUp: Set Initial For SOLine Comment PopUp"
        private void SetInitialsForLinePopUp()
        {
            #region Need To Check
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                if ((List<Comment>)ViewState["LineComments"] == null)
                {
                    List<Comment> templstLineComment = new List<Comment>();
                    ViewState["LineComments"] = templstLineComment;
                }
                //getting comment FRom view state
                List<Comment> lstLineComment = (List<Comment>)ViewState["LineComments"];
                Comment CommentE = new Comment(SOE, 802);
                //CommentE.ExtCmntUsedKey = 0;
                //CommentE.CommentText = string.Empty;
                //CommentE.DocOwnerKey = 0;
                //CommentE.ParentDocType = 802;                     //?, If that is for SOLine, then, 802, why 0? //Convert.ToInt32(rbSalesQuote.SelectedItem.Value);
                //CommentE.SeqNo = lstLineComment.Count + 1;
                //CommentE.ExtCmntID = string.Empty;
                //CommentE.ExtCmntKey = 0;
                if (lstLineComment.Count == 0)
                {
                    CommentE.SeqNo = 1;
                    lstLineComment.Add(CommentE);
                }
                else if (lstLineComment[lstLineComment.Count - 1].ExtCmntID != string.Empty)
                {
                    if (lstLineComment[lstLineComment.Count - 1].ExtCmntID != null)
                    {
                        CommentE.SeqNo = lstLineComment.Count + 1;
                        lstLineComment.Add(CommentE);
                    }
                }


                ViewState["LineComments"] = lstLineComment;
                // Binding LIne Comments to grid view
                BindLineCommentToGridView(lstLineComment);
                CommentE = null;
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
            #endregion
        }   // end "SetInitialsForLinePopUp"
        #endregion

        // Usage: SOLine Comment's CommentID Text Changed
        #region "txtSOLineCommID_OnTextChanged: SOLine Comment's CommentID TextChanged Event"
        /// <summary>
        /// Handles the OnTextChanged event of the txtSOLineCommID control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        protected void txtSOLineCommID_OnTextChanged(Object sender, System.EventArgs e)
        {
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                
                List<Comment> lstCommentEntity = (List<Comment>)ViewState["LineComments"];  // Retreiving line comment from view state
                lblCommMessage.Visible = false;
                Comment CommE = new Comment(SOE, 802);
                TextBox txtSOCommLineID = (TextBox)sender;                                  // Finding out CommentID Text Box
                GridViewRow row = (GridViewRow)txtSOCommLineID.Parent.Parent;               // Finding out which row generate a event
                int index = row.RowIndex;                                                   // index of row
                ViewState["lineindex"] = index;
               

                // split the hdfield. It contains comment iD and Comment
                string[] splitLineComment = hidLIneComment.Value.ToString().Split('~');
                if (splitLineComment[0].ToString().Equals("0"))
                {
                   // string CheckValidComment(string CommentID,string CompanyID)
                    string str = SOE.CheckValidComment(txtSOCommLineID.Text, strCompanyID);

                    if (!str.Equals(string.Empty))
                    {
                        txtSOCommLineID.Text = txtSOCommLineID.Text.Trim().ToUpper();
                        BindLineCommentToTextBox(str, index);
                    }
                    else
                    {
                        // findng comment textbox
                        TextBox txtSOLineCommText = grdLineComments.Rows[index].FindControl("txtSOLineCommText") as TextBox;
                        //findng comment ID text box
                        TextBox txtSOLineCommID = grdLineComments.Rows[index].FindControl("txtSOLineCommID") as TextBox;
                        Label lblLineExtCommentKey = grdLineComments.Rows[index].FindControl("lblLineExtCommentKey") as Label;
                        Label lblLineExtCmntUsedKey = grdLineComments.Rows[index].FindControl("lblLineExtCmntUsedKey") as Label;
                        Label lblLineSeqNo = grdLineComments.Rows[index].FindControl("lblLineSeqNo") as Label;
                        //InitializeComments(string CommentID, int CommentKey, string CommentText, int ExtCmntUsedKey)
                        hidLIneComment.Value = "0"; // IF user didnot select the comment id then hidden field value is zero
                        Comment TempCommentE = lstCommentEntity[index];
                        TempCommentE.InitializeComments(string.Empty, 0, txtSOLineCommText.Text, Convert.ToInt32(lblLineExtCmntUsedKey.Text));

                        lblLineCommentError.Text = "Invalid Comment";
                        lblLineCommentError.Visible = true;
                       
                        return;
                    }
                    
                }
                else
                {

                    BindLineCommentToTextBox(hidLIneComment.Value.ToString(), index);
                    // end "inner else"
                }   // end "outter else"
                upLineComment.Update();
            }
            catch (Exception exe)
            {

                lblLineCommentError.Text = "Invalid Comment";
                lblLineCommentError.Visible = true;
                upLineComment.Update();
            }
        }

        private void BindLineCommentToTextBox(string str, int index)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            // Header Comment? in documentation?
            List<Comment> lstCommentEntity = (List<Comment>)ViewState["LineComments"];  // Retreiving line comment from view state

            Comment CommE = new Comment(SOE, 802);

            // findng comment textbox
            TextBox txtSOLineCommText = grdLineComments.Rows[index].FindControl("txtSOLineCommText") as TextBox;
            //findng comment ID text box
            TextBox txtSOLineCommID = grdLineComments.Rows[index].FindControl("txtSOLineCommID") as TextBox;
            Label lblLineExtCommentKey = grdLineComments.Rows[index].FindControl("lblLineExtCommentKey") as Label;
            Label lblLineExtCmntUsedKey = grdLineComments.Rows[index].FindControl("lblLineExtCmntUsedKey") as Label;
            Label lblLineSeqNo = grdLineComments.Rows[index].FindControl("lblLineSeqNo") as Label;

            StringBuilder sb = new StringBuilder();
            lblLineCommentError.Visible = false;
            string[] splitLineComment = str.Split('~');
            for (int i = 1; i <= splitLineComment.Length - 1; i++)
            {
                sb.Append(splitLineComment[i]); // appending comment
            }
            txtSOLineCommText.Text = sb.ToString();// adding comment to text box

            lblLineExtCommentKey.Text = splitLineComment[0];
            if (index < (lstCommentEntity.Count - 1))
            {
                // assigning comment to 

                Comment TempCommentE = lstCommentEntity[index];
                TempCommentE.InitializeComments(txtSOLineCommID.Text, Convert.ToInt32(splitLineComment[0]), txtSOLineCommText.Text, 0);
            }   // end "if"
            else
            {
                //adding comment to last row
                lstCommentEntity.RemoveAt(lstCommentEntity.Count - 1);

                if (lblLineExtCmntUsedKey.Text.ToString().Equals(string.Empty))
                {
                    CommE.InitializeComments(txtSOLineCommID.Text, Convert.ToInt32(splitLineComment[0]), txtSOLineCommText.Text, 0);
                    CommE.SeqNo = lstCommentEntity.Count + 1;
                    lstCommentEntity.Add(CommE);
                }
                else
                {
                    CommE.InitializeComments(txtSOLineCommID.Text, Convert.ToInt32(splitLineComment[0]), txtSOLineCommText.Text, Convert.ToInt32(lblLineExtCmntUsedKey.Text));
                    CommE.SeqNo = lstCommentEntity.Count + 1;
                    lstCommentEntity.Add(CommE);
                    //CommE.ExtCmntUsedKey = Convert.ToInt32(lblLineExtCmntUsedKey.Text);
                }

                // adding blank row
                Comment CommentE = new Comment(SOE, 802);
                CommentE.SeqNo = lstCommentEntity.Count + 1;
                lstCommentEntity.Add(CommentE);

                List<SOLine> lstSOLIne = SOE.LineItems;// (List<SOLine>)Session["ListSoLIne"];
                lstSOLIne[Convert.ToInt32(ViewState["rowindex"])].SOLineComments = lstCommentEntity;

                //binding grid view

                BindLineCommentToGridView(lstCommentEntity);
                hidLIneComment.Value = "0";

            }
            grdLineComments.Rows[index].FindControl("txtSOLineCommText").Focus();
            hidLIneComment.Value = "0";
        }   // end "txtSOLineCommID_OnTextChanged"
        #endregion

        // Usage: Binding Blank Commnet Row for SOLine Comment
        #region Bind Blank Row in Line Comment GridView
        private Comment BindLineCommentBlankRow()
        {
            try
            {
                // binding blank row
                Comment CommentE = new Comment();
                #region Need to Check
                //CommentE.DocOwnerKey = 0;
                //CommentE.ParentDocType = 0;             /
                //CommentE.ExtCmntUsedKey = 0;
                //CommentE.ExtCmntKey = 0;
                //CommentE.CommentText = string.Empty;
                //CommentE.ExtCmntID = string.Empty;
                //CommentE.SeqNo = 0;
                // oComment.SeqNo = lstCommentEntity.Count + 1;
                #endregion
                return CommentE;
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                return null;
            }
        }   // end "BindLineCommentBlankRow"
        #endregion

        // Usage: SOLine Comment's Comment Changed
        #region "txtSOLineCommText_OnTextChanged:SOLine Comment's Comment TextChanged Event"
        /// <summary>
        /// Handles the OnTextChanged event of the txtSOCommText control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs" /> instance containing the event data.</param>
        protected void txtSOLineCommText_OnTextChanged(Object sender, System.EventArgs e)
        {
            try
            {
                List<Comment> lstCommentEntity = (List<Comment>)ViewState["LineComments"];  // retreiving header comment : why? HeaderComment

                lblCommMessage.Visible = false;

                TextBox txtSOLineCommID = (TextBox)sender;
                GridViewRow row = (GridViewRow)txtSOLineCommID.Parent.Parent;
                int index = row.RowIndex;                   // getting index
                TextBox txtSOLineCommText = grdLineComments.Rows[index].FindControl("txtSOLineCommText") as TextBox;// finding Comment textBox

                ViewState["lineindex"] = index;
                Comment TempCommentE = lstCommentEntity[index];
                TempCommentE.InitializeComments(TempCommentE.ExtCmntID, TempCommentE.ExtCmntKey, txtSOLineCommText.Text, Convert.ToInt32(TempCommentE.ExtCmntUsedKey));
                //TempCommentE.CommentText = txtSOLineCommText.Text;
                //TempCommentE.DocOwnerKey = Convert.ToInt32(ViewState["lblLineKey"]);
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                List<SOLine> lstSOLIne = SOE.LineItems;// (List<SOLine>)Session["ListSoLIne"];
                lstSOLIne[Convert.ToInt32(ViewState["rowindex"])].SOLineComments = lstCommentEntity;

                BindLineCommentToGridView(lstCommentEntity);// binding grid view
                if (lstCommentEntity.Count > 1)
                {
                    grdLineComments.Rows[index + 1].FindControl("txtSOLineCommID").Focus();// setting focus on next row
                }

            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "txtSOLineCommText_OnTextChanged"
        #endregion

        // Usage: Saving SOLine Comment
        #region "btnCommSaveLine_Click: SOLine Comment Saving"
        protected void btnCommSaveLine_Click(Object sender, EventArgs e)
        {
            try
            {
                mpeLineComments.Show();

                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                List<Comment> lstTempComment = new List<Comment>();

                // retreiving value from view state
                List<Comment> lstComment = (List<Comment>)ViewState["LineComments"];
                int flag = 0;
                if (lstComment != null)
                {

                    #region "Checking whether all the rows has valid CommentID or not + Comment or not"
                    // ? Actually, Comment is optional, what if CommentID is valid and Comment is blank? will it goes/pass?

                    for (int i = 0; i < lstComment.Count - 1; i++)// loop from 0 to count-1
                    {
                        Comment TempCommentE = lstComment[i];// assigning row to comment entity
                        if (string.IsNullOrEmpty(TempCommentE.ExtCmntID) || string.IsNullOrEmpty(TempCommentE.CommentText))
                        {
                            flag = 1;       // Means not valid
                            break;
                        }
                        else
                        {

                        }
                        // end "if (str.Length >= 1)"
                    }   // end "for loop"
                    TextBox txtSOLineCommID = (TextBox)grdLineComments.Rows[lstComment.Count - 1].FindControl("txtSOLineCommID");
                    TextBox txtSOLineCommText = (TextBox)grdLineComments.Rows[lstComment.Count - 1].FindControl("txtSOLineCommText");
                    if (txtSOLineCommID.Text.Equals(string.Empty) && txtSOLineCommText.Text.Equals(string.Empty))
                    {

                    }
                    else
                    {
                        flag = 1;
                    }
                    #endregion


                    if (flag == 1)      // for not valid comment condition
                    {
                        lblLineCommentError.Text = "Please add valid comment";
                        lblLineCommentError.ForeColor = System.Drawing.Color.Red;
                        lblLineCommentError.Visible = true;
                        mpeLineComments.Show();
                    }
                    else
                    {
                        mpeLineComments.Hide();

                        ViewState["tempLine"] = SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])].SOLineComments;
                        SetInitialsForLinePopUp();
                        lblDebug.Text = "";
                        lblDebug.Visible = true;
                        upTop.Update();
                    }
                }

                //LinkButton btnLineComment = (LinkButton)webgvSOLineItem.Rows[SOE.LineItems.Count - 1].Items[12].FindControl("btnLineComment");
                //if (btnLineComment != null)
                //{
                //    btnLineComment.Visible = false;
                //}
                // end "if (flag == 0)"
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
            }
        }   // end "btnCommSaveLine_Click"
        #endregion

        /// <summary>
        /// Handles the Click event of the imgLineCmtClose control.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void imgLineCmtClose_Click(object sender, EventArgs e)
        {
            try
            {
                mpeLineComments.Hide();
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                //List<SOLine> lstSOLIne = SOE.LineItems;// (List<SOLine>)Session["ListSoLIne"];
                List<Comment> lstTemp = (List<Comment>)ViewState["tempLine"];
                ViewState["LineComments"] = lstTemp;
                SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])].SOLineComments = lstTemp;
                SOE.CancelSavingLineComment(Convert.ToInt32(ViewState["rowindex"]));
                upSOLine.Update(); upsoheader.Update(); ;
                SetInitialsForLinePopUp();
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;

            }
        }   // end of "imgLineCmtClose_Click"

        // Usage: Cancelling SOLine Comment
        #region "btnCommCancelLine_Click: SOLine Comment Cancel Button Click"
        /// <summary>
        /// Handles the Click event of the btnCommCancelLine control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnCommCancelLine_Click(object sender, EventArgs e)
        {
            try
            {
                mpeLineComments.Hide();
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                //List<SOLine> lstSOLIne = SOE.LineItems;// (List<SOLine>)Session["ListSoLIne"];
                List<Comment> lstTemp = (List<Comment>)ViewState["tempLine"];
                ViewState["LineComments"] = lstTemp;
                SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])].SOLineComments = lstTemp;
                SOE.CancelSavingLineComment(Convert.ToInt32(ViewState["rowindex"]));
                //for (int i = 0; i <= lstTemp.Count - 1; i++)
                //{
                //    for (int j = 0; j <= SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])]._DeletedSOLineComments.Count - 1; j++)
                //    {
                //        if (j <= SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])]._DeletedSOLineComments.Count)
                //        {
                //            if (lstTemp[i].ExtCmntUsedKey == SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])]._DeletedSOLineComments[j].ExtCmntUsedKey)
                //            {
                //                SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])]._DeletedSOLineComments.RemoveAt(j);
                //                break;
                //            }
                //        }
                //    }
                //}
                upSOLine.Update(); upsoheader.Update(); ;
                SetInitialsForLinePopUp();
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "btnCommCancelLine_Click"
        #endregion

        // Usage: ?
        #region "gvSOLineComment Row Command"
        /// <summary>
        /// Handles the RowCommand event of the grdLineComments control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="GridViewCommandEventArgs"/> instance containing the event data.</param>
        protected void grdLineComments_RowCommand(object sender, GridViewCommandEventArgs e)
        {
            try
            {
                if (e.CommandName == "Delete")// For delete operations
                {
                    SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

                    GridViewRow gvr = (GridViewRow)((Control)e.CommandSource).NamingContainer;
                    int rowIndex = gvr.RowIndex;// row index
                    List<Comment> lstComment = (List<Comment>)ViewState["LineComments"];//(List<Comment>)lstSoLine[Convert.ToInt32(ViewState["rowindex"])].SOLineComments;//getting header comment from view state
                    string strSeqNo = string.Empty;

                    Comment TempCommentE = lstComment[rowIndex];// retrving the particular row for delete
                    if (TempCommentE.DocOwnerKey == 0)
                    {
                        SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])].SOLineComments.RemoveAt(rowIndex);// (TempCommentE);
                    }
                    else
                    {
                        SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])].DeleteCommentLine(rowIndex, TempCommentE);
                    }
                    for (int i = 0; i <= SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])].SOLineComments.Count - 1; i++)
                    {
                        SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])].SOLineComments[i].SeqNo = i + 1;
                    }
                    ViewState["LineComments"] = SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])].SOLineComments;
                    BindLineCommentToGridView(SOE.LineItems[Convert.ToInt32(ViewState["rowindex"])].SOLineComments);

                }
                else
                    if (e.CommandName == "commentlookup")// For delete operations
                    {
                        GridViewRow gvr = (GridViewRow)((Control)e.CommandSource).NamingContainer;
                        ViewState["LineCommentRowIndex"] = gvr.RowIndex;
                        BindLookData("LineComment");
                        mpeLookupShipTo.Show();
                        upLookUP.Update();
                        upLineComment.Update();
                    }
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
            }
        }   // end "grdLineComments_RowCommand"
        #endregion

        // Usage: SOLine Comment Modle PopUp Trigger
        #region "grdLineComments_RowDataBound: gvSOLineComment RowDataBound"

        /// <summary>
        /// Handles the RowDataBound event of the grdLineComments control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="GridViewRowEventArgs"/> instance containing the event data.</param>
        protected void grdLineComments_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            try
            {
                if (e.Row.RowType == DataControlRowType.DataRow)
                {
                    if (ViewState["lineindex"] != null)
                    {
                        if (grdLineComments.Rows.Count > Convert.ToInt32(ViewState["lineindex"]))
                        {
                            // for mouse focus
                            grdLineComments.Rows[Convert.ToInt32(ViewState["lineindex"])].FindControl("txtSOLineCommText").Focus();
                        }
                    }
                }
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;

            }
        }   // end "grdLineComments_RowDataBound"


        #endregion
        #endregion

        // SOLine Grid Business 
        // Usage: SOLine GridView's Row Deleting ( Nothing? )
        #region  "Grid View SO LINE Row Deleting"
        protected void gvOrderitem_RowDeleting(object sender, GridViewDeleteEventArgs e)
        {
            // ?
        }   // end "gvOrderitem_RowDeleting"
        #endregion

        #region Get temporary SOLINEKEY
        public string Get8Digits()
        {
            var bytes = new byte[4];
            var rng = RandomNumberGenerator.Create();
            rng.GetBytes(bytes);
            uint random = BitConverter.ToUInt32(bytes, 0) % 100000000;
            return String.Format("{0:D8}", random);
        }
        #endregion
        // Usage: it used
        #region "BindSoLIneItem:Bind SO LINE ITEM"

        /// <summary>
        /// Binds the so L ine item.
        /// </summary>
        /// <param name="lstSoLine">The LST so line.</param>
        private void BindSoLIneItem(List<SOLine> lstSoLine)
        {
            hidUOM.Value = "0";
            hfItemKey.Value = string.Empty;
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            //SOE.LineItems = null;
            SOE.LineItems = lstSoLine;
            try
            {

                if (lstSoLine == null)
                {
                    SOLine SOLineE = new SOLine(SOE);
                    SOE.LineItems.Add(SOLineE);
                }
                else
                if (lstSoLine.Count == 0)
                    {
                        SOLine SOLineE = new SOLine(SOE);
                        SOE.LineItems.Add(SOLineE);
                    }
                else
                if (lstSoLine[lstSoLine.Count - 1].ItemID != string.Empty && lstSoLine[lstSoLine.Count - 1].ItemKey!=0)
                        {
                            SOLine SOLineE = new SOLine(SOE);
                            SOE.LineItems.Add(SOLineE);
                        }

                WDG_DropDownProviderItemID.EditorControl.DataSource = null;
                WDG_DropDownProviderItemID.EditorControl.DataBind();

                webgvSOLineItem.DataSource = null;
                webgvSOLineItem.DataBind();
                webgvSOLineItem.Rows.Clear();

                for (int i = 0; i < SOE.LineItems.Count; i++)
                {
                    int num;
                    bool check = int.TryParse(SOE.LineItems[i].UnitMeasID, out num);
                    if (check)
                    {
                        SOE.LineItems[i].CheckValidUOMID();
                    }
                }

                webgvSOLineItem.DataSource = SOE.LineItems;
                webgvSOLineItem.DataBind();

                // Added by Vinay G on 11/05/2013
                /*
                 * If it is quote then hide status column else make visible it is Sales order.
                */
                if (rbQuote.Checked)
                {
                    this.webgvSOLineItem.Columns[11].Hidden = true;
                    upSOLine.Update();
                }
                else if (rbSO.Checked)
                {
                    this.webgvSOLineItem.Columns[11].Hidden = false;
                    upSOLine.Update();
                }
                //----
                btnRefresh.Text = lstSoLine[lstSoLine.Count - 1].SOLineKey.ToString();

                LinkButton btnLineComment = (LinkButton)webgvSOLineItem.Rows[SOE.LineItems.Count - 1].Items[12].FindControl("btnLineComment");
                if (btnLineComment != null)
                {
                    btnLineComment.Visible = false;
                }
                // txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                upTop.Update();
                upsoheader.Update();
                upSOLine.Update();
                ViewState["SalesOrder"] = null;
                ViewState["SalesOrder"] = SOE;
                // upTop.Update();

            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;
            }
        }   // end "BindSoLIneItem"



        #endregion

        protected void btnDisplayErrorMessage_Click(object sender, EventArgs e)
        {
           SalesOrder SOE= (SalesOrder)ViewState["SalesOrder"];
           BindSoLIneItem(SOE.LineItems);
           mpeShowErrorMessage.Hide();
           upSOLine.Update();
           UpErrorMessage.Update();
          
          }
        #region Click event to bind all the SOLine items
        /// <summary>
        /// Handles the Click event of the btnRefresh control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnRefresh_Click(object sender, EventArgs e)
        {
            //ChangeItem();
        }

        /// <summary>
        /// Changes the item.
        /// </summary>
        private void ChangeItem()
        {
            try
            {
                string[] data = hdfItemIDRowIndex.Value.Split(';');
                if (data.Length > 1)
                {
                    SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                    List<SOLine> lstSoLine = SOE.LineItems;
                    int rowindex = Convert.ToInt32(data[0].ToString());
                    string celltext = data[1].ToString();
                    string datakeyfield = (data[3].ToString());
                    if (lstSoLine != null)
                    {

                        if (datakeyfield == "Status")
                        {
                            if (SOE.LineItems[rowindex].ItemKey != 0)
                            {
                                #region Line Status
                                SOE.LineItems[rowindex].ChangeStatus(celltext);
                                BindSoLIneItem(SOE.LineItems);
                                #endregion
                            }
                            else
                            {
                                BindSoLIneItem(SOE.LineItems);
                            }
                        }
                        else
                            if (datakeyfield == "ItemID")
                            {
                                #region Change Item
                                int rowid = Convert.ToInt32(rowindex.ToString());
                                WDG_DropDownProviderItemID.EditorControl.EnableAjaxViewState = false;
                                WDG_DropDownProviderItemID.EditorControl.EnableViewState = false;
                                WDG_DropDownProviderItemID.EditorControl.DataSource = null;
                                WDG_DropDownProviderItemID.EditorControl.DataBind();

                                //if line is deletable then only we can change the item otherwise we can't
                                if (SOE.LineItems[rowindex].Deletable)
                                {
                                    if (celltext.Length <= 40)
                                    {
                                        if (SOE.LineItems[rowindex].QtyShip > 0)
                                        {
                                           // ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "Shipment already done.Cannot change the item;", true);
                                            lblShowMessage.Text = "Shipment already done.Cannot change the item";
                                            UpErrorMessage.Update();
                                            hfItemKey.Value = string.Empty;
                                            mpeShowErrorMessage.Show();
                                            

                                           

                                        }
                                        else
                                        {
                                            BindItemRow(celltext, 0, rowindex);
                                            //if (hfItemKey.Value.Equals(string.Empty))
                                            //{

                                            //}
                                            //else
                                            //{
                                            //    BindItemRow(celltext, Convert.ToInt32(hfItemKey.Value), rowindex);
                                            //}
                                            hfItemKey.Value = string.Empty;

                                        }
                                    }
                                    else
                                    {
                                        lblShowMessage.Text = "ItemID should be less than 40 chars.";
                                        UpErrorMessage.Update();
                                        mpeShowErrorMessage.Show();
                                    

                                      
                                    }
                                }
                                else
                                {

                                    BindSoLIneItem(SOE.LineItems);
                                }

                                #endregion
                                // GetItemKey(rowid, itemid);
                            }
                            else if (datakeyfield == "Description")
                            {
                                if (SOE.LineItems[rowindex].ItemKey != 0)
                                {
                                    #region Change Desc
                                    if (txtDescProvdr.EditorControl.Text != string.Empty)
                                    {
                                        if (SOE.LineItems[rowindex].Deletable)
                                        {
                                            BindDescRow(celltext, Convert.ToInt32(rowindex));
                                        }
                                        else
                                        {

                                            BindSoLIneItem(SOE.LineItems);
                                        }
                                    }
                                    #endregion
                                }
                                else
                                {

                                    BindSoLIneItem(SOE.LineItems);
                                }
                            }
                            else if (datakeyfield == "QtyOrd")
                            {

                                if (SOE.LineItems[rowindex].ItemKey != 0)
                                {
                                    #region Qty Ord
                                    if (txtQtyProvider.EditorControl.Text != string.Empty)
                                    {
                                        SOLine SOLineItemE = lstSoLine[Convert.ToInt32(rowindex)];
                                        if (Convert.ToDecimal(celltext) < SOLineItemE.QtyShip)
                                        {
                                            if (SOLineItemE.ItemType != 2)
                                            {
                                               // ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('Quantity order should not be less than quantity shipped.');", true);

                                                lblShowMessage.Text = "Quantity order should not be less than quantity shipped.";
                                                UpErrorMessage.Update();
                                                mpeShowErrorMessage.Show();
                                               
                                                // lblDebug.Text = "Quantity order should not be less than quantity shipped.";
                                                //lblDebug.Visible = true;
                                            }
                                            else
                                            {
                                                decimal qty = Convert.ToDecimal(celltext);
                                                BindQtyRow(qty, Convert.ToInt32(rowindex), SOE);
                                            }
                                        }
                                        else
                                        {
                                            if (SOLineItemE.UnitPriceOvrd)
                                            {
                                                ViewState["ChkQtyUOM"] = "qty";
                                                ViewState["qtyordrowindex"] = Convert.ToDecimal(celltext) + "~" + Convert.ToInt32(rowindex);
                                                mpeCheckOvrrdnPrice.Show();

                                            }
                                            else
                                            {
                                                decimal qty = Convert.ToDecimal(celltext);
                                                // BindQtyRow(qty, Convert.ToInt32(rowindex), SOE);

                                                if (qty <= 0)
                                                {
                                                    lblShowMessage.Text = "Quantity cannot be zero.";
                                                    UpErrorMessage.Update();
                                                    mpeShowErrorMessage.Show();
                                                   
                                                   // ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('Quantity cannot zero.');", true);
                                                    //BindQtyRow(1, Convert.ToInt32(rowindex), SOE);
                                                }
                                                else
                                                {
                                                    BindQtyRow(qty, Convert.ToInt32(rowindex), SOE);
                                                }
                                            }
                                        }
                                    }
                                    #endregion
                                }
                                else
                                {
                                    BindSoLIneItem(SOE.LineItems);
                                }

                            }
                            else if (datakeyfield == "UnitMeasID")
                            {
                                if (SOE.LineItems[rowindex].ItemKey != 0)
                                {
                                    try
                                    {
                                        ddp_UnitMeasID.EditorControl.DataSource = null;
                                        ddp_UnitMeasID.EditorControl.DataBind();
                                        ddp_UnitMeasID.EditorControl.EnableAjaxViewState = false;
                                        ddp_UnitMeasID.EditorControl.EnableViewState = false;
                                        #region Change UOM
                                        if (SOE.LineItems[rowindex].Deletable)
                                        {
                                            if (hidUOM.Value.ToString().Equals("0"))
                                            {
                                               // ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('Invalid Entry.');", true);
                                                lblShowMessage.Text = "Invalid Entry..";
                                                UpErrorMessage.Update();
                                                mpeShowErrorMessage.Show();
                                            
                                            }
                                            else
                                            {
                                                if (SOE.LineItems[rowindex].QtyShip > 0)
                                                {
                                                   // ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('This line item is already shipped.Not allowed to change UOM.');", true);
                                                    //ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "This line item is already shipped.Not allowed to change UOM.", true);
                                                    lblShowMessage.Text = "This line item is already shipped.Not allowed to change UOM.";
                                                    UpErrorMessage.Update();
                                                    mpeShowErrorMessage.Show();
                                                 
                                                    //lblDebug.Text = "This line item is already shipped.Not allowed to change UOM.";
                                                    //lblDebug.Visible = true;
                                                }
                                                else
                                                {
                                                    if (SOE.LineItems[rowindex].UnitPriceOvrd)
                                                    {
                                                        ViewState["ChkQtyUOM"] = "uom";
                                                        ViewState["qtyordrowindex"] = (celltext) + "~" + Convert.ToInt32(rowindex);
                                                        mpeCheckOvrrdnPrice.Show();

                                                    }
                                                    else
                                                    {
                                                        // GetUOMKey(rowindex.ToString(), data[1]);
                                                        BindUOMRow(celltext, Convert.ToInt32(hidUOM.Value), Convert.ToInt32(rowindex));
                                                        // BindSoLIneItem(SOE.LineItems);
                                                        hidUOM.Value = "0";
                                                    }
                                                }

                                            }
                                        }
                                        else
                                        {
                                           // ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('This line is not changeable.');", true);
                                            //
                                            lblShowMessage.Text = "This line is not changeable.";
                                            UpErrorMessage.Update();
                                            mpeShowErrorMessage.Show();
                                            
                                            //lblDebug.Text = "This line is not changeable.";
                                            //lblDebug.Visible = true;
                                        }
                                        #endregion
                                    }
                                    catch (Exception exe)
                                    {

                                        BindSoLIneItem(SOE.LineItems);
                                    }
                                }
                                else
                                {
                                    BindSoLIneItem(SOE.LineItems);
                                }
                            }
                            else if (datakeyfield == "UnitPrice")
                            {
                                if (SOE.LineItems[rowindex].ItemKey != 0)
                                {
                                    #region Change Unit Price
                                    if (SOE.LineItems[rowindex].Deletable)
                                    {
                                        BindUnitPriceRow(Convert.ToDecimal(celltext), Convert.ToInt32(rowindex));
                                    }
                                    else
                                    {
                                        BindSoLIneItem(SOE.LineItems);
                                    }
                                    #endregion
                                }
                                else
                                {
                                    BindSoLIneItem(SOE.LineItems);
                                }
                            }
                            else if (datakeyfield == "TradeDiscAmt")
                            {
                                if (SOE.LineItems[rowindex].ItemKey != 0)
                                {
                                    #region TradeDiscAmt
                                    Decimal tradedisc = Convert.ToDecimal(celltext.Replace("$", ""));
                                    if (SOE.LineItems[rowindex].Deletable)
                                    {
                                        if (tradedisc >= 0)
                                        {

                                            string str = SOE.LineItems[rowindex].CalcTradeDiscPercent(tradedisc);
                                            SOE.CalculateSOTotal();
                                            txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                                            BindSoLIneItem(SOE.LineItems);
                                            if (str != string.Empty)
                                            {
                                               // ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('" + str + "');", true);
                                                lblShowMessage.Text = str;
                                                mpeShowErrorMessage.Show();
                                            }
                                        }
                                        else
                                        {
                                            SOE.CalculateSOTotal();
                                            txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                                            BindSoLIneItem(SOE.LineItems);
                                        }
                                    }
                                    else
                                    {
                                        BindSoLIneItem(SOE.LineItems);
                                    }

                                    #endregion
                                }
                                else
                                {
                                    BindSoLIneItem(SOE.LineItems);
                                }

                            }
                            else if (datakeyfield == "TradeDiscPct")
                            {
                                if (SOE.LineItems[rowindex].ItemKey != 0)
                                {
                                    #region Trade Disc Percnt

                                    if (SOE.LineItems[rowindex].Deletable)
                                    {
                                        //if (tradediscpct >= 0 && tradediscpct <= 100)
                                        //{
                                        if (Convert.ToDecimal(celltext) <= 100)
                                        {
                                            Decimal tradediscpct = Convert.ToDecimal(celltext) / 100;
                                            string str = SOE.LineItems[rowindex].CalcTradeDiscountAmt(tradediscpct);
                                            SOE.CalculateSOTotal();
                                            txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                                            BindSoLIneItem(SOE.LineItems);
                                            if (str != string.Empty)
                                            {
                                               // ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('" + str + "');", true);
                                                lblShowMessage.Text = str;
                                                mpeShowErrorMessage.Show();
                                            }
                                        }
                                        else
                                        {
                                            //ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('Trade disc percent should be less than or equal to 100.');", true);
                                            lblShowMessage.Text = "Trade disc percent should be less than or equal to 100.";
                                            UpErrorMessage.Update();
                                            mpeShowErrorMessage.Show();
                                           
                                        }
                                    }
                                    else
                                    {
                                        BindSoLIneItem(SOE.LineItems);
                                    }
                                    #endregion
                                }
                                else
                                {
                                    BindSoLIneItem(SOE.LineItems);
                                }
                            }
                            else
                            {
                                BindSoLIneItem(SOE.LineItems);
                            }


                    }

                    if (hfCheckEdit.Value.Equals("1"))
                    {
                        SOE.IsSOChanged = true;
                        SOE.IsSOChang = true;
                    }
                    if (ViewState["result"] != null)
                    {
                        SOE.IsSOChang = true;
                    }
                    hdfItemIDRowIndex.Value = "0";

                }
            }
            catch (Exception Ex)
            {
                lblDebug.Text = Ex.Message;
                lblDebug.Visible = true;
            }
        }

        /// <summary>
        /// Checks the line status.
        /// </summary>
        /// <param name="SOLE">The SOLE.</param>
        /// <returns></returns>
        private bool CheckLineStatus(SOLine SOLE)
        {
            if (SOLE.Status == 0)
            {
                return false;
            }
            else
            {
                return true;
            }
        }
        #endregion
        /// <summary>
        /// Handles the Click event of the btnDisableAllControls control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnDisableAllControls_Click(object sender, EventArgs e)
        {
            DisableAllControls();
        }
        #region Binding UnitPrice
        /// <summary>
        /// Binds the unit price row.
        /// </summary>
        /// <param name="unitprice">The unitprice.</param>
        /// <param name="rowindex">The rowindex.</param>
        private void BindUnitPriceRow(decimal unitprice, int rowindex)
        {
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

                string val = SOE.LineItems[rowindex].ChangeUnitPrice(unitprice);

                if (val != string.Empty)
                {
                    lblShowMessage.Text = val;
                    UpErrorMessage.Update();
                    mpeShowErrorMessage.Show();
                    //ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('" + val + "');", true);
                }
                else
                {
                    SOE.CalculateSOTotal();
                    txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                    BindSoLIneItem(SOE.LineItems);
                    upSOLine.Update();

                    upSOLine.Update();
                }


            }
            catch (Exception ex)
            {
                lblMessage.Text = ex.Message;
            }
        }
        #endregion

        #region Description of Item
        /// <summary>
        /// Binds the desc row.
        /// </summary>
        /// <param name="desc">The desc.</param>
        /// <param name="index">The index.</param>
        private void BindDescRow(string desc, int index)
        {
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                SOE.LineItems[index].ChangeItemDescription(desc);
                BindSoLIneItem(SOE.LineItems);

            }
            catch (Exception ex)
            {
                lblMessage.Text = ex.Message;
            }
        }
        #endregion

        #region Bind Whse ROW
        private void BindWHSERow(string WhseID, string WhseKey, int rowindex)
        {

        }
        #endregion



        #region Bind UOM Row
        /// <summary>
        /// Binds the UOM row.
        /// </summary>
        /// <param name="UOMID">The UOMID.</param>
        /// <param name="UOMKey">The UOM key.</param>
        /// <param name="rowindex">The rowindex.</param>
        private void BindUOMRow(string UOMID, int UOMKey, int rowindex)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            try
            {
                
                SOE.LineItems[rowindex].ChangeUOM(UOMKey, UOMID, SOE.CustKey);

               // MSISalesOrderAPI MSISOE = new MSISalesOrderAPI();
               // DataTable dtUOM = MSISOE.GetLookupList("", "", MSISalesOrderAPI.LookupType.UnitOfMeasurement, "", SOE.LineItems[rowindex].ItemKey);

                //ddp_UnitMeasID.EditorControl.DataSource = null;
                //ddp_UnitMeasID.EditorControl.DataSource = dtUOM;

                //ddp_UnitMeasID.EditorControl.TextField = "UnitMeasID";
                //ddp_UnitMeasID.EditorControl.ValueField = "UnitMeasID";
                //ddp_UnitMeasID.EditorControl.DataBind();

                SOE.CalculateSOTotal();
                txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                BindSoLIneItem(SOE.LineItems);
                
                upSOLine.Update();

            }
            catch (Exception exe)
            {
                //ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "This unit of measure is not valid for this item.", true);
                lblShowMessage.Text = "This unit of measure is not valid for this item.";
                mpeShowErrorMessage.Show();

                //ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('This unit of measure is not valid for this item.');", true);
               

            }
        }
        private void BindUOMRow_OverridenPrice(string UOMID, int UOMKey, int rowindex)
        {
        }
        #endregion

        #region Bind Qty ROW
        /// <summary>
        /// Binds the qty row.
        /// </summary>
        /// <param name="QtyOrd">The qty ord.</param>
        /// <param name="rowIndex">Index of the row.</param>
        /// <param name="SOE">The SOE.</param>
        private void BindQtyRow(Decimal QtyOrd, int rowIndex, SalesOrder SOE)
        {
            try
            {

                SOE.LineItems[rowIndex].ChangeQty(Convert.ToInt32(QtyOrd), SOE.CustKey, SOE._ShipToAddress.CustAddrKey);
                SOE.CalculateSOTotal();
                txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                BindSoLIneItem(SOE.LineItems);



            }
            catch (Exception ex)
            {
                lblDebug.Text = ex.Message;
                lblDebug.Visible = true;
            }

        }
        #endregion
        /// <summary>
        /// Handles the Click event of the btnRecentSales control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnRecentSales_Click(object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            int itemkey = SOE.LineItems[Convert.ToInt32(hidLineIndex.Value)].ItemKey;
            if (itemkey != 0)
            {
                DataTable dtRecentSales = SOE.RecentSales(itemkey);
                if (dtRecentSales != null)
                {
                    lblRCustID.Text = SOE.CustID;
                    lblRCustName.Text = SOE._BillToAddress.AddrName;
                    lblRAddrName.Text = SOE._ShipToAddress.AddrName;
                    lblRAddrLine.Text = SOE._ShipToAddress.AddrLine1;
                    lblRItemID.Text = SOE.LineItems[Convert.ToInt32(hidLineIndex.Value)].ItemID;
                    lblRItemDesc.Text = SOE.LineItems[Convert.ToInt32(hidLineIndex.Value)].Description;
                    grdrecentSales.DataSource = dtRecentSales;
                    grdrecentSales.DataBind();
                    upSOLine.Update();
                    upRecentSales.Update();
                    mpeRecenSales.Show();
                }
            }

        }
        /// <summary>
        /// Handles the Click event of the btnUOM control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnUOM_Click(object sender, EventArgs e)
        {
            /*SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            int itemkey = SOE.LineItems[Convert.ToInt32(hdfRowindexUOM.Value)].ItemKey;
            if (itemkey != 0)
            {
                MSISalesOrderAPI MSISOE = new MSISalesOrderAPI();
                ddp_UnitMeasID.EditorControl.DataSource = MSISOE.GetLookupList("", "", MSISalesOrderAPI.LookupType.UnitOfMeasurement, "", itemkey);//
                ddp_UnitMeasID.EditorControl.TextField = "UnitMeasID";
                ddp_UnitMeasID.EditorControl.ValueField = "UnitMeasKey";
                ddp_UnitMeasID.EditorControl.DataBind();
               

                //hdfCheckUOM.Value = hdfRowindexUOM.Value;
                //BindSoLIneItem(SOE.LineItems);

                //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing.ColumnSettings["UnitMeasID"].ReadOnly = false;
            }*/
            // ddp_UnitMeasID.EditorControl.
            //webgvSOLineItem.Behaviors.EditingCore.Behaviors.CellEditing..EditModeActions..EnableOnActive = true;
        }

        #region Bind Item Row
        /// <summary>
        /// Binds the item row.
        /// </summary>
        /// <param name="ItemID">The item ID.</param>
        /// <param name="ItemKey">The item key.</param>
        /// <param name="rowindex">The rowindex.</param>
        private void BindItemRow(string ItemID, int ItemKey, int rowindex)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            try
            {
                 string message= SOE.LineItems[rowindex].ChangeItem(ItemKey, ItemID.ToUpper(), SOE._ShipToAddress.CustAddrKey, strCompanyID, SOE.DfltWhseKey);

                 if (message.Equals("1"))
                 {
                     MSISalesOrderAPI MSISOE = new MSISalesOrderAPI();
                     DataTable dtUOM = MSISOE.GetLookupList("", "", MSISalesOrderAPI.LookupType.UnitOfMeasurement, "", SOE.LineItems[rowindex].ItemKey);

                     ddp_UnitMeasID.EditorControl.DataSource = dtUOM;

                     ddp_UnitMeasID.EditorControl.TextField = "UnitMeasID";
                     ddp_UnitMeasID.EditorControl.ValueField = "UnitMeasKey";
                     ddp_UnitMeasID.EditorControl.DataBind();

                     txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                     ViewState["SalesOrder"] = null;
                     ViewState["SalesOrder"] = SOE;
                     BindSoLIneItem(SOE.LineItems);
                     SOE.CalculateSOTotal();
                 }
                 else
                 {
                     lblShowMessage.Text = message;

                     UpErrorMessage.Update();

                     mpeShowErrorMessage.Show();
                 }

           
            }
            catch (Exception exe)
            {
                lblShowMessage.Text = exe.Message;
                UpErrorMessage.Update();
                mpeShowErrorMessage.Show();
                //ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('" + exe.Message + "');", true);
               // BindSoLIneItem(SOE.LineItems);
            }
        }
        #endregion
        //[System.Web.Services.WebMethod]
        #region Get Item Key
        public void GetItemKey(string rowID, string ItemID)
        {

        }
        #endregion

        #region Get UOM KEY
        public void GetUOMKey(string rowID, string UOMID)
        {


        }
        public int GetUOMKey_Overriden(string rowID, string UOMID)
        {

            return 0;//For tym being i have written

        }
        #endregion

        #region Get WHSE KEy
        public void GetWHSEKey(string rowID, string WHSEID)
        {


        }

        #endregion

        /// <summary>
        /// Handles the Click event of the loadgrid control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void loadgrid_Click(Object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            BindSoLIneItem(SOE.LineItems);
        }

        #region Cancel Overriden Button Click
        /// <summary>
        /// Cancel Overriden Button Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnCancelOvrridn_Click(Object sender, EventArgs e)
        {
            try
            {

                if (ViewState["ChkQtyUOM"] != null)
                {
                    if (ViewState["ChkQtyUOM"].ToString() == "qty")
                    {
                        string strQty_rowindex = ViewState["qtyordrowindex"].ToString();
                        string[] arrQty_rowindex = strQty_rowindex.Split('~');
                        int Qty = Convert.ToInt32(Convert.ToDecimal(arrQty_rowindex[0]));
                        int rowindex = Convert.ToInt32(arrQty_rowindex[1]);
                        SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                        SOE.LineItems[rowindex].GetItemPrice(Qty);
                        BindSoLIneItem(SOE.LineItems);

                    }
                    else if (ViewState["ChkQtyUOM"].ToString() == "uom")
                    {
                        string struom_rowindex = ViewState["qtyordrowindex"].ToString();
                        string[] arrUOM_rowindex = struom_rowindex.Split('~');
                        string UOMID = (arrUOM_rowindex[0]).ToString();
                        int rowindex = Convert.ToInt32(arrUOM_rowindex[1]);

                        SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                        SOE.LineItems[rowindex].ChangeUOM(Convert.ToInt32(ViewState["UOMKey"].ToString()), UOMID, SOE.CustKey);
                        BindSoLIneItem(SOE.LineItems);

                    }
                }
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;
            }
        }

        #endregion


        /// <summary>
        /// Handles the Click event of the btnMoveLineUP control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnMoveLineUP_Click(Object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (!hidLineIndex.Value.ToString().Equals(string.Empty))
            {
                int lineindex = Convert.ToInt32(hidLineIndex.Value);
                int linecount = SOE.LineItems.Count;
                if (lineindex > 0 && lineindex < SOE.LineItems.Count-1)
                {
                    SOE.MoveLineUp(lineindex);
                    BindSoLIneItem(SOE.LineItems);
                }
                hidLineIndex.Value = string.Empty;
            }
        }
        /// <summary>
        /// Handles the Click event of the btnMoveLineDOWN control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnMoveLineDOWN_Click(Object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (!hidLineIndex.Value.ToString().Equals(string.Empty))
            {
                int lineindex = Convert.ToInt32(hidLineIndex.Value);
                int linecount = SOE.LineItems.Count;
                if (lineindex >= 0 &&(lineindex+1!=SOE.LineItems.Count-1))
                {
                    SOE.MoveLineDown(lineindex);
                    BindSoLIneItem(SOE.LineItems);
                }
                hidLineIndex.Value = string.Empty;
            }
        }

        /// <summary>
        /// Handles the Click event of the btnInsertCurrent control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnInsertCurrent_Click(Object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (!hidLineIndex.Value.ToString().Equals(string.Empty))
            {
                int lineindex = Convert.ToInt32(hidLineIndex.Value);
                int linecount = SOE.LineItems.Count;
                if (lineindex != SOE.LineItems.Count - 1)
                {
                    SOE.InsertLine(lineindex);
                    BindSoLIneItem(SOE.LineItems);
                }
                hidLineIndex.Value = string.Empty;
            }
        }
        #region Overriden button ok Click
        /// <summary>
        ///  Overriden button ok Click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnOKOvrridn_Click(Object sender, EventArgs e)
        {
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                if (ViewState["ChkQtyUOM"] != null)
                {
                    if (ViewState["ChkQtyUOM"].ToString() == "qty")
                    {
                        string strQty_rowindex = ViewState["qtyordrowindex"].ToString();
                        string[] arrQty_rowindex = strQty_rowindex.Split('~');
                        int QtyOrd = Convert.ToInt32(Convert.ToDecimal(arrQty_rowindex[0].ToString()));
                        int rowIndex = Convert.ToInt32(arrQty_rowindex[1]);
                        SOE.LineItems[rowIndex].ChangeQty(QtyOrd, SOE.CustKey, SOE._ShipToAddress.CustAddrKey);
                        SOE.CalculateSOTotal();
                        txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                        BindSoLIneItem(SOE.LineItems);

                    }
                    else if (ViewState["ChkQtyUOM"].ToString() == "uom")
                    {
                        string strUOM_rowindex = ViewState["qtyordrowindex"].ToString();
                        string[] arrUOM_rowindex = strUOM_rowindex.Split('~');
                        string UOM = arrUOM_rowindex[0];
                        int rowIndex = Convert.ToInt32(arrUOM_rowindex[1]);
                        SOE.LineItems[rowIndex].ChangeUOMForOveriddenPrice(Convert.ToInt32(hidUOM.Value), UOM);
                        hidUOM.Value = "0";
                        SOE.CalculateSOTotal();
                        txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                        BindSoLIneItem(SOE.LineItems);
                    }
                }
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;
            }

        }

        #endregion

        #region Bind QTY with overriden Price
        /// <summary>
        /// Bind QTY with overriden Price
        /// </summary>
        private void BindQtyWithOverddnPrice()
        {

        }
        #endregion

        // Usage: Collecting Each SOLine's Values for total? ( TotalAmt, TotalTradeDiscountAmt, STax? )
        protected void webgvSOLineItem_FixStateChange(object sender, FixedStateChangedEventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            BindSoLIneItem(SOE.LineItems);
        }

        #region Line Gridview Initialize row event
        Decimal Price = 0;

        /// <summary>
        /// Handles the InitializeRow event of the webgvSOLineItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Infragistics.Web.UI.GridControls.RowEventArgs"/> instance containing the event data.</param>
        protected void webgvSOLineItem_InitializeRow(object sender, Infragistics.Web.UI.GridControls.RowEventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            try
            {

                if (!e.Row.Items[16].Text.Equals(string.Empty))
                {
                    e.Row.Items[5].Tooltip = e.Row.Items[16].Text;

                }
                LinkButton btnLineComment = (LinkButton)e.Row.Items[13].FindControl("btnLineComment");
                if ((e.Row.Items[11].Text.Equals("1")) || (e.Row.Items[11].Text.Equals("OPEN")))
                {
                    e.Row.Items[11].Text = "OPEN";

                    if (ddStatus.SelectedValue == "4" || ddStatus.SelectedValue == "3")
                    {

                        e.Row.CssClass = "GridRowColor";
                        e.Row.Items[11].Text = "CLOSED";
                        btnLineComment.Visible = false;
                        e.Row.Items[1].FindControl("imgbtnLookupSOLINE").Visible = false;
                    }
                    else
                    {
                        if (btnLineComment != null)
                        {
                            //e.Row.Items[12].FindControl("btnLineComment").Visible = true;
                            btnLineComment.Visible = true;
                            e.Row.Items[1].FindControl("imgbtnLookupSOLINE").Visible = true;
                        }
                    }

                }
                else
                {

                    e.Row.CssClass = "GridRowColor";
                    e.Row.Items[11].Text = "CLOSED";
                    btnLineComment.Visible = false;
                    e.Row.Items[1].FindControl("imgbtnLookupSOLINE").Visible = false;

                }


            }
            catch (Exception ex)
            {
                //Tracing Exception
                lblMessage.Text = ex.Message;
            }
        }
        #endregion

        #region "webgvSOLineItem_OnRowDataBound:Grid View SOLINE Row Data Bound"
        /// <summary>
        /// Handles the RowIslandDataBinding event of the webgvSOLineItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="GridViewRowEventArgs"/> instance containing the event data.</param>
        protected void webgvSOLineItem_RowIslandDataBinding(Object sender, GridViewRowEventArgs e)
        {
            double totalamt = 0.00; int qty = 0; double unit = 0.00; double amt = 0.00;

            try
            {
                if (e.Row.RowType == DataControlRowType.DataRow)
                {
                    // Finding Quantity Order Textbox
                    TextBox txtQtyOrdered = (TextBox)e.Row.FindControl("txtQtyOrdered");

                    string str = txtQtyOrdered.Text.Trim();
                    if (str.Length > 0)
                    {
                        // If Row Contains Minus then i m making it false
                        if (str[0].ToString().Equals("-"))
                        {
                            e.Row.Visible = false;
                        }
                        else
                        {

                        }
                    }
                }
                if (totalamt != 0)
                {
                    Session["totalamt"] = Convert.ToString(totalamt);
                }
            }
            catch (Exception ex)
            {
                //Tracing Exception
                lblMessage.Text = ex.Message;
            }
        }   // end "webgvSOLineItem_OnRowDataBound"



        #endregion

        // Usage: No Usage
        #region "grdLineComments_RowDeleting:gvSOLineComment Row Deleting"
        protected void grdLineComments_RowDeleting(object sender, GridViewDeleteEventArgs e)
        {
            // ?
        }
        #endregion

        // Usage: Click on 'Delete' Button at SOLine GridView Works/Actions
        #region "webgvSOLineItem_OnRowCommand: SOLINE GridView's Row Command"
        protected void webgvSOLineItem_OnRowCommand(Object sender, GridViewCommandEventArgs e)
        {
        }   // end "webgvSOLineItem_OnRowCommand"
        #endregion

        // Usage: UnitPrice Generation Mechanism: ItemID, QtyOrdered, UOM, and WhseID


        // Usage: Whole SO Saving
        // Correction: Checking/Validation before saving. Whether things are complete or not.
        // Not found SOHeader Saving + SOHeader Comment Saving.
        // Final SOHeader Saving ( with STax and Values updating is not done yet for DB )

        #region "Whole SO Save Button Click"
        /// <summary>
        /// Handles the Click event of the btnOkChangeOrder control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnOkChangeOrder_Click(Object sender, EventArgs e)
        {
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                if (!txtChanreason.Text.ToString().Equals(string.Empty))
                {

                    SOE.ChangeOrder(txtChanreason.Text, getUserID());
                    SaveSO();
                    if (ViewState["finish"] != null)
                    {
                        Response.Redirect(Request.RawUrl);
                    }
                }
                else
                {
                    mpeChnageorder.Show();
                }


            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
            }
        }

        #endregion
        #region Saving SO HEADER
        /// <summary>
        /// Saving SO HEADER
        /// </summary>
        private void SaveSOHeader()
        {
            // Stan_Note: why CustPONo, Hold, Req/Ship/Prom date are here, all should be at SO Object, isn't it?
            try
            {


            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;
            }
        }
        #endregion

        #region Save SO HEADER Comment
        private void SaveSOHEaderComment()
        {


            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];


        }
        #endregion
        // Usage: ? SOLine Data Committing? Delete/Save/Update?
        // To Honor, Mr. Manu, we have new word: Manupulate. Bravo!
        // Correction: Where is SOHeader Saving?
        #region "SaveSOLineItem SOLINE Item"
        private void SaveSOLineItem()
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

        }   // end "ManupulateSOLineItem"
        #endregion


        // Usage: Cancel SO Entry anytime.
        #region "btnCancel_Click"
        /// <summary>
        /// Handles the Click event of the btnCancel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnCancel_Click(object sender, EventArgs e)
        {
            try
            {


                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                if (hfCheckEdit.Value.ToString().Equals("1"))
                {

                }
                if (Session["SessionKey"] != null)
                {
                    if (SOE != null)
                    {
                        SOE.ReleaseLock(Session["SessionKey"].ToString());
                    }
                }
                ViewState["SalesOrder"] = null;
                ViewState["NewComment"] = null;
                ViewState["rowindex"] = null;
                ViewState["LineComments"] = null;
                Response.Redirect(Request.RawUrl);
            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;
            }
        }   // end "btnCancel_Click"
        #endregion

        // Usage: Save Whole SO and Clear Screen for next Action.
        #region "btnFinish_Click"
        /// <summary>
        /// Handles the Click event of the btnFinish control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnFinish_Click(object sender, EventArgs e)
        {
            /*
             * Call SaveSO
             * Release SOEdit
             * All Thing in BrandNew
             */
            FinishSO();

        }

        /// <summary>
        /// Finishes the SO.
        /// </summary>
        private void FinishSO()
        {
            try
            {

                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

                ViewState["finish"] = true;

                if (rbSO.Checked)//For Sales Order
                {
                    if (SOE.IsSOChang)
                    {

                        txtChangeNo.Text = (SOE.ChngOrdNo + 1).ToString();
                        upTop.Update();
                        txtChanreason.Text = "";
                        mpeChnageorder.Show();


                    }
                    else
                    {

                        SaveSO();
                    }
                }
                else//For Quote
                {
                    SaveSO();
                }


                //ViewState["SalesOrder"] = null;
                //Session["CustEntity"] = null;
                //ViewState["NewComment"] = null;
                //ViewState["rowindex"] = null;
                //Session["ListSoLIne"] = null;
                //ViewState["LineComments"] = null;
                // hfCheckEdit.Value = "0";
                //Response.Redirect(Request.RawUrl);

            }
            catch (Exception exe)
            {

                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;
            }
            // Save SO
            // And Clear All Fields like Page_Load
        }   // end "btnFinish_Click"
        #endregion

        // Usage: ?
        #region SOLINE row Updating
        /// <summary>
        /// SOLINE row Updating
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void webgvSOLineItem_OnRowUpdating(Object sender, GridViewUpdateEventArgs e)
        {
            // ?
        }   // end "webgvSOLineItem_OnRowUpdating"
        #endregion
        // Usage: ?/Unknown.
        #region "CompanyID Changed"
        /// <summary>
        /// Companies the changed.
        /// </summary>
        /// <param name="CompanyID">The company ID.</param>
        public void CompanyChanged(string CompanyID)
        {
            strCompanyID = CompanyID;
        }   // end "CompanyChanged"
        #endregion

        // Usage: 
        #region "ValueEnableDisable: making enable/disable objects"
        /// <summary>
        /// Values the enable disable.
        /// </summary>
        protected void ValueEnableDisable()
        {
            if (this.txtSalesOrder.Text != string.Empty)
            {
                this.btnHeaderCom.Enabled = true;
                this.btnSalesOrder.Enabled = true;
                this.txtSalesOrder.Enabled = false;
                this.btnCreate.Enabled = false;

                if (txtCustomer.Text != string.Empty)
                {
                    try
                    {
                        //Customer Cust = new Customer();



                        //Assigning cust name
                        txtBillToCustName.Enabled = false;
                        txtBillTo.Enabled = false;

                        txtBillToName.Enabled = false;
                        // Assigning Address line 
                        txtBillToAddr1.Enabled = false;
                        txtBillToAddr2.Enabled = false;
                        txtBillToAddr3.Enabled = false;
                        txtBillToAddr4.Enabled = false;
                        txtBillToAddr5.Enabled = false;
                        // Assigning Address Key

                        txtBillToAddrCity.Enabled = false;
                        txtBillToPostalCode.Enabled = false;
                        txtBillToCountry.Enabled = false;
                        // binding state From Function to drop down

                        txtBillToState.Enabled = false;

                    }
                    catch (Exception exe)
                    {
                        lblDebug.Text = exe.Message;

                    }

                    rbQuote.Enabled = false;
                    rbSO.Enabled = false;
                    chkHold.Enabled = true;
                    txtCommPlan.Enabled = true;
                    txtPromised.Enabled = true;
                    txtSchedShip.Enabled = true;
                    txtrequest.Enabled = true;
                    btnFinish.Enabled = true;
                    ddStatus.Enabled = true;
                    this.btnSave.Enabled = true;
                    this.btnCancel.Enabled = true;

                    this.webgvSOLineItem.Visible = true;

                    txtShipTo.Enabled = true;
                    this.txtOrderDate.Enabled = false;
                    this.lblMessage.Visible = false;
                    //this.btnCustomer.Enabled = true;
                    this.btnBillTo.Enabled = true;
                    this.btnShipTo.Enabled = true;


                    this.imgLookupShpTo.Enabled = true;
                    this.btnLookupCommPlan.Enabled = true;
                    imgbtnCustClass.Enabled = true;
                    imgbtnWareHouse.Enabled = true;
                    imgbtnShipVia.Enabled = true;
                    txtHeadShipVia.Enabled = true;
                    txtHeadWarehouse.Enabled = true;
                    txtCustomerPO.Enabled = true;


                    DataTable dtStatus = new DataTable();
                    dtStatus.Columns.Add("Status", typeof(string));
                    dtStatus.Columns.Add("Value", typeof(string));
                    dtStatus.Rows.Add("OPEN", "1");// typeof(String));
                    dtStatus.Rows.Add("CLOSED", "2");
                    ddpStatus.EditorControl.DataSource = dtStatus;
                    ddpStatus.EditorControl.TextField = "Status";
                    ddpStatus.EditorControl.ValueField = "Value";
                    ddpStatus.EditorControl.DataBind();
                }
                else
                {
                    txtCustomer.Enabled = true;
                    txtCustomer.Focus();

                    PageDefault();
                }
            }
            else
            {   // SONumber Text Box is empty
                PageDefault();
            }
        }   // end "ValueEnableDisable"
        #endregion


        // Usage: UI Page Default Setting ( which fields on/off and visibilty )
        #region "PageDefault: Setting Default Value of Page"
        protected void PageDefault()
        {
            this.btnFinish.Enabled = false;
            this.btnSave.Enabled = false;
            this.btnCancel.Enabled = true;

            this.btnCreate.Enabled = true;
            this.txtSalesOrder.Enabled = true;
            this.lblMessage.Visible = false;
            this.btnHeaderCom.Enabled = false;
            this.btnShipTo.Enabled = false;
            this.btnBillTo.Enabled = false;
            this.btnSalesOrder.Enabled = true;
            this.imgBtnCustmer.Enabled = false;
            //this.pnlOrderTotal.Enabled = false;


            this.imgLookupShpTo.Enabled = false;
            this.btnLookupCommPlan.Enabled = false;
            imgbtnCustClass.Enabled = false;

            // ? what is that for?
            webgvSOLineItem.DataSource = null;
            webgvSOLineItem.DataBind();
            webgvSOLineItem.Visible = false;
            chkHold.Enabled = false;
            txtHold.Enabled = false;


        }   // end "PageDefault"
        #endregion

        protected void ddServerFiltering_PreRender(object sender, EventArgs e)
        {

            // wddItem.CurrentValue = "";
        }

        #region Header Hold and its commnet checkbox
        /// <summary>
        /// Header Hold and its comment checkbox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void chkHold_CheckedChanged(object sender, EventArgs e)
        {
            if (chkHold.Checked)
            {
                txtHold.Enabled = true;
            }
            else
            {
                txtHold.Enabled = false;
                txtHold.Text = string.Empty;
            }
        }
        #endregion

        #region ROW DELETing of so-line
        /// <summary>
        /// Handles the RowsDeleting event of the webgvSOLineItem control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="RowDeletingEventArgs"/> instance containing the event data.</param>
        protected void webgvSOLineItem_RowsDeleting(object sender, RowDeletingEventArgs e)
        {
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                int LineNo = Convert.ToInt32(e.Row.DataKey[0]);
                List<SOLine> lstSoLine = SOE.LineItems;// (List<SOLine>)Session["ListSoLIne"];
                if (LineNo != SOE.LineItems.Count)
                {
                    int solinekey = Convert.ToInt32(lstSoLine[LineNo - 1].SOLineKey);
                    bool isDelete = Convert.ToBoolean(lstSoLine[LineNo - 1].Deletable);

                    if (solinekey == 0)
                    {
                        //lstSoLine.RemoveAt(LineNo - 1);
                        SOE.DeleteLine(LineNo, solinekey);
                        // Session["ListSoLIne"] = lstSoLine;
                        if (hfCheckEdit.Value.Equals("1"))
                        {
                            SOE.IsSOChanged = true;
                            SOE.IsSOChang = true;
                        }
                        if (ViewState["result"] != null)
                        {
                            SOE.IsSOChang = true;
                        }
                        SOE.CalculateSOTotal();
                        txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                        lblDebug.Text = "Line Deleted Successfully";
                        lblDebug.Visible = true;
                        BindSoLIneItem(SOE.LineItems);

                    }
                    else
                    {
                        if (isDelete)
                        {

                            SOE.DeleteLine(LineNo, solinekey);
                            SOE.CalculateSOTotal();
                            txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                            if (hfCheckEdit.Value.Equals("1"))
                            {
                                SOE.IsSOChanged = true;
                                SOE.IsSOChang = true;
                            }
                            if (ViewState["result"] != null)
                            {
                                SOE.IsSOChang = true;
                            }
                            lblDebug.Text = "Line Deleted Successfully";
                            lblDebug.Visible = true;
                            BindSoLIneItem(SOE.LineItems);


                        }
                        else
                        {
                            //0011088865

                            BindSoLIneItem(SOE.LineItems);

                            ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert(' This line is not deletable. ');", true);
                            //upTop.Update();
                            //upsoheader.Update();
                            //upSOLine.Update();

                            // 


                        }
                    }
                }
                else
                {

                    BindSoLIneItem(SOE.LineItems);
                }




            }
            catch (Exception exe) { }

        }






        #endregion
        //// Usage: equavalent to '#' at MAS500 Form UI.
        //// Generate next available SO# by MAS500 System.


        #region "btnCreate_OnClick Creating New SO Number"
        /// <summary>
        /// "btnCreate_OnClick Creating New SO Number"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnCreate_OnClick(Object sender, System.EventArgs e)
        {
            try
            {
                rbSO.Enabled = false;
                rbQuote.Enabled = false;
                CreateSalesOrder();

            }
            catch (Exception exe)
            {
                lblDebug.Text = exe.Message;
                lblDebug.Visible = true;
                upTop.Update();
            }
        }

        /// <summary>
        /// Creates the sales order.
        /// </summary>
        private void CreateSalesOrder()
        {
            try
            {
                SalesOrder SOE = new SalesOrder();
                txtSalesOrder.Text = string.Empty;
                SOE.GetNextTranNo(strCompanyID, hdfSalesQuote.Value.ToString(), getUserID(), Session["SessionKey"].ToString());//,(strCompanyID, hdfSalesQuote.Value,getUserID(), Session["SessionKey"].ToString());
                txtSalesOrder.Text = SOE.TranNo;
                txtOrderDate.Text = System.DateTime.Now.ToShortDateString();
                ValueEnableDisable();
                if (rbQuote.Checked)
                {
                    txtChngOrderNo.Visible = false;
                }
                else
                {
                    txtChngOrderNo.Visible = true;
                }
                txtSalesOrder.Enabled = false;
                txtCustomer.Focus();
                btnCreate.Enabled = false;
                imgBtnCustmer.Enabled = true;
                txtChngOrderNo.Text = "0";
                ViewState["SalesOrder"] = SOE;
                upsoheader.Update();
            }
            catch (Exception Exe)
            {
                lblDebug.Text = Exe.Message;
                lblDebug.Visible = true;
                upTop.Update();
            }


        }   // end "btnCreate_OnClick"
        #endregion

        // Usage: Actions after CustomerID Text Changed or CustomerID is entered.
        /* 1. Get CustKey from ( CustID + CompanyID ) combo.
         * 2. Get Customer Information ( such as DfltShipTo/BillTo, Contact, SalesSource, CustClass/Payment Terms, etc.
         */
        #region "txtCustomer_OnTextChanged "
        /// <summary>
        /// Cust ID tex box changed event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void txtCustomer_OnTextChanged(Object sender, System.EventArgs e)
        {
            try
            {


                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

                if (hdCustKey.Value.ToString().Equals(string.Empty))
                {

                    SOE.GetCustomer(txtCustomer.Text.Trim().ToUpper(), strCompanyID, getUserID(), 0);

                    if (SOE.CustKey == 0)
                    {
                        ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "Invalid Value", true);
                        //lblDebug.Text = "Invalid Value";
                        //lblDebug.Visible = true;
                        upsoheader.Update();
                        upTop.Update();
                        upBillTo.Update();
                        udpShipTo.Update();

                    }
                    else
                    {

                        ViewState["SalesOrder"] = SOE;
                        hdCustKey.Value = string.Empty;
                        ShowCustomerInfo(SOE);
                        SetInitialsForPopUp();
                        BindSoLIneItem(SOE.LineItems);

                        txtrequest.Text = System.DateTime.Now.ToShortDateString();
                        txtSchedShip.Text = System.DateTime.Now.ToShortDateString();
                        txtPromised.Text = System.DateTime.Now.ToShortDateString();

                        SOE.ChangeLinesDate(System.DateTime.Now, System.DateTime.Now, System.DateTime.Now);


                        // GetShipToAddress(SOE.CustKey.ToString(), txtShipTo.Text.Trim());
                        upsoheader.Update();
                        upTop.Update();
                        upBillTo.Update();
                        udpShipTo.Update();
                    }
                }
                else
                {
                    SOE.GetCustomer(txtCustomer.Text.Trim().ToUpper(), strCompanyID, getUserID(), Convert.ToInt32(hdCustKey.Value));//(txtCustomer.Text.Trim(), strCompanyID, getUserID());
                    if (SOE.CustKey == 0)
                    {
                        ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "Invalid Value", true);
                        //lblDebug.Text = "Invalid Value";
                        //lblDebug.Visible = true;
                        upsoheader.Update();
                        upTop.Update();
                        upBillTo.Update();
                        udpShipTo.Update();

                    }
                    else
                    {

                        ViewState["SalesOrder"] = SOE;
                        //ViewState["CustomerInfo"] = CustE;
                        hdCustKey.Value = string.Empty;
                        ShowCustomerInfo(SOE);
                        SetInitialsForPopUp();
                        BindSoLIneItem(SOE.LineItems);

                        txtrequest.Text = System.DateTime.Now.ToShortDateString();
                        txtSchedShip.Text = System.DateTime.Now.ToShortDateString();
                        txtPromised.Text = System.DateTime.Now.ToShortDateString();

                        SOE.ChangeLinesDate(System.DateTime.Now, System.DateTime.Now, System.DateTime.Now);

                        //GetShipToAddress(SOE.CustKey.ToString(), txtShipTo.Text.Trim());

                        upsoheader.Update();
                        upTop.Update();
                        upBillTo.Update();
                        udpShipTo.Update();
                    }
                }
                ValueEnableDisable();
                //For first time opening a header comment modal popup,i have to bind one blank row in header comment grid.This function is used for this.
                //Some initial logic i have written in this function

            }
            catch (Exception exe)
            {
                lblDebug.Visible = true;
                lblDebug.Text = "Invalid Customer";
                upsoheader.Update();
                upTop.Update();
                upBillTo.Update();
                udpShipTo.Update();
            }
        }
        #endregion



        #region "ShowCustomerInfo"
        /// <summary>
        /// Get Customer Information to Customer Object/Entity.
        /// </summary>
        private void ShowCustomerInfo(SalesOrder SOE)
        {

            try
            {
               
                txtCustomer.Text = SOE.CustID;
                txtCustClass.Text = SOE.CustClassID;
                aceShipTo.ContextKey = SOE.CustKey.ToString();
                if (SOE.SalesSourceKey != 0)
                {
                    ddlSource.SelectedValue = SOE.SalesSourceKey.ToString();
                }
                else
                {
                    ddlSource.SelectedIndex = 0;
                }
                txtSalesPerson.Text = SOE._ShipToAddress.SalesPersonID;
                txtPaymentTerms.Text = SOE._ShipToAddress.PmtTermsID;//

                txtCommPlan.Text = SOE._ShipToAddress.CommPlanID;// (dtAddress.Tables[0].Rows[0]["CommPlanID"].ToString());
                txtHeadShipVia.Text = SOE._ShipToAddress.ShipMethID;// (dtAddress.Tables[0].Rows[0]["ShipMethID"].ToString());
                txtHeadWarehouse.Text = SOE._ShipToAddress.WhseID;// (dtAddress.Tables[0].Rows[0]["WhseID"].ToString());
                #region Assigning Ship To Address To the controls

                //Assigning cust name
                txtCustName.Text = SOE._ShipToAddress.AddrName;
                txtShipTo.Text = SOE._ShipToAddress.CustAddrID;

                // Assigning Address line 
                txtShipToAddrLine1.Text = SOE._ShipToAddress.AddrLine1;
                txtShipToAddrLine2.Text = SOE._ShipToAddress.AddrLine2;
                txtShipToAddrLine3.Text = SOE._ShipToAddress.AddrLine3;
                txtShipToAddrLine4.Text = SOE._ShipToAddress.AddrLine4;
                txtShipToAddrLine5.Text = SOE._ShipToAddress.AddrLine5;
                // Assigning Address Key
                hdShipToToAddrKey.Value = SOE._ShipToAddress.CustAddrKey.ToString();
                txtShipAddrCity.Text = SOE._ShipToAddress.City.ToString().Trim();
                txtShipPostCode.Text = SOE._ShipToAddress.PostalCode.ToString().Trim();
                txtShipToCountryID.Text = SOE._ShipToAddress.CountryID.ToString().Trim();
                // binding state From Function to drop down
               
                txtShipAddrState.Text = SOE._ShipToAddress.StateID.Trim();// (dtBillToShipTo.Rows[0]["StateID"].Equals(DBNull.Value) ? string.Empty : dtBillToShipTo.Rows[0]["StateID"]).ToString();// assigning state
                #endregion

                #region Assigning Bill To Address to the controls
                try
                {
                    //Customer Cust = new Customer();
                    


                    //Assigning cust name
                    txtBillToCustName.Text = SOE._BillToAddress.AddrName;
                    txtBillTo.Text = SOE._BillToAddress.CustAddrID;

                    txtBillToName.Text = SOE._BillToAddress.AddrName;
                    // Assigning Address line 
                    txtBillToAddr1.Text = SOE._BillToAddress.AddrLine1;
                    txtBillToAddr2.Text = SOE._BillToAddress.AddrLine2;
                    txtBillToAddr3.Text = SOE._BillToAddress.AddrLine3;
                    txtBillToAddr4.Text = SOE._BillToAddress.AddrLine4;
                    txtBillToAddr5.Text = SOE._BillToAddress.AddrLine5;
                    // Assigning Address Key
                    hdBillToAddrKey.Value = SOE._BillToAddress.AddrKey.ToString();
                    txtBillToAddrCity.Text = SOE._BillToAddress.City;
                    txtBillToPostalCode.Text = SOE._BillToAddress.PostalCode;
                    txtBillToCountry.Text = SOE._BillToAddress.CountryID;
                   // ddlBillToCountry.SelectedValue = SOE._BillToAddress.CountryID;
                    // binding state From Function to drop down
                    
                    txtBillToState.Text = SOE._BillToAddress.StateID;

                }
                catch (Exception exe)
                {
                    lblDebug.Text = exe.Message;

                }
                #endregion

                aceShipToState.ContextKey = txtShipToCountryID.Text.Trim();
                aceShipToCity.ContextKey = txtShipToCountryID.Text.Trim() + "~" + txtShipAddrState.Text.Trim();
                aceShipToZipCode.ContextKey = txtShipToCountryID.Text.Trim() + "~" + txtShipAddrState.Text.Trim() + "~" + txtShipAddrCity.Text.Trim();
            

            }
            catch (Exception exe)
            { }

        }   // end "getCustomerInfo"
        #endregion

        #region "Salesorder/Quote Radio Button"
        /// <summary>
        /// Salesorder/Quote Radio Button Select Changed Event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void rbSalesQuote_SelectedIndexChanged(object sender, EventArgs e)
        {
            aceSalesOrder.ContextKey = strCompanyID + "-" + this.lblLogin.Text + "-" + hdfSalesQuote.Value;
            if (hdfSalesQuote.Value == "840")
            {
                imgQuoteToSO.Visible = true;
                upTop.Update();
            }
            else
            {
                imgQuoteToSO.Visible = false;
                upTop.Update();
            }

        }   // end "txtCustomer_OnTextChanged"
        #endregion

        #region "All Lookup Works"
        #region "Image Button ShipTo LookUp Click"
        /// <summary>
        /// ShipTo Address LookUp Button Click Event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnLookupShipTo_OnClick(Object sender, EventArgs e)
        {
            ResetLookUpTextBox();
            mpeLookupShipTo.Show();
            BindLookData("ShipTo");
        }   // end "btnLookupShipTo_OnClick"
        #endregion

        #region  CommPlan Look Up Button click event
        /// <summary>
        /// Handles the OnClick event of the btnLookupCommPlan control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnLookupCommPlan_OnClick(Object sender, EventArgs e)
        {
            ResetLookUpTextBox();
            mpeLookupShipTo.Show();
            BindLookData("CommPlan");
            upLookUP.Update();
        }
        #endregion

        #region "Image Button SalesPerson LookUp Click"
        /// <summary>
        /// SalesPerson LookUp Botton Click Event. Per Kush, it is not used.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void imgbtnSalesPersonLookUP_OnClick(Object sender, EventArgs e)
        {
            ResetLookUpTextBox();
            mpeLookupShipTo.Show();
            BindLookData("Salesperson");
        }   // end "imgbtnSalesPersonLookUP_OnClick"
        #endregion

        #region "Image Button Bill to LookUp CLick"
        /// <summary>
        /// BillTo Address LookUp Botton Click Event. Per Kush, it is not used.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void imgbtnLookupBillTo_OnClick(Object sender, EventArgs e)
        {
            mpeLookupShipTo.Show();
            BindLookData("BillTo");
        }   // end "imgbtnLookupBillTo_OnClick"
        #endregion

        #region "Image Button Cust Class LookUp CLick"
        /// <summary>
        /// Customer Class LookUp Botton Click Event. Per Kush, it is not used.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void imgbtnCustClass_OnClick(Object sender, EventArgs e)
        {
            mpeLookupShipTo.Show();
            BindLookData("CustClass");
        }   // end "imgbtnCustClass_OnClick"
        #endregion

        #region "Image Button Customer LookUp Click"
        /// <summary>
        /// Customer LookUp Botton Click Event. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void imgBtnCustmer_OnClick(Object sender, EventArgs e)
        {
            ResetLookUpTextBox();
            mpeLookupShipTo.Show();
            BindLookData("Customer");


        }   // end "imgBtnCustmer_OnClick"
        #endregion

        #region "Image Button Warehouse LookUp Click"
        /// <summary>
        /// Warehoues LookUp Botton Click Event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void imgbtnWareHouse_OnClick(Object sender, EventArgs e)
        {
            ResetLookUpTextBox();
            BindLookData("WareHouse");
            upsoheader.Update();
            upLookUP.Update();
            mpeLookupShipTo.Show();
        }   // end "imgbtnWareHouse_OnClick"
        #endregion

        #region  Ship Via Look UP
        /// <summary>
        /// Handles the OnClick event of the imgbtnShipVia control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void imgbtnShipVia_OnClick(Object sender, EventArgs e)
        {
            ResetLookUpTextBox();

            BindLookData("ShipVia");
            upsoheader.Update();
            upLookUP.Update();
            mpeLookupShipTo.Show();
        }
        #endregion

        #region Reset LookUp Fields
        /// <summary>
        /// Handles the OnClick event of the btnReset control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void btnReset_OnClick(Object sender, EventArgs e)
        {
            if (ViewState["text"].ToString().Equals("SOLookUP"))
            {
                BindLookData("SOLookUP");

            }
            else
                if (ViewState["text"].ToString().Equals("ShipTo"))
                {
                    BindLookData("ShipTo");

                }
                else
                    if (ViewState["text"].ToString().Equals("Customer"))
                    {
                        BindLookData("Customer");

                    }
                    else
                        if (ViewState["text"].ToString().Equals("CommPlan"))
                        {
                            BindLookData("CommPlan");
                        }
                        else
                            if (ViewState["text"].ToString().Equals("WareHouse"))
                            {
                                BindLookData("WareHouse");
                            }
                            else
                                if (ViewState["text"].ToString().Equals("ShipVia"))
                                {
                                    BindLookData("ShipVia");
                                }
                                else
                                    if (ViewState["text"].ToString().Equals("HeaderComment"))
                                    {
                                        BindLookData("HeaderComment");
                                    }
                                    else
                                        if (ViewState["text"].ToString().Equals("LineComment"))
                                        {
                                            BindLookData("LineComment");
                                        }
                                        else
                                            if (ViewState["text"].ToString().Equals("LineItem"))
                                            {
                                                BindLookData("LineItem");
                                            }

            ResetLookUpTextBox();
            upLookUP.Update();

        }
       
        private void ResetLookUpTextBox()
        {
            txtValue1_1.Text = string.Empty;
            txtValue2_1.Text = string.Empty;
            txtValue1_2.Text = string.Empty;
            txtValue2_2.Text = string.Empty;
            txtValue1_3.Text = string.Empty;
            txtValue2_3.Text = string.Empty;
            txtValue1_4.Text = string.Empty;
            txtValue2_4.Text = string.Empty;
            txtValue2_4.Visible = false;
            txtValue2_3.Visible = false;
            txtValue2_2.Visible = false;
            txtValue2_1.Visible = false;



            txtValue1_1.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
            txtValue2_1.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

            txtValue1_2.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
            txtValue2_2.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

            txtValue1_3.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
            txtValue2_3.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

            txtValue1_4.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
            txtValue2_4.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

        }
        #endregion

        #region "Image Button SOLINE LookUp Click"
        /// <summary>
        /// SOLine Item LookUp Button Click Event: PopUp ItemLookup to choose Item for SOLine.
        /// Stanley Note: it should be imgbtnSOlineItemLookUp
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void imgbtnLookupItem_Click(Object sender, EventArgs e)
        {

            ResetLookUpTextBox();
            mpeLookupShipTo.Show();
            LinkButton imgbtnLookup = (LinkButton)sender;
            GridRecord row = (GridRecord)((GridRecordItem)((System.Object)((Infragistics.Web.UI.TemplateContainer)((System.Web.UI.Control)imgbtnLookup.Parent)).Item)).Row;
            ViewState["LookupSOLineKey"] = row.Index;
            BindLookData("LineItem");

            upsoheader.Update();
            upSOLine.Update();
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            for (int i = 0; i <= SOE.LineItems.Count - 1; i++)
            {
                LinkButton imgbtnLookupSOLINE = (LinkButton)webgvSOLineItem.Rows[i].Items[1].FindControl("imgbtnLookupSOLINE");
                if (imgbtnLookupSOLINE != null)
                {
                    imgbtnLookupSOLINE.Visible = true;
                }
            }

            //
        }   // end "imgbtnLookupSOLINE_Click"
        #endregion

        #region "Bind LookUp Data"
        /// <summary>
        /// Binding LookUp data
        /// </summary>
        /// <param name="text"></param>
        private void BindLookData(string text)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            DataTable dtFields = CreateDataset().Tables[0];
            MSISalesOrderAPI MSIAPIE = new MSISalesOrderAPI();
            if (SOE != null)
            {
                if (text == "SOLookUP")
                {
                    #region "SOLookUP"
                    ViewState["text"] = "SOLookUP";//string iCompanyID, LookupType LT,DataTable dt

                    DataSet dsSOLookUP = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Orders, dtFields, getUserID());

                    BindDropDown(dsSOLookUP);

                    #endregion
                }
                else
                    if (text == "CommPlan")
                    {
                        #region "CommPlan"
                        ViewState["text"] = "CommPlan";//string iCompanyID, LookupType LT,DataTable dt

                        DataSet dsCommPlan = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.CommissionPlan, dtFields);

                        BindDropDown(dsCommPlan);

                        #endregion
                    }
                    else
                        if (text == "WareHouse")
                        {
                            #region "WareHouse"
                            ViewState["text"] = "WareHouse";//string iCompanyID, LookupType LT,DataTable dt

                            DataSet dsWareHouse = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.WareHouseLookUP, dtFields);

                            BindDropDown(dsWareHouse);

                            #endregion
                        }
                        else
                            if (text == "ShipVia")
                            {
                                #region "WareHouse"
                                ViewState["text"] = "ShipVia";//string iCompanyID, LookupType LT,DataTable dt

                                DataSet dsShipVia = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.ShipVia, dtFields);

                                BindDropDown(dsShipVia);

                                #endregion
                            }
                            else
                                if (text == "HeaderComment")
                                {
                                    #region "Header Comments"
                                    ViewState["text"] = "HeaderComment";//string iCompanyID, LookupType LT,DataTable dt

                                    DataSet dsHeader = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Comments, dtFields);

                                    BindDropDown(dsHeader);

                                    #endregion
                                }
                                else
                                    if (text == "LineComment")
                                    {
                                        #region "Header Comments"
                                        ViewState["text"] = "LineComment";//string iCompanyID, LookupType LT,DataTable dt
                                        DataSet dsLine = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Comments, dtFields);
                                        BindDropDown(dsLine);

                                        #endregion
                                    }
                                    else
                                        if (text == "LineItem")
                                        {
                                            #region "Item Look Up"
                                            ViewState["text"] = "LineItem";//string iCompanyID, LookupType LT,DataTable dt

                                            DataSet dsItem = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Item, dtFields, SOE.DfltWhseKey);

                                            BindDropDown(dsItem);

                                            #endregion
                                        }
                                        else
                                            if (text == "ShipTo")
                                            {
                                                #region "Customer Adress Look Up"
                                                ViewState["text"] = "ShipTo";//string iCompanyID, LookupType LT,DataTable dt

                                                DataSet dsItem = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.CustomerAddress, dtFields, getUserID(), SOE.CustID);

                                                BindDropDown(dsItem);

                                                #endregion

                                            }
                                            else
                                                if (text == "Customer")
                                                {
                                                    #region Customer Look UP
                                                    ViewState["text"] = "Customer";
                                                    DataSet dsCustomer = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Customer, dtFields, getUserID());

                                                    BindDropDown(dsCustomer);
                                                    #endregion

                                                }


            }
            else if (text == "SOLookUP")
            {
                #region "SOLookUP"
                ViewState["text"] = "SOLookUP";//string iCompanyID, LookupType LT,DataTable dt

                DataSet dsSOLookUP = MSIAPIE.GetLookupList(strCompanyID, MSISalesOrderAPI.LookupType.Orders, dtFields, getUserID());

                BindDropDown(dsSOLookUP);

                #endregion
            }


        }
        /// <summary>
        /// Creates the dataset.
        /// </summary>
        /// <returns></returns>
        public DataSet CreateDataset()
        {
            DataSet ds = new DataSet();

            DataTable dtFields = new DataTable();

            dtFields.Columns.Add("LookUp", typeof(string));
            dtFields.Columns.Add("FieldName", typeof(string));
            dtFields.Columns.Add("FieldDescription", typeof(string));
            dtFields.Columns.Add("FieldSize", typeof(string));

            DataTable dtValues = new DataTable();
            dtValues.Columns.Add("FieldName", typeof(string));
            dtValues.Columns.Add("Operator", typeof(string));
            dtValues.Columns.Add("Value1", typeof(string));
            dtValues.Columns.Add("Value2", typeof(string));
            ds.Tables.Add(dtFields);
            ds.Tables.Add(dtValues);
            return ds;


        }

        /// <summary>
        /// Binds the drop down.
        /// </summary>
        /// <param name="dsLookUP">The ds look UP.</param>
        private void BindDropDown(DataSet dsLookUP)
        {
            if (dsLookUP.Tables[0] != null)
            {
                DataTable dtCondition = new DataTable();
                dtCondition.Columns.Add("condition", typeof(string));
                dtCondition.Columns.Add("value", typeof(string));

                dtCondition.Rows.Add("Is equal to", "=");
                dtCondition.Rows.Add("Is not equal to", "<>");
                // dtCondition.Rows.Add("Is between", "");
                //dtCondition.Rows.Add("Is not between", "");
                dtCondition.Rows.Add("Is greater than", ">");
                dtCondition.Rows.Add("Is less than", "<");
                dtCondition.Rows.Add("Is greater than or equal to", ">=");
                dtCondition.Rows.Add("Is less than or equal to", "<=");
                dtCondition.Rows.Add("Begins with", "BEGINS WITH");
                dtCondition.Rows.Add("Does not begins with", "NOT BEGINS WITH");

                dtCondition.Rows.Add("Between", "BETWEEN");
                dtCondition.Rows.Add("Not Between", "Not Between");


                dtCondition.Rows.Add("Ends with", "ENDS WITH");
                dtCondition.Rows.Add("Does not end with", "NOT ENDS WITH");
                dtCondition.Rows.Add("Is less than or equal to", "");
                dtCondition.Rows.Add("Contains", "CONTAINS");
                dtCondition.Rows.Add("Does not contain", "NOT CONTAINS");


                drpfields1.DataSource = dsLookUP.Tables[0];
                drpfields1.DataTextField = "FieldDescription";
                drpfields1.DataValueField = "FieldName";
                drpfields1.DataBind();

                drpcondition1.DataSource = dtCondition;
                drpcondition1.DataTextField = "condition";
                drpcondition1.DataValueField = "value";
                drpcondition1.DataBind();

                drpfields2.DataSource = dsLookUP.Tables[0];
                drpfields2.DataTextField = "FieldDescription";
                drpfields2.DataValueField = "FieldName";
                drpfields2.DataBind();

                drpcondition2.DataSource = dtCondition;
                drpcondition2.DataTextField = "condition";
                drpcondition2.DataValueField = "value";
                drpcondition2.DataBind();

                drpfields3.DataSource = dsLookUP.Tables[0];
                drpfields3.DataTextField = "FieldDescription";
                drpfields3.DataValueField = "FieldName";
                drpfields3.DataBind();
                drpcondition3.DataSource = dtCondition;
                drpcondition3.DataTextField = "condition";
                drpcondition3.DataValueField = "value";
                drpcondition3.DataBind();

                drpfields4.DataSource = dsLookUP.Tables[0];
                drpfields4.DataTextField = "FieldDescription";
                drpfields4.DataValueField = "FieldName";
                drpfields4.DataBind();
                drpcondition4.DataSource = dtCondition;
                drpcondition4.DataTextField = "condition";
                drpcondition4.DataValueField = "value";
                drpcondition4.DataBind();

                #region Important Code
                //for (int i = 0; i < dsLookUP.Tables[0].Rows.Count; i++)
                //{
                //    System.Web.UI.HtmlControls.HtmlTableRow r = new System.Web.UI.HtmlControls.HtmlTableRow();
                //    System.Web.UI.HtmlControls.HtmlTableCell c1 = new System.Web.UI.HtmlControls.HtmlTableCell();
                //    System.Web.UI.HtmlControls.HtmlTableCell c2 = new System.Web.UI.HtmlControls.HtmlTableCell();

                //    DropDownList drpLookUpField = new DropDownList();
                //    drpLookUpField.DataSource = dsLookUP.Tables[0];
                //    drpLookUpField.DataTextField = "FieldName";
                //    drpLookUpField.DataValueField = "FieldName";
                //    drpLookUpField.DataBind();

                //    DropDownList drpCondition = new DropDownList();
                //    drpCondition.DataSource = dtCondition;
                //    drpCondition.DataTextField = "condition";
                //    drpCondition.DataValueField = "condition";
                //    drpCondition.DataBind();

                //    c1.Controls.Add(drpLookUpField);
                //    c2.Controls.Add(drpCondition);
                //    r.Cells.Add(c1);
                //    r.Cells.Add(c2);
                //    table1.Rows.Add(r);

                //}
                #endregion
                //SOLookUP
                if (ViewState["text"].ToString().Equals("SOLookUP"))
                {
                    wdgLookup.Visible = false;
                    wdgLookSO.Visible = true;
                    wdgLookSO.Rows.Clear();
                    wdgLookSO.DataSource = dsLookUP.Tables[1];
                    wdgLookSO.DataBind();
                    ViewState["dsSOLookUP"] = dsLookUP.Tables[1];
                    upSOLine.Update(); upsoheader.Update(); ;
                    upLookUP.Update();
                }
                else
                    if (ViewState["text"].ToString().Equals("ShipTo"))
                    {
                        wdgLookup.Visible = true;
                        wdgLookSO.Visible = false;

                        wdgLookup.Rows.Clear();
                        wdgLookup.DataSource = dsLookUP.Tables[1];
                        wdgLookup.DataBind();
                        ViewState["dsShipTo"] = dsLookUP.Tables[1];
                        upSOLine.Update(); upsoheader.Update(); ;
                        upLookUP.Update();
                    }
                    else
                        if (ViewState["text"].ToString().Equals("HeaderComment"))
                        {
                            wdgLookup.Visible = true;
                            wdgLookSO.Visible = false;
                            wdgLookup.Rows.Clear();
                            wdgLookup.DataSource = dsLookUP.Tables[1];
                            wdgLookup.DataBind();
                            ViewState["dsHeader"] = dsLookUP.Tables[1];
                        }
                        else
                            if (ViewState["text"].ToString().Equals("CommPlan"))
                            {
                                wdgLookup.Visible = true;
                                wdgLookSO.Visible = false;
                                wdgLookup.Rows.Clear();
                                wdgLookup.DataSource = dsLookUP.Tables[1];
                                wdgLookup.DataBind();
                                ViewState["dsCommPlan"] = dsLookUP.Tables[1];
                            }
                            else
                                if (ViewState["text"].ToString().Equals("WareHouse"))
                                {
                                    wdgLookup.Visible = true;
                                    wdgLookSO.Visible = false;
                                    wdgLookup.Rows.Clear();
                                    wdgLookup.DataSource = dsLookUP.Tables[1];
                                    wdgLookup.DataBind();
                                    ViewState["dsWareHouse"] = dsLookUP.Tables[1];
                                }
                                else
                                    if (ViewState["text"].ToString().Equals("ShipVia"))
                                    {
                                        wdgLookup.Visible = true;
                                        wdgLookSO.Visible = false;
                                        wdgLookup.Rows.Clear();
                                        wdgLookup.DataSource = dsLookUP.Tables[1];
                                        wdgLookup.DataBind();
                                        ViewState["dsShipVia"] = dsLookUP.Tables[1];
                                    }
                                    else
                                        if (ViewState["text"].ToString().Equals("LineComment"))
                                        {
                                            wdgLookup.Visible = true;
                                            wdgLookSO.Visible = false;
                                            wdgLookup.Rows.Clear();
                                            wdgLookup.DataSource = dsLookUP.Tables[1];
                                            wdgLookup.DataBind();
                                            ViewState["dsLine"] = dsLookUP.Tables[1];
                                            int count = dsLookUP.Tables[1].Rows.Count;
                                        }
                                        else
                                            if (ViewState["text"].ToString().Equals("LineItem"))
                                            {
                                                wdgLookup.Visible = true;
                                                wdgLookSO.Visible = false;
                                                wdgLookup.Rows.Clear();
                                                wdgLookup.DataSource = dsLookUP.Tables[1];
                                                wdgLookup.DataBind();
                                                ViewState["dsItem"] = dsLookUP.Tables[1];
                                                int count = dsLookUP.Tables[1].Rows.Count;
                                                upSOLine.Update(); upsoheader.Update(); ;
                                                upLookUP.Update();
                                            }
                                            else
                                                if (ViewState["text"].ToString().Equals("Customer"))
                                                {
                                                    wdgLookup.Visible = true;
                                                    wdgLookSO.Visible = false;

                                                    wdgLookup.Rows.Clear();
                                                    wdgLookup.DataSource = dsLookUP.Tables[1];
                                                    wdgLookup.DataBind();
                                                    ViewState["dsCustomer"] = dsLookUP.Tables[1];
                                                    upsoheader.Update();
                                                    upSOLine.Update(); upsoheader.Update(); ;
                                                    upLookUP.Update();
                                                }
                                                else
                                                    if (ViewState["text"].ToString().Equals("SalesOrder"))
                                                    {
                                                        wdgLookup.Visible = true;
                                                        wdgLookSO.Visible = false;

                                                        wdgLookup.Rows.Clear();
                                                        wdgLookup.DataSource = dsLookUP.Tables[1];
                                                        wdgLookup.DataBind();
                                                        ViewState["dsOrders"] = dsLookUP.Tables[1];
                                                        upsoheader.Update();
                                                        upSOLine.Update(); upsoheader.Update(); ;
                                                        upLookUP.Update();
                                                    }
                mpeLookupShipTo.Show();
            }




        }   // end "BindLookData"
        #endregion

        #region "LookUpSO Grid : Data Filtering Event"
        /// <summary>
        /// Look up Grid : Data Filtering Event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void wdgLookSO_DataFiltering(object sender, Infragistics.Web.UI.GridControls.FilteringEventArgs e)
        {
            try
            {
                if (ViewState["text"].ToString().Equals("SOLookUP"))
                {
                    wdgLookSO.DataSource = (DataTable)ViewState["dsSOLookUP"];
                    wdgLookSO.DataBind();
                }
            }
            catch (Exception exe) { }
        }
        #endregion

        #region "LookUp Grid : Data Filtering Event"
        /// <summary>
        /// Look up Grid : Data Filtering Event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void wdgLookup_DataFiltering(object sender, Infragistics.Web.UI.GridControls.FilteringEventArgs e)
        {
            try
            {
                if (ViewState["text"].ToString().Equals("ShipTo"))
                {
                    wdgLookup.DataSource = (DataTable)ViewState["dsShipTo"];
                    wdgLookup.DataBind();
                }
                else
                    if (ViewState["text"].ToString().Equals("CommPlan"))
                    {
                        wdgLookup.DataSource = (DataTable)ViewState["dsCommPlan"];
                        wdgLookup.DataBind();
                    }
                    else
                        if (ViewState["text"].ToString().Equals("WareHouse"))
                        {
                            wdgLookup.DataSource = (DataTable)ViewState["dsWareHouse"];
                            wdgLookup.DataBind();
                        }
                        else
                            if (ViewState["text"].ToString().Equals("ShipVia"))
                            {

                                wdgLookup.DataSource = (DataTable)ViewState["dsShipVia"];
                                wdgLookup.DataBind();

                            }
                            else if (ViewState["text"].ToString().Equals("HeaderComment"))
                            {
                                wdgLookup.DataSource = (DataTable)ViewState["dsHeader"];
                                wdgLookup.DataBind();
                            }
                            else if (ViewState["text"].ToString().Equals("LineComment"))
                            {
                                wdgLookup.DataSource = (DataTable)ViewState["dsLine"];
                                wdgLookup.DataBind();
                            }
                            else if (ViewState["text"].ToString().Equals("LineItem"))
                            {
                                wdgLookup.DataSource = (DataTable)ViewState["dsItem"];
                                wdgLookup.DataBind();

                            }
                            else if (ViewState["text"].ToString().Equals("Customer"))
                            {
                                wdgLookup.DataSource = (DataTable)ViewState["dsCustomer"];
                                wdgLookup.DataBind();

                            }
                            else if (ViewState["text"].ToString().Equals("SalesOrder"))
                            {
                                wdgLookup.DataSource = (DataTable)ViewState["dsOrders"];
                                wdgLookup.DataBind();

                            }

                // 
                mpeLookupShipTo.Show();
            }
            catch (Exception exe) { }
        }   // end "wdgLookup_DataFiltering"
        #endregion

        #region "LookUp Grid : Column Sorted Event"
        /// <summary>
        /// Lookup Grid : LookUp Column Sorted Event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void wdgLookup_ColumnSorted(object sender, Infragistics.Web.UI.GridControls.SortingEventArgs e)
        {
            try
            {
                if (ViewState["text"].ToString().Equals("ShipTo"))
                {
                    wdgLookup.DataSource = (DataTable)ViewState["dsShipTo"];
                    wdgLookup.DataBind();
                }
                else
                    if (ViewState["text"].ToString().Equals("CommPlan"))
                    {
                        wdgLookup.DataSource = (DataTable)ViewState["dsCommPlan"];
                        wdgLookup.DataBind();
                    }
                    else
                        if (ViewState["text"].ToString().Equals("WareHouse"))
                        {
                            wdgLookup.DataSource = ViewState["dsWareHouse"];
                            wdgLookup.DataBind();
                        }
                        else if (ViewState["text"].ToString().Equals("ShipVia"))
                        {
                            wdgLookup.DataSource = (DataTable)ViewState["dsShipVia"];
                            wdgLookup.DataBind();
                        }
                        else if (ViewState["text"].ToString().Equals("HeaderComment"))
                        {
                            wdgLookup.DataSource = (DataTable)ViewState["dsHeader"];
                            wdgLookup.DataBind();
                        }
                        else if (ViewState["text"].ToString().Equals("LineComment"))
                        {
                            wdgLookup.DataSource = (DataTable)ViewState["dsLine"];
                            wdgLookup.DataBind();
                        }
                        else if (ViewState["text"].ToString().Equals("LineItem"))
                        {
                            wdgLookup.DataSource = (DataTable)ViewState["dsItem"];
                            wdgLookup.DataBind();
                        }
                        else if (ViewState["text"].ToString().Equals("Customer"))
                        {
                            wdgLookup.DataSource = (DataTable)ViewState["dsCustomer"];
                            wdgLookup.DataBind();
                        }
                        else if (ViewState["text"].ToString().Equals("SalesOrder"))
                        {
                            wdgLookup.DataSource = (DataTable)ViewState["dsOrders"];
                            wdgLookup.DataBind();
                        }
            }
            catch (Exception ex)
            {
                lblDebug.Text = ex.Message;
                lblDebug.Visible = true;
            }
        }   // end "wdgLookup_ColumnSorted"
        #endregion



        #region "LookUp SO Grid : Column Sorted Event"
        /// <summary>
        /// Lookup Grid : LookUp Column Sorted Event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void wdgLookSO_ColumnSorted(object sender, Infragistics.Web.UI.GridControls.SortingEventArgs e)
        {
            try
            {
                if (ViewState["text"].ToString().Equals("SOLookUP"))
                {
                    wdgLookSO.DataSource = (DataTable)ViewState["dsSOLookUP"];
                    wdgLookSO.DataBind();
                }

            }
            catch (Exception ex)
            {
                lblDebug.Text = ex.Message;
                lblDebug.Visible = true;
            }
        }   // end "wdgLookup_ColumnSorted"
        #endregion
        #region "Image Button Sales Order LookUp Click"
        /// <summary>
        /// Customer LookUp Botton Click Event. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void imgSOLookUp_OnClick(Object sender, EventArgs e)
        {
            ResetLookUpTextBox();
            mpeLookupShipTo.Show();
            BindLookData("SOLookUP");


        }   // end "imgBtnCustmer_OnClick"
        #endregion


        protected void webgvSOLineItem_RowSelectionChanged(object sender, Infragistics.Web.UI.GridControls.SelectedRowEventArgs e)
        {
            Response.Write(e.CurrentSelectedRows);
        }

        #region  "LookUp SO Grid : Row Selection Changed event"
        /// <summary>
        ///  Look up grid : LookUp Row Selection Changed event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void wdgLookSO_RowSelectionChanged(object sender, Infragistics.Web.UI.GridControls.SelectedRowEventArgs e)
        {
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

                if (ViewState["text"].ToString().Equals("SOLookUP"))
                {
                    #region "SO  LookUp Selection:"
                    string LookUpKey = e.CurrentSelectedRows[0].DataKey[0].ToString();
                    hidSOKey.Value = LookUpKey.ToString();
                    if (txtSalesOrder.Text.Equals(e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text) && (rbQuote.Text.Equals(e.CurrentSelectedRows[0].Items.FindItemByKey("TranType ID")) || rbSO.Text.Equals(e.CurrentSelectedRows[0].Items.FindItemByKey("TranType ID"))))
                    {

                    }
                    else
                    {
                        if (!txtSalesOrder.Text.Equals(string.Empty) && hfCheckEdit.Value.Equals("1"))
                        {
                            if (Session["SessionKey"] != null)
                            {
                                if (SOE != null)
                                {
                                    SOE.ReleaseLock(Session["SessionKey"].ToString());
                                }
                            }
                        }


                        txtSalesOrder.Text = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                        if (e.CurrentSelectedRows[0].Items.FindItemByKey("TranType ID").Text.Equals("SO"))
                        {
                            hdfSalesQuote.Value = "801";
                        }
                        else
                        {
                            hdfSalesQuote.Value = "840";
                        }
                        CheckInitailsConditionsForSO();
                    }
                    //   ViewState["LookUpID"] = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;


                    mpeLookupShipTo.Hide();

                    upLookUP.Update();
                    upTop.Update();
                    upSOLine.Update();
                    upsoheader.Update();

                    #endregion
                }
            }
            catch (Exception exe) { }
        }
        #endregion


        #region  "LookUp Grid : Row Selection Changed event"
        /// <summary>
        ///  Look up grid : LookUp Row Selection Changed event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void wdgLookup_RowSelectionChanged(object sender, Infragistics.Web.UI.GridControls.SelectedRowEventArgs e)
        {
            try
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                if (hfCheckEdit.Value.Equals("1"))
                {
                    SOE.IsSOChanged = true;
                }
                if (ViewState["text"].ToString().Equals("ShipTo"))
                {
                    #region "Ship To LookUp Selection:"
                    string LookUpKey = e.CurrentSelectedRows[0].DataKey[0].ToString();
                    ViewState["LookUpKey"] = LookUpKey;
                    //   ViewState["LookUpID"] = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                    txtShipTo.Text = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                    if (SOE != null)
                    {
                        GetShipToAddress(SOE.CustKey.ToString(), txtShipTo.Text.Trim());
                    }
                    mpeLookupShipTo.Hide();

                    upLookUP.Update();
                    upSOLine.Update();
                    upsoheader.Update();

                    #endregion
                }
                else
                    if (ViewState["text"].ToString().Equals("CommPlan"))
                    {
                        #region "Comm LookUp Selection:"
                        string LookUpKey = e.CurrentSelectedRows[0].DataKey[0].ToString();
                        ViewState["LookUpKey"] = LookUpKey;
                        ViewState["LookUpID"] = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                        if (SOE != null)
                        {

                            SOE.ChangeCommPlan(e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text, Convert.ToInt32(LookUpKey));
                            txtCommPlan.Text = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text.ToString();
                        }
                        mpeLookupShipTo.Hide();

                        upLookUP.Update();
                        upSOLine.Update();
                        upsoheader.Update();

                        #endregion
                    }
                    else
                        if (ViewState["text"].ToString().Equals("WareHouse"))
                        {
                            #region "WareHouse LookUp Selection:"
                            string LookUpKey = e.CurrentSelectedRows[0].DataKey[0].ToString();
                            ViewState["LookUpKey"] = LookUpKey;
                            ViewState["LookUpID"] = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                            ChangeWareHouse(LookUpKey, e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text);
                            upsoheader.Update();
                            mpeLookupShipTo.Hide();
                            upLookUP.Update();

                            #endregion
                        }
                        else
                            if (ViewState["text"].ToString().Equals("ShipVia"))
                            {
                                #region "Ship Via LookUp Selection:"

                                string LookUpKey = e.CurrentSelectedRows[0].DataKey[0].ToString();
                                ViewState["LookUpKey"] = LookUpKey;
                                ViewState["LookUpID"] = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                                txtHeadShipVia.Text = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                                SOE.ChangeShipVia(e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text, Convert.ToInt32(LookUpKey));
                                upsoheader.Update();
                                mpeLookupShipTo.Hide();
                                upLookUP.Update();
                                //ScriptManager.GetCurrent(this).RegisterDataItem(upLookUP, txtHeadWarehouse.Text);

                                #endregion
                            }
                            else if (ViewState["text"].ToString().Equals("HeaderComment"))
                            {
                                #region Header Comment Row Selection
                                string LookUpKey = e.CurrentSelectedRows[0].DataKey[0].ToString();
                                string CommentID = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                                string Commentdesc = e.CurrentSelectedRows[0].Items.FindItemByKey("Description").Text;
                                try
                                {
                                    List<Comment> lstCommentEntity = (List<Comment>)ViewState["HeaderComment"];     //retreiving header comment from view state
                                    int index = Convert.ToInt32(ViewState["HeaderCommentRowIndex"]);
                                    Comment TempCommentE = lstCommentEntity[index];
                                    if (index < (lstCommentEntity.Count - 1))
                                    {
                                        // assigning comment to 
                                        TempCommentE.InitializeComments(CommentID, Convert.ToInt32(LookUpKey), Commentdesc, TempCommentE.ExtCmntUsedKey);
                                        lstCommentEntity[index] = TempCommentE;
                                    }
                                    else
                                    {
                                        //adding comment  to last row
                                        lstCommentEntity.RemoveAt(lstCommentEntity.Count - 1);
                                        Comment CommentE = new Comment(SOE, 801);

                                        if (TempCommentE.ExtCmntUsedKey == 0)
                                        {
                                            CommentE.InitializeComments(CommentID, Convert.ToInt32(LookUpKey), Commentdesc, Convert.ToInt32(0));
                                        }
                                        else
                                        {
                                            CommentE.InitializeComments(CommentID, Convert.ToInt32(LookUpKey), Commentdesc, TempCommentE.ExtCmntUsedKey);
                                        }
                                        lstCommentEntity.Add(CommentE);
                                        lstCommentEntity.Add(new Comment(SOE, 801));
                                    }
                                    ViewState["HeaderComment"] = lstCommentEntity;
                                    BindGridView(lstCommentEntity);
                                    mpeLookupShipTo.Hide();
                                    upGrid.Update();

                                }   // end "try"
                                catch (Exception exe)
                                {
                                    lblDebug.Text = exe.Message;
                                }

                                #endregion

                            }
                            else if (ViewState["text"].ToString().Equals("LineComment"))
                            {

                                #region LineComments
                                string LookUpKey = e.CurrentSelectedRows[0].DataKey[0].ToString();
                                string CommentID = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                                string Commentdesc = e.CurrentSelectedRows[0].Items.FindItemByKey("Description").Text;
                                try
                                {

                                    // Header Comment? in documentation?
                                    List<Comment> lstCommentEntity = (List<Comment>)ViewState["LineComments"];  // Retreiving line comment from view state
                                    lblCommMessage.Visible = false;
                                    Comment CommE = new Comment(SOE, 802);
                                    int index = Convert.ToInt32(ViewState["LineCommentRowIndex"]);                                                   // index of row

                                    //// findng comment textbox


                                    // split the hdfield. It contains comment iD and Comment
                                    Comment TempCommentE = lstCommentEntity[index];

                                    //StringBuilder sb = new StringBuilder();
                                    //for (int i = 1; i <= splitLineComment.Length - 1; i++)
                                    //{
                                    //    sb.Append(splitLineComment[i]); // appending comment
                                    //}
                                    //txtSOLineCommText.Text = sb.ToString();// adding comment to text box

                                    //lblLineExtCommentKey.Text = splitLineComment[0];
                                    if (index < (lstCommentEntity.Count - 1))
                                    {
                                        // assigning comment to 
                                        TempCommentE.InitializeComments(CommentID, Convert.ToInt32(LookUpKey), Commentdesc, TempCommentE.ExtCmntUsedKey);
                                    }   // end "if"
                                    else
                                    {
                                        //adding comment to last row
                                        lstCommentEntity.RemoveAt(lstCommentEntity.Count - 1);

                                        if (TempCommentE.ExtCmntUsedKey == 0)
                                        {
                                            CommE.InitializeComments(CommentID, Convert.ToInt32(LookUpKey), Commentdesc, 0);
                                            CommE.SeqNo = lstCommentEntity.Count + 1;
                                            lstCommentEntity.Add(CommE);
                                        }
                                        else
                                        {
                                            CommE.InitializeComments(CommentID, Convert.ToInt32(LookUpKey), Commentdesc, TempCommentE.ExtCmntUsedKey);
                                            CommE.SeqNo = lstCommentEntity.Count + 1;
                                            lstCommentEntity.Add(CommE);
                                        }

                                        // adding blank row
                                        Comment CommentE = new Comment(SOE, 802);
                                        CommentE.SeqNo = lstCommentEntity.Count + 1;
                                        lstCommentEntity.Add(CommentE);
                                        List<SOLine> lstSOLIne = SOE.LineItems;// (List<SOLine>)Session["ListSoLIne"];
                                        lstSOLIne[Convert.ToInt32(ViewState["rowindex"])].SOLineComments = lstCommentEntity;
                                        hidLIneComment.Value = "0";
                                    }

                                    grdLineComments.Rows[index].FindControl("txtSOLineCommText").Focus();
                                    hidLIneComment.Value = string.Empty;
                                    BindLineCommentToGridView(lstCommentEntity);
                                    mpeLookupShipTo.Hide();
                                    upSOLine.Update(); upsoheader.Update(); ;

                                    // end "inner else"
                                    // end "outter else"
                                }
                                catch (Exception exe) { }
                                #endregion
                            }
                            else if (ViewState["text"].ToString().Equals("LineItem"))
                            {
                                #region Line item
                                int rowindex = Convert.ToInt32(ViewState["LookupSOLineKey"]);
                                string ItemKey = e.CurrentSelectedRows[0].DataKey[0].ToString();
                                string ItemID = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                                bool isDelete = Convert.ToBoolean(SOE.LineItems[rowindex].Deletable);
                                if (SOE.LineItems[rowindex].SOLineKey == 0)
                                {
                                    SOE.LineItems.RemoveAt(rowindex);
                                    SOE.LineItems.Insert(rowindex, new SOLine(SOE));
                                    SOE.ResetLineNo();

                                    BindItemRow(ItemID, Convert.ToInt32(ItemKey), rowindex);
                                    BindSoLIneItem(SOE.LineItems);
                                    mpeLookupShipTo.Hide();
                                    upSOLine.Update(); upsoheader.Update(); ;

                                }
                                else
                                {
                                    if (isDelete)
                                    {
                                        if (SOE.LineItems[rowindex].QtyShip == 0)
                                        {
                                            //ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "Shipment already done.Cannot change the item;", true);

                                            SOE.DeleteLine(SOE.LineItems[rowindex].SOLineNo, Convert.ToInt32(SOE.LineItems[rowindex].SOLineKey));
                                            //SOE.LineItems.RemoveAt(rowindex);
                                            SOE.LineItems.Insert(rowindex, new SOLine(SOE));
                                            SOE.ResetLineNo();
                                            BindItemRow(ItemID, Convert.ToInt32(ItemKey), rowindex);
                                            //MSISalesOrderAPI MSISOE = new MSISalesOrderAPI();
                                            //GetLookupList(string prefixText, string iCompanyID, LookupType LT, string UserID, string ContextID)
                                            // ddp_UnitMeasID.EditorControl.DataSource = MSISOE.GetLookupList("", "", MSISalesOrderAPI.LookupType.UnitOfMeasurement, "", Convert.ToInt32(ItemKey));//
                                            // ddp_UnitMeasID.EditorControl.TextField = "UnitMeasID";
                                            // ddp_UnitMeasID.EditorControl.ValueField = "UnitMeasKey";
                                            // ddp_UnitMeasID.EditorControl.DataBind();
                                            mpeLookupShipTo.Hide();
                                            upSOLine.Update(); upsoheader.Update();
                                        }
                                        else
                                        {

                                            BindSoLIneItem(SOE.LineItems);
                                            mpeLookupShipTo.Hide();
                                            upSOLine.Update(); upsoheader.Update(); ;
                                        }

                                    }
                                    else
                                    {
                                        BindSoLIneItem(SOE.LineItems);
                                        mpeLookupShipTo.Hide();
                                        upSOLine.Update(); upsoheader.Update(); ;
                                    }
                                }

                                #endregion
                            }
                            else if (ViewState["text"].ToString().Equals("Customer"))
                            {
                                #region "Customer LookUp Selection:"

                                string LookUpKey = e.CurrentSelectedRows[0].DataKey[0].ToString();
                                string LookupID = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                                SOE.GetCustomer(LookupID, strCompanyID, getUserID(), Convert.ToInt32(LookUpKey));//(txtCustomer.Text.Trim(), strCompanyID, getUserID());
                                ViewState["SalesOrder"] = SOE;
                                //ViewState["CustomerInfo"] = CustE;
                                txtCustomer.Text = LookupID;
                                hdCustKey.Value = string.Empty;
                                ValueEnableDisable();
                                btnHeaderCom.Enabled = true;
                                ShowCustomerInfo(SOE);
                                upGrid.Update();
                                mpeLookupShipTo.Hide();
                                upLookUP.Update();
                                upSOLine.Update();
                                upsoheader.Update(); ;

                                upSOLine.Update(); upsoheader.Update(); ;
                                upLookUP.Update();
                                SetInitialsForPopUp();
                                BindSoLIneItem(SOE.LineItems);
                                upsoheader.ChildrenAsTriggers = true;
                                upsoheader.Update();


                                #endregion
                            }
                            else if (ViewState["text"].ToString().Equals("SalesOrder"))
                            {
                                #region "Sales Order LookUp Selection:"

                                string LookUpKey = e.CurrentSelectedRows[0].DataKey[0].ToString();
                                string LookupID = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                                txtSalesOrder.Text = e.CurrentSelectedRows[0].Items.FindItemByKey("LookUpID").Text;
                                CheckInitailsConditionsForSO();

                                mpeLookupShipTo.Hide();

                                upLookUP.Update();
                                upSOLine.Update();
                                upsoheader.Update();


                                #endregion
                            }
            }
            catch (Exception ex)
            {
                lblDebug.Text = ex.Message;
                lblDebug.Visible = true;
            }

        }   // end "wdgLookup_RowSelectionChanged"
        #endregion

        /// <summary>
        /// Handles the InitializeRow event of the wdgLookSO control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Infragistics.Web.UI.GridControls.RowEventArgs"/> instance containing the event data.</param>
        protected void wdgLookSO_InitializeRow(object sender, Infragistics.Web.UI.GridControls.RowEventArgs e)
        {

        }
        #region "LookUp Grid : Initialize Row Event"
        /// <summary>
        /// Lookup Grid : Initialize Row Event for objects ( ShipToAddress, WareHouse, LineItem )
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void wdgLookup_InitializeRow(object sender, Infragistics.Web.UI.GridControls.RowEventArgs e)
        {

        }   // end "wdgLookup_InitializeRow"
        #endregion
        #endregion

        #region "Getting ShipTo Address by addresskey"
        /// <summary>
        /// txtShipTo_TextChanged Event: 
        /// Stanley: ?? don't know the logic.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void txtShipTo_TextChanged(object sender, EventArgs e)
        {
            if (!(hidCustAddrKey.Value.ToString().Equals("0")))
            {
                int addrKey = Convert.ToInt32(hidCustAddrKey.Value);
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];


                hidCustAddrKey.Value = "0";
                GetShipToAddress(SOE.CustKey.ToString(), txtShipTo.Text.Trim());
                if (hfCheckEdit.Value.Equals("1"))
                {
                    SOE.IsSOChanged = true;
                    SOE.IsSOChang = true;
                }
                if (ViewState["result"] != null)
                {
                    SOE.IsSOChang = true;
                }
            }
            else
            {
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

                txtShipTo.Text = SOE._ShipToAddress.CustAddrID;
                upsoheader.Update();
                upSOLine.Update();
            }

            upBillTo.Update();
            udpShipTo.Update();
        }   // end "txtShipTo_TextChanged"
        #endregion

        #region "Recalculate Item UnitPrice After ShipTo Adress Changed"
        /// <summary>
        /// ReCalculatePriceofEachLine Method: 
        /// </summary>
        /// <param name="ItemID"></param>
        /// <param name="ItemKey"></param>
        /// <param name="rowindex"></param>
        /// <param name="addrKey"></param>
        private void ReCalculatePriceofEachLine(string ItemID, int ItemKey, int rowindex, int addrKey)
        {
            //try
            //{
            //    var currentVal = ItemID;
            //    var currentKey = ItemKey;

            //    List<SOLine> lstSoLine = (List<SOLine>)Session["ListSoLIne"];
            //    if (lstSoLine != null)
            //    {
            //        string UOM = string.Empty;
            //        int qty = 1;
            //        SOLine LineItemE = new SOLine();
            //        Item ItemE = new Item();
            //        if (rowindex == -1)
            //        {
            //            // LineItemE.SOLineNo = lstSoLine.Count + 1;
            //        }
            //        else
            //        {
            //            LineItemE = lstSoLine[rowindex];

            //            string str = LineItemE.QtyOrd.ToString();
            //            if (str.Equals(string.Empty))
            //            {   // default: QtyOrd, if empty.
            //                str = "1";
            //            }
            //            if (str.Length > 0)
            //            {
            //                // If Row Contains Minus then i m making it false
            //                if (str[0].ToString().Equals("-"))
            //                {
            //                    //LineItemE = lstSoLine[lstSoLine.Count - 1];
            //                }
            //                else
            //                {

            //                }
            //            }
            //        }

            //        //LineItemE.ItemID = currentVal;
            //        LineItemE.ItemKey = Convert.ToInt32(currentKey);

            //        SalesOrder SoE = (SalesOrder)ViewState["SalesOrder"];
            //        if (ViewState["SalesOrder"] == null)
            //        {
            //            if (Session["CustEntity"] != null)
            //            {

            //                Customer CustE = (Customer)Session["CustEntity"];
            //                //Need To Check
            //                // ItemE = ItemE.GetItemStandardInfo(Convert.ToInt32(CustE.WhseKey), Convert.ToInt32(LineItemE.ItemKey), Convert.ToInt32(CustE.CustKey), addrKey, CustE.CurrID, Convert.ToInt32(0), Convert.ToInt32(1), strCompanyID, System.DateTime.Now, float.Parse((1).ToString()));
            //                //LineItemE.WhseID = CustE.WhseID;
            //                // LineItemE.WhseKey = CustE.WhseKey;
            //            }
            //        }
            //        else
            //        {
            //            //Need To Check
            //            //ItemE = ItemE.GetItemStandardInfo(Convert.ToInt32(SoE.WhseKey), Convert.ToInt32(LineItemE.ItemKey), Convert.ToInt32(SoE.CustKey), addrKey, SoE.CurrID, Convert.ToInt32(0), Convert.ToInt32(1), SoE.CompanyID, System.DateTime.Now, float.Parse((1).ToString()));
            //            //LineItemE.WhseID = SoE.WhseID;
            //            //LineItemE.WhseKey = SoE.WhseKey;
            //        }
            //        //Need To Check
            //        //LineItemE.CommClassKey = ItemE.CommClassKey;
            //        //LineItemE.ReturnValue = ItemE.ReturnValue;
            //        //LineItemE.SystemPriceDetermination = ItemE.SystemPriceDeterm;
            //        //LineItemE.GLAcctKey = Convert.ToInt32(ItemE.GLAcctKey);
            //        //LineItemE.QtyAvailable = ItemE.QtyAvailable;


            //        //   LineItemE.CmntOnly = Convert.ToBoolean(ItemE.IsCmntOnly);

            //        //LineItemE.QtyOrd = qty;
            //        LineItemE.Description = ItemE.ItemDescription;

            //        //LineItemE.PriceMethod = ItemE.PriceMethod;
            //        //LineItemE.SchedPriceDeterm = ItemE.SchedPriceDeterm;

            //        //LineItemE.UnitPriceFromSchedule = Convert.ToDecimal(ItemE.UnitPriceFromSched);
            //        //LineItemE.UnitPriceFromPromo = ItemE.UnitPriceFromPromo;
            //        //LineItemE.SalesPromotionID = ItemE.SalesPromotionID;
            //        //LineItemE.SalesPromotionKey = Convert.ToInt32(ItemE.SalesPromotionKey);
            //        //LineItemE.STaxClassKey = ItemE.STaxClassKey;

            //        // LineItemE.UnitMeasKey = Convert.ToInt32(ItemE.Prime_UOMKey);
            //        //LineItemE.UnitMeasID = ItemE.Prime_UOM_ID;
            //        // LineItemE.UnitPrice = Convert.ToDecimal(ItemE.UnitPrice);

            //        LineItemE.SOLineComments = null;

            //        string LineSalesAmt = string.Format("{0:N" + Math.Abs(2) + "}", Convert.ToDecimal(LineItemE.QtyOrd * LineItemE.UnitPrice));

            //        //LineItemE.ExtAmt = Convert.ToDecimal(LineSalesAmt);
            //        //Making unit price ovrride false
            //        LineItemE.UnitPriceOvrd = false;

            //        lstSoLine[rowindex] = LineItemE;

            //        BindSoLIneItem((List<SOLine>)Session["ListSoLIne"]);
            //    }
            //    else
            //    {

            //    }
            //}
            //catch (Exception exe)
            //{
            //    lblDebug.Text = exe.Message;
            //    lblDebug.Visible = true;

            //}
        }   // end "ReCalculatePriceofEachLine"
        #endregion

        #region "Custmer PO Textbox changed event: Need To Check"
        /// <summary>
        /// txtCustomerPO_TextChanged event: 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void txtCustomerPO_TextChanged(object sender, EventArgs e)
        {
            //SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (SOE != null)
            {
                try
                {
                    DataTable dt = SOE.ValidateCustPONumber(txtCustomerPO.Text.Trim());
                    if (dt != null)
                    {
                        if (dt.Rows.Count > 0)
                        {
                            ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('" + dt.Rows[0]["Message"].ToString() + "');", true);
                        }
                    }
                }
                catch (Exception exe)
                {
                    ScriptManager.RegisterClientScriptBlock(this, this.GetType(), "Key", "alert('" + exe.Message + "');", true);
                }
            }

            //if (ViewState["SalesOrder"] != null)
            //{   // ?? revier question: what is this?
            //    SOE.IsSOChanged = true;
            //    DataTable dtCustPoNum = SOE.CustPONumValidation(SOE.CustKey, txtCustomerPO.Text.Trim());
            //    if (dtCustPoNum.Rows.Count > 0)
            //    {
            //        btnFinish.Enabled = false;
            //        btnSave.Enabled = false;
            //    }
            //    else
            //    {
            //        btnFinish.Enabled = true;
            //        btnSave.Enabled = true;
            //        SOE.CustPONo = txtCustomerPO.Text.Trim();
            //    }
            //}
            //else
            //{   // ?? revier question: what is this?
            //    Customer CustE = (Customer)Session["CustEntity"];
            //    SalesOrder SO = new SalesOrder();
            //    DataTable dtCustPoNum = SO.CustPONumValidation(Convert.ToInt32(CustE.CustKey), txtCustomerPO.Text.Trim());
            //    if (dtCustPoNum.Rows.Count > 0)
            //    {
            //        btnFinish.Enabled = false;
            //        btnSave.Enabled = false;
            //    }
            //    else
            //    {
            //        btnFinish.Enabled = true;
            //        btnSave.Enabled = true;
            //    }
            //    //Ship to address key that is generated after saving
            //    // CustE.DfltShipToAddrKey = Convert.ToInt32(LookUpKey);
            //}
        }       // end "txtCustomerPO_TextChanged"
        #endregion

        #region rbSO_CheckedChanged
        /// <summary>
        ///rbSO_CheckedChanged
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void rbSO_CheckedChanged(object sender, EventArgs e)
        {
            imgQuoteToSO.Visible = false;
            txtChngOrderNo.Visible = false;
            hdfSalesQuote.Value = "801";
            aceSalesOrder.ContextKey = strCompanyID + "-" + this.lblLogin.Text + "-" + hdfSalesQuote.Value;
            upTop.Update();
            upsoheader.Update();
        }   // 
        #endregion

        #region "rbQuote_CheckedChanged"
        /// <summary>
        /// rbQuote_CheckedChanged event: once rbQuote Checked Changed, ??
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void rbQuote_CheckedChanged(object sender, EventArgs e)
        {
            imgQuoteToSO.Visible = true;
            txtChngOrderNo.Visible = false;
            hdfSalesQuote.Value = "840";
            aceSalesOrder.ContextKey = strCompanyID + "-" + this.lblLogin.Text + "-" + hdfSalesQuote.Value;
            upTop.Update();
            upsoheader.Update();
        }   // end "rbQuote_CheckedChanged"
        #endregion

        /// <summary>
        /// Handles the TextChanged event of the txtShipAddrCity control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void txtShipAddrCity_TextChanged(object sender, EventArgs e)
        {
            
            
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                bool check = SOE.CheckValidCity(txtShipAddrState.Text, txtShipToCountryID.Text, txtShipAddrCity.Text.Trim());
                if (check)
                {
                    aceShipToZipCode.ContextKey = txtShipToCountryID.Text.Trim() + "~" + txtShipAddrState.Text.Trim() + "~" + txtShipAddrCity.Text.Trim();
                    txtShipAddrCity.Text = txtShipAddrCity.Text.Trim().ToUpper(); 
                }
                else
                {
                    txtShipAddrCity.Text = SOE._ShipToAddress.City.Trim();
                }
            
        }
        /// <summary>
        /// Handles the textChanged event of the txtShipAddrState control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void txtShipAddrState_textChanged(object sender, EventArgs e)
        {
           
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                bool check = SOE.CheckValidState(txtShipAddrState.Text, txtShipToCountryID.Text);
                if (check)
                {
                    aceShipToCity.ContextKey = txtShipToCountryID.Text.Trim() + "~" + txtShipAddrState.Text.Trim();
                    aceShipToZipCode.ContextKey = txtShipToCountryID.Text.Trim() + "~" + txtShipAddrState.Text.Trim() + "~" + txtShipAddrCity.Text.Trim();
                    txtShipAddrState.Text = txtShipAddrState.Text.Trim().ToUpper();
                }
                else
                {
                    txtShipAddrState.Text = SOE._ShipToAddress.StateID.Trim();
                }
            
        }
        /// <summary>
        /// Handles the TextChanged event of the txtShipToCountryID control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void txtShipToCountryID_TextChanged(object sender, EventArgs e)
        {
            
                SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
                bool check = SOE.CheckValidCountryID(txtShipToCountryID.Text);
                if (check)
                {
                    aceShipToState.ContextKey = txtShipToCountryID.Text.Trim();
                    aceShipToCity.ContextKey = txtShipToCountryID.Text.Trim() + "~" + txtShipAddrState.Text.Trim();
                    aceShipToZipCode.ContextKey = txtShipToCountryID.Text.Trim() + "~" + txtShipAddrState.Text.Trim() + "~" + txtShipAddrCity.Text.Trim();
                    txtShipToCountryID.Text = txtShipToCountryID.Text.Trim().ToUpper(); 
                }
                else
                {
                    txtShipToCountryID.Text = SOE._ShipToAddress.CountryID.Trim();
                }
          
            
         }
        /// <summary>
        /// Handles the TextChanged event of the txtShipPostCode control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void txtShipPostCode_TextChanged(object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (!txtShipPostCode.Text.Equals(string.Empty))
            {
                DataTable dt = SOE.ChangeZipCode(Convert.ToInt32(txtShipPostCode.Text.Trim()));
                if (dt != null)
                {
                    if (dt.Rows.Count > 0)
                    {
                        txtShipAddrCity.Text = dt.Rows[0]["City"].ToString();
                        txtShipToCountryID.Text = dt.Rows[0]["CountryID"].ToString();


                        txtShipAddrState.Text = dt.Rows[0]["StateID"].ToString();
                        //ddlState.SelectedItem.Text = dt.Rows[0]["StateID"].ToString();
                        txtShipPostCode.Text = dt.Rows[0]["PostalCode"].ToString().Trim();

                        udpShipTo.Update();
                    }
                    else
                    {
                        txtShipPostCode.Text = SOE._ShipToAddress.PostalCode.Trim();
                    }
                }
            }
            
            mdlShipToAddress.Show();
        }
        #region "txtCommPlan_TextChanged"
        /// <summary>
        /// txtCommPlan_TextChanged event: if changed CommPlanID and saved existing one are different, SO is changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>       
        protected void txtCommPlan_TextChanged(object sender, EventArgs e)
        {
            // Wrong Checking... need to check existing saved CommanPlan ID and Newly changed are different.
            // then, yes, SO is changed.
            SalesOrder oSO = (SalesOrder)ViewState["SalesOrder"];
            if (hdCommPlan.Value.Equals("0") || hdCommPlan.Value.Equals(string.Empty))
            {
                if (oSO != null)
                {
                    bool check = oSO.CheckCommPlan(txtCommPlan.Text.Trim(),strCompanyID);
                    if (check)
                    {
                        if (hfCheckEdit.Value.Equals("1"))
                        {
                            oSO.IsSOChanged = true;
                            oSO.IsSOChang = true;
                        }
                        if (ViewState["result"] != null)
                        {
                            oSO.IsSOChang = true;
                        }
                        txtCommPlan.Text = oSO.DfltCommPlanID;
                    }
                    else
                    {
                        txtCommPlan.Text = oSO.DfltCommPlanID;
                       
                    }
                    upsoheader.Update();
                }
            }
            else
            {
                if (oSO != null)
                {
                    if (hfCheckEdit.Value.Equals("1"))
                    {
                        oSO.IsSOChanged = true;
                        oSO.IsSOChang = true;
                    }
                    if (ViewState["result"] != null)
                    {
                        oSO.IsSOChang = true;
                    }
                    // oSO.DfltCommPlanKey = Convert.ToInt32(hdCommPlan.Value);
                    oSO.ChangeCommPlan(txtCommPlan.Text, Convert.ToInt32(hdCommPlan.Value));
                    txtCommPlan.Text = oSO.DfltCommPlanID;
                }
            }
            hdCommPlan.Value = string.Empty;
        }   // end "txtCommPlan_TextChanged"
        #endregion

        #region "txtHeadWarehouse_TextChanged"
        /// <summary>
        /// txtHeadWarehouse_TextChanged event: if changed WhseID ( any WhseID at SOheader or SOL ) and different 
        /// from saved existing WhseID, then, SO is changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void txtHeadWarehouse_TextChanged(object sender, EventArgs e)
        {
            ChangeWareHouse(hdDefaultWH.Value, txtHeadWarehouse.Text.Trim());
            upSOLine.Update(); upsoheader.Update(); ;
        }

        private void ChangeWareHouse(string WhseKey, string WhseID)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (WhseKey.Equals("0") || WhseKey.Equals(string.Empty))
            {
                if (SOE != null)
                {
                    txtHeadWarehouse.Text = SOE.WhseID;
                }
            }
            else
            {
                if (SOE != null)
                {
                    if (hfCheckEdit.Value.Equals("1"))
                    {
                        SOE.IsSOChanged = true;
                        SOE.IsSOChang = true;
                    }
                    if (ViewState["result"] != null)
                    {
                        SOE.IsSOChang = true;
                    }
                    string resultItem = SOE.ChangeWhse(Convert.ToInt32(WhseKey), WhseID);

                    upSOLine.Update();
                    upsoheader.Update();
                    SOE.CalculateSOTotal();
                    txtOrderTotal.Text = SOE.SalesAmt.ToString("c");
                    hdDefaultWH.Value = "0";
                    txtHeadWarehouse.Text = WhseID;
                    BindSoLIneItem(SOE.LineItems);
                    if (resultItem != string.Empty)
                    {
                        //ScriptManager.RegisterStartupScript(this.upsoheader, this.upsoheader.GetType(), "Key", "<script type='text/javascript'>alert('j')</script>", true);
                        ClientScript.RegisterStartupScript(this.GetType(), "hi", "<script type='text/javascript'>alert('j')</script>", true);
                       // ScriptManager.RegisterStartupScript(this.upSOLine, this.upSOLine.GetType(), "Key", resultItem + " is not in " + WhseID + " warehouse", true);
                        lblDebug.Text = resultItem + " is not in " + WhseID + " warehouse.";
                        lblDebug.Visible = true;
                        upSOLine.Update();
                        upsoheader.Update();
                    }

                }
            }
        }   // end "txtHeadWarehouse_TextChanged"
        #endregion

        #region "txtHeadShipVia_TextChanged"

        /// <summary>
        /// Handles the TextChanged event of the txtHeadShipVia control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void txtHeadShipVia_TextChanged(object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (hdShipVia.Value.ToString().Equals("0"))
            {
                if (SOE != null)
                {
                    bool check = SOE.CheckHeadShipVia(txtHeadShipVia.Text.Trim(), strCompanyID);
                    if (check)
                    {
                        txtHeadShipVia.Text = SOE._ShipToAddress.ShipMethID;
                    }
                    else {
                        txtHeadShipVia.Text = SOE._ShipToAddress.ShipMethID;
                    }

                   
                    upsoheader.Update();
                }
            }
            else
            {
                if (hfCheckEdit.Value.Equals("1"))
                {
                    SOE.IsSOChanged = true;
                    SOE.IsSOChang = true;
                }
                SOE.ChangeShipVia(txtHeadShipVia.Text.Trim(), Convert.ToInt32(hdShipVia.Value));
                // SOE._ShipToAddress.ShipMethID = txtHeadShipVia.Text.Trim();
                //SOE._ShipToAddress.ShipMethKey = Convert.ToInt32(hdShipVia.Value);
                hdShipVia.Value = "0";
            }

        }   // end "txtHeadShipVia_TextChanged"
        #endregion


        /// <summary>
        /// Handles the TextChanged event of the txtRequest control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void txtrequest_TextChanged(object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];

            if (SOE != null)
            {
                if (hfCheckEdit.Value.Equals("1"))
                {
                    SOE.IsSOChanged = true;
                }
                // SOE.ChangeRequestDate(Convert.ToDateTime(txtrequest.Text));

            }

        }
        /// <summary>
        /// Handles the TextChanged event of the txtSchedShip control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void txtSchedShip_TextChanged(object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (SOE != null)
            {
                if (hfCheckEdit.Value.Equals("1"))
                {
                    SOE.IsSOChanged = true;
                }
                //SOE.ChangeShipDate(Convert.ToDateTime(txtSchedShip.Text));
            }

        }


        /// <summary>
        /// Handles the TextChanged event of the txtPromised control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void txtPromised_TextChanged(object sender, EventArgs e)
        {
            SalesOrder SOE = (SalesOrder)ViewState["SalesOrder"];
            if (SOE != null)
            {
                if (hfCheckEdit.Value.Equals("1"))
                {
                    SOE.IsSOChanged = true;
                }
                // SOE.ChangePromiseDate(Convert.ToDateTime(txtPromised.Text));
            }

        }


        /// <summary>
        /// Handles the RowsRequested event of the VirtualScrolling control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="VirtualScrollingEventArgs"/> instance containing the event data.</param>
        ///
        void VirtualScrolling_RowsRequested(object sender, VirtualScrollingEventArgs e)
        {
            if (ViewState["text"].ToString().Equals("SOLookUP"))
            {
                wdgLookSO.Rows.Clear();
                wdgLookSO.DataSource = ViewState["dsSOLookUP"];
                wdgLookSO.DataBind();

            }
            else
                if (ViewState["text"].ToString().Equals("Customer"))
                {
                    wdgLookup.Rows.Clear();
                    wdgLookup.DataSource = ViewState["dsCustomer"];
                    wdgLookup.DataBind();

                }
                else
                    if (ViewState["text"].ToString().Equals("CommPlan"))
                    {
                        wdgLookup.Rows.Clear();
                        wdgLookup.DataSource = ViewState["dsCommPlan"];
                        wdgLookup.DataBind();

                    }
                    else
                        if (ViewState["text"].ToString().Equals("WareHouse"))
                        {
                            wdgLookup.Rows.Clear();
                            wdgLookup.DataSource = ViewState["dsWareHouse"];
                            wdgLookup.DataBind();

                        }
                        else
                            if (ViewState["text"].ToString().Equals("ShipVia"))
                            {
                                wdgLookup.Rows.Clear();
                                wdgLookup.DataSource = ViewState["dsShipVia"];
                                wdgLookup.DataBind();

                            }
                            else if (ViewState["text"].ToString().Equals("HeaderComment"))
                            {
                                wdgLookup.Rows.Clear();
                                wdgLookup.DataSource = (DataTable)ViewState["dsHeader"];
                                wdgLookup.DataBind();
                            }
                            else if (ViewState["text"].ToString().Equals("LineComment"))
                            {
                                wdgLookup.Rows.Clear();
                                wdgLookup.DataSource = (DataTable)ViewState["dsLine"];
                                wdgLookup.DataBind();
                            }
                            else if (ViewState["text"].ToString().Equals("LineItem"))
                            {
                                wdgLookup.Rows.Clear();
                                wdgLookup.DataSource = (DataTable)ViewState["dsItem"];
                                wdgLookup.DataBind();
                                upLookUP.Update();
                            }
                            else if (ViewState["text"].ToString().Equals("ShipTo"))
                            {
                                wdgLookup.Rows.Clear();
                                wdgLookup.DataSource = (DataTable)ViewState["dsShipTo"];
                                wdgLookup.DataBind();
                                upLookUP.Update();
                            }
                            else if (ViewState["text"].ToString().Equals("SalesOrder"))
                            {
                                wdgLookup.Rows.Clear();
                                wdgLookup.DataSource = (DataTable)ViewState["dsOrders"];
                                wdgLookup.DataBind();
                                upLookUP.Update();
                            }
        }
        /// <summary>
        /// Handles the SelectedIndexChanged event of the drpfields1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void drpfields1_SelectedIndexChanged(object sender, EventArgs e)
        {
            txtValue1_1.Text = string.Empty;
            txtValue2_1.Text = string.Empty;
            string fieldvalue = drpfields1.SelectedValue;
            if (ViewState["text"].ToString().Equals("ShipTo"))
            {
                if (fieldvalue.Equals("City") || fieldvalue.Equals("StateID") || fieldvalue.Equals("CountryID") || fieldvalue.Equals("WhseID"))
                {
                    txtValue1_1.Attributes.Add("onkeydown", "return isAlpha(event.keyCode);");
                    txtValue2_1.Attributes.Add("onkeydown", "return isAlpha(event.keyCode);");
                }
                else if (fieldvalue.Equals("PostalCode"))
                {
                    txtValue1_1.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_1.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_1.Attributes.Add("onpaste", "return false");
                    txtValue2_1.Attributes.Add("onpaste", "return false");


                    txtValue1_1.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_1.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                }
                else
                {
                    txtValue1_1.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_1.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }
            }
            else if (ViewState["text"].ToString().Equals("SOLookUP"))
            {
                if (fieldvalue.Equals("TranAmt") || fieldvalue.Equals("OpenAmt"))
                {
                    txtValue1_1.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_1.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_1.Attributes.Add("onpaste", "return false");
                    txtValue2_1.Attributes.Add("onpaste", "return false");


                    txtValue1_1.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_1.Attributes.Add("onkeyup", "keyUP(event.keyCode)");

                }
                else if (fieldvalue.Equals("TranDate"))
                {
                    txtValue1_1.Attributes.Add("onkeydown", "return isDate(event.keyCode);");
                    txtValue2_1.Attributes.Add("onkeydown", "return isDate(event.keyCode);");

                }

                else
                {
                    txtValue1_1.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_1.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }
            }
            else if (ViewState["text"].ToString().Equals("Customer"))
            {
                
                if (fieldvalue.Equals("ContactPhone"))
                {
                    txtValue1_1.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_1.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_1.Attributes.Add("onpaste", "return false");
                    txtValue2_1.Attributes.Add("onpaste", "return false");


                    txtValue1_1.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_1.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                }
                else
                {
                    txtValue1_1.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_1.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }

            }
           
          
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the drpfields2 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void drpfields2_SelectedIndexChanged(object sender, EventArgs e)
        {

            txtValue1_2.Text = string.Empty;
            txtValue2_2.Text = string.Empty;

            string fieldvalue = drpfields2.SelectedValue;
            if (ViewState["text"].ToString().Equals("ShipTo"))
            {
                if (fieldvalue.Equals("City") || fieldvalue.Equals("StateID") || fieldvalue.Equals("CountryID") || fieldvalue.Equals("WhseID"))
                {
                    txtValue1_2.Attributes.Add("onkeydown", "return isAlpha(event.keyCode);");
                    txtValue2_2.Attributes.Add("onkeydown", "return isAlpha(event.keyCode);");
                }
                else if (fieldvalue.Equals("PostalCode"))
                {
                    txtValue1_2.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_2.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_2.Attributes.Add("onpaste", "return false");
                    txtValue2_2.Attributes.Add("onpaste", "return false");


                    txtValue1_2.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_2.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                }
                else
                {
                    txtValue1_2.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_2.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }
            }
            else
            if (ViewState["text"].ToString().Equals("SOLookUP"))
            {
                if (fieldvalue.Equals("TranAmt") || fieldvalue.Equals("OpenAmt"))
                {
                    txtValue1_2.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_2.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_2.Attributes.Add("onpaste", "return false");
                    txtValue2_2.Attributes.Add("onpaste", "return false");


                    txtValue1_2.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_2.Attributes.Add("onkeyup", "keyUP(event.keyCode)");

                } if (fieldvalue.Equals("TranDate"))
                {
                    txtValue1_1.Attributes.Add("onkeydown", "return isDate(event.keyCode);");
                    txtValue2_1.Attributes.Add("onkeydown", "return isDate(event.keyCode);");

                }
                else
                {
                    txtValue1_2.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_2.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }
            }
            else if (ViewState["text"].ToString().Equals("Customer"))
            {
                
                
                if (fieldvalue.Equals("ContactPhone"))
                {
                    txtValue1_2.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_2.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_2.Attributes.Add("onpaste", "return false");
                    txtValue2_2.Attributes.Add("onpaste", "return false");


                    txtValue1_2.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_2.Attributes.Add("onkeyup", "keyUP(event.keyCode)");

                }
                else
                {
                    txtValue1_2.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_2.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the drpfields3 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void drpfields3_SelectedIndexChanged(object sender, EventArgs e)
        {
            txtValue1_3.Text = string.Empty;
            txtValue2_3.Text = string.Empty; 

            string fieldvalue = drpfields3.SelectedValue;
            if (ViewState["text"].ToString().Equals("ShipTo"))
            {
                if (fieldvalue.Equals("City") || fieldvalue.Equals("StateID") || fieldvalue.Equals("CountryID") || fieldvalue.Equals("WhseID"))
                {
                    txtValue1_3.Attributes.Add("onkeydown", "return isAlpha(event.keyCode);");
                    txtValue2_3.Attributes.Add("onkeydown", "return isAlpha(event.keyCode);");
                }
                else if (fieldvalue.Equals("PostalCode"))
                {
                    txtValue1_3.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_3.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_3.Attributes.Add("onpaste", "return false");
                    txtValue2_3.Attributes.Add("onpaste", "return false");


                    txtValue1_3.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_3.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                }
                else
                {
                    txtValue1_3.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_3.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }
            }

            else if (ViewState["text"].ToString().Equals("SOLookUP"))
            {
                if (fieldvalue.Equals("TranAmt") || fieldvalue.Equals("OpenAmt"))
                {
                    txtValue1_3.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_3.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_3.Attributes.Add("onpaste", "return false");
                    txtValue2_3.Attributes.Add("onpaste", "return false");


                    txtValue1_3.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_3.Attributes.Add("onkeyup", "keyUP(event.keyCode)");

                } if (fieldvalue.Equals("TranDate"))
                {
                    txtValue1_1.Attributes.Add("onkeydown", "return isDate(event.keyCode);");
                    txtValue2_1.Attributes.Add("onkeydown", "return isDate(event.keyCode);");

                }
                else
                {
                    txtValue1_3.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_3.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }
            }
            else if (ViewState["text"].ToString().Equals("Customer"))
            {

                if (fieldvalue.Equals("ContactPhone"))
                {

                    txtValue1_3.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_3.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_3.Attributes.Add("onpaste", "return false");
                    txtValue2_3.Attributes.Add("onpaste", "return false");


                    txtValue1_3.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_3.Attributes.Add("onkeyup", "keyUP(event.keyCode)");

                }
                else
                {
                    txtValue1_3.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_3.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }
            }

        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the drpfields4 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void drpfields4_SelectedIndexChanged(object sender, EventArgs e)
        {
            txtValue1_4.Text = string.Empty;
            txtValue2_4.Text = string.Empty; 
           
            string fieldvalue = drpfields4.SelectedValue;
            if (ViewState["text"].ToString().Equals("ShipTo"))
            {
                if (fieldvalue.Equals("City") || fieldvalue.Equals("StateID") || fieldvalue.Equals("CountryID") || fieldvalue.Equals("WhseID"))
                {
                    txtValue1_4.Attributes.Add("onkeydown", "return isAlpha(event.keyCode);");
                    txtValue2_4.Attributes.Add("onkeydown", "return isAlpha(event.keyCode);");
                }
                else if (fieldvalue.Equals("PostalCode"))
                {
                    txtValue1_4.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_4.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_4.Attributes.Add("onpaste", "return false");
                    txtValue2_4.Attributes.Add("onpaste", "return false");


                    txtValue1_4.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_4.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                }
                else
                {
                    txtValue1_4.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_4.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }
            }
            else if (ViewState["text"].ToString().Equals("SOLookUP"))
            {
                if (fieldvalue.Equals("TranAmt") || fieldvalue.Equals("OpenAmt"))
                {
                    txtValue1_4.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_4.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_4.Attributes.Add("onpaste", "return false");
                    txtValue2_4.Attributes.Add("onpaste", "return false");


                    txtValue1_4.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_4.Attributes.Add("onkeyup", "keyUP(event.keyCode)");

                } if (fieldvalue.Equals("TranDate"))
                {
                    txtValue1_1.Attributes.Add("onkeydown", "return isDate(event.keyCode);");
                    txtValue2_1.Attributes.Add("onkeydown", "return isDate(event.keyCode);");

                }
                else
                {
                    txtValue1_4.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_4.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }
            }
            else if (ViewState["text"].ToString().Equals("Customer"))
            {

                if (fieldvalue.Equals("ContactPhone"))
                {
                    txtValue1_4.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");
                    txtValue2_4.Attributes.Add("onkeydown", "return isNumeric(event.keyCode);");

                    txtValue1_4.Attributes.Add("onpaste", "return false");
                    txtValue2_4.Attributes.Add("onpaste", "return false");


                    txtValue1_4.Attributes.Add("onkeyup", "keyUP(event.keyCode)");
                    txtValue2_4.Attributes.Add("onkeyup", "keyUP(event.keyCode)");

                }
                else
                {
                    txtValue1_4.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");
                    txtValue2_4.Attributes.Add("onkeydown", "return isAlphaNumeric(event.keyCode);");

                }
            }

        }
        /// <summary>
        /// Handles the SelectedIndexChanged event of the drpcondition1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void drpcondition1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (drpcondition1.SelectedItem.Value == "BETWEEN" || drpcondition1.SelectedItem.Value == "Not Between")
            {
                txtValue2_1.Visible = true;


            }
            else
            {
                txtValue2_1.Visible = false;
            }

        }
        /// <summary>
        /// Handles the SelectedIndexChanged event of the drpcondition2 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void drpcondition2_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (drpcondition2.SelectedItem.Value == "BETWEEN" || drpcondition2.SelectedItem.Value == "Not Between")
            {
                txtValue2_2.Visible = true;
            }
            else
            {
                txtValue2_2.Visible = false;
            }
        }
        /// <summary>
        /// Handles the SelectedIndexChanged event of the drpcondition3 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void drpcondition3_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (drpcondition3.SelectedItem.Value == "BETWEEN" || drpcondition3.SelectedItem.Value == "Not Between")
            {
                txtValue2_3.Visible = true;
            }
            else
            {
                txtValue2_3.Visible = false;
            }
        }
        /// <summary>
        /// Handles the SelectedIndexChanged event of the drpcondition4 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        protected void drpcondition4_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (drpcondition4.SelectedItem.Value == "BETWEEN" || drpcondition4.SelectedItem.Value == "Not Between")
            {
                txtValue2_4.Visible = true;
            }
            else
            {
                txtValue2_4.Visible = false;
            }
        }

        //GetLookupList(string prefixText, string iCompanyID, LookupType LT, string UserID)


    }   // end "class SalesOrderCreationQuote"
}       // end "namespace SalesOrderCreationQuote"

/*
 * .aspx UI object naming issue
 * btnCloseTable_Click      ->      btnCloseSOLookUp_Click
 * 
*/